<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Dotenv\Dotenv;
use Rc\Metrics\Metrics;

/**
 * Don't change order of _init* methods
 */
class Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{

    protected function _initEnvironment()
    {
        $env = new Dotenv(APPLICATION_PATH . '/../');
        $env->load();
    }

    protected function _initConfig()
    {
        $config = [];

        /** @var SplFileInfo $file */
        foreach (new FilesystemIterator(APPLICATION_PATH . '/../config', FilesystemIterator::SKIP_DOTS) as $file) {
            $config[$file->getBasename('.' . $file->getExtension())] = require $file->getPathname();
        }

        Zend_Registry::set('Config', $config);

        enforce_valid_env(app_env());
    }

    protected function _initUniqueVisitor()
    {
        if (empty($_COOKIE['uniquevisitor'])) {
            $_COOKIE['uniquevisitor'] = md5(uniqid('', true));
        }

        $_COOKIE['uniquevisitor'] = preg_replace('~[^\d\w]~', '', $_COOKIE['uniquevisitor']);
        setcookie('uniquevisitor', $_COOKIE['uniquevisitor'], time() + 86400 * 90, '/');
    }

    protected function _initMetrics()
    {
        if ($_SERVER['REMOTE_ADDR'] == '127.0.0.1') {
            return;
        }

        Metrics::set('users', Identity::getUserId() ?: $_SERVER['REMOTE_ADDR']);

        register_shutdown_function(function () {
            Metrics::timing('response_times', microtime(true) - $_SERVER['REQUEST_TIME_FLOAT']);
        });
    }

    protected function _initSecurityPlugin()
    {
        $front = Zend_Controller_Front::getInstance();
        $front->registerPlugin(new SecurityPlugin());
    }

    protected function _initAuthorizationPlugin()
    {
        $front = Zend_Controller_Front::getInstance();
        $front->registerPlugin(new AuthorizationPlugin());
    }

    protected function _initFileUploadLayoutPlugin()
    {
        $front = Zend_Controller_Front::getInstance();
        $front->registerPlugin(new FileUploadLayoutPlugin());
    }

    protected function _initDatabaseConnection()
    {
        $database = config('databases.mysql');

        MySqlRepository::setDefaultConnection([
            'host'     => $database['host'],
            'user'     => $database['username'],
            'password' => $database['password'],
            'database' => $database['database'],
        ]);

        Zend_Registry::set('Zend_Db_Adapter', new Zend_Db_Adapter_Pdo_Mysql([
            'host'     => $database['host'],
            'username' => $database['username'],
            'password' => $database['password'],
            'dbname'   => $database['database'],
        ]));

        if (Identity::exists()) {
            $user = 'user' . Identity::getUserId();
        } else {
            $user = 'user';
        }

        $userGroups = new UserGroups($database);
        $userGroups->login($user);
        $userGroups->addGroupsFromSession();
        Zend_Registry::set('RC_DILPS_AUTH', $userGroups);
    }

    protected function _initLanguage()
    {
        $path = APPLICATION_PATH . '/../resources/lang/';

        if (Identity::exists() && file_exists(sprintf($path . '/%s/main.php', Identity::getUser()->getSettings()->getLanguage()))) {
            $locale = Identity::getUser()->getSettings()->getLanguage();
        } else {
            $locale = config('app.lang');
        }

        $translate = new Zend_Translate([
            'adapter' => 'array',
            'content' => sprintf($path . '/%s/', $locale),
            'MyLocal' => $locale,
            'scan'    => Zend_Translate::LOCALE_DIRECTORY,
        ]);

        Zend_Registry::set('Zend_Locale', new MyLocal($locale));
        Zend_Registry::set('Zend_Translate', $translate);
    }

    protected function _initRedirectUrl()
    {
        preg_match('~^([^?]*)(\?.*)?$~i', $_SERVER['REQUEST_URI'], $hits);
        $_SERVER['REDIRECT_URL'] = $hits[1];
    }

    protected function _initAccountStatusRedirects()
    {
        if (!Identity::exists()) {
            return;
        }

        if (Identity::isUserSwitched() && strpos($_SERVER['REDIRECT_URL'], '/session/') === 0) {
            return;
        }

        if (Identity::getUser()->getDeleted() && !Identity::isUserSwitched()) {
            Identity::clear();
            $redirectTarget = '/';
        } else if (!Identity::getUser()->isApproved()) {
            $redirectTarget = '/portal/confirm-email';
        } else {
            return;
        }

        if ($_SERVER['REDIRECT_URL'] == $redirectTarget) {
            return;
        }

        $response = new Zend_Controller_Response_Http();
        $response->setRedirect($redirectTarget);

        Zend_Controller_Front::getInstance()->setResponse($response);
    }

}