<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Iterator\NoteFilterIterator;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;
use Rc\Domain\Repository\Research\WorkRepository;

class EditorController extends BasicControllerAction
{

    /**
     * @return bool|null|Exposition
     * @throws Exception
     */
    public function indexAction()
    {
        $exposition = $this->_getRequestedResearchAndCheckForInvitation();
        if (!isset($exposition)) {
            return null;
        }

        if ($exposition->isStatus(Exposition::STATUS_REVIEW) && $exposition->getPortal()->isAdmin(Identity::getUser())) {
            // portal admins can access researches in review
        } elseif (Identity::getUser()->isLimitedUser()) {
            return $this->_handleInsufficientRights('Your account only permits reviewing! If you would like to start uploading your own research, please <a href="/portal/upgrade">upgrade</a> for a full account!<br/>Check the <a href="/portal/faq#2.4">FAQ</a> for further information.');
        } elseif ($exposition->isStatus(Exposition::STATUS_REVIEW)) {
            return $this->_handleInsufficientRights('This research is in review and cannot be edited anymore!<br/>Please refer the <a href="/portal/faq#2.9">FAQ</a> for further information.');
        } elseif ($exposition->isPublished()) {
            return $this->_handleInsufficientRights('This research is published and cannot be edited anymore!<br/>Please refer the <a href="/portal/faq#2.9">FAQ</a> for further information.');
        } elseif (!$exposition->hasEditRights(Identity::getUser())) {
            $loginAs = null;
            if (Identity::isAdmin()) {
                $loginAs = '<a href="/admin/user/instant-switch?user=' . $exposition->getAuthor()->getId() . '" style="float:right; margin-right: 20px">LOGIN AS</a>';
            }
            return $this->_handleInsufficientRights('You do not have permissions to edit this research! Please check if you have entered a correct url, the research still exists or if you are still a collaborator. If you think it\'s a bug please contact the <a href="mailto:support@researchcatalogue.net">RC User Support</a>.' . $loginAs);
        }

        try {
            $weave = $this->getRequestedWeave();

            if ($weave instanceof Popover && !in_array($weave->getWeave(), $exposition->getWeaves()) || !$weave instanceof Popover && !in_array($weave, $exposition->getWeaves())) {
                throw new InsufficientRightsException('The request weave is not part of the exposition!');
            }
        } catch (InsufficientRightsException $exception) {
            $this->redirect($this->view->editorUrl($exposition));
            return false;
        }

        $this->view->additionalCss = ($exposition->isPublished() === false || $exposition->getPublishedDate(false)->getTimestamp() >= 1364462257 || in_array($exposition->getId(),
                [21732]));
        $this->view->research = $exposition;
        $this->view->weave = $weave;
        $this->view->isPopover = $weave instanceof Popover;
        $this->view->returnTo = $weave instanceof Popover ? $this->_loadRequestedObject('weave', $this->_getParam('return_to')) : null;
        $this->view->user = Identity::getUser();

        if ($weave instanceof TextWeave) {
            $this->render('nmh-editor');
        }
    }

    public function thumbAction()
    {
        $thumb = new WeaveThumb(new NoteFilterIterator(new ArrayIterator($this->getRequestedWeave()->getItems()), $this->_getParam('commentfilter')),
            [
                'width'     => $this->_getParam('width'),
                'height'    => $this->_getParam('height'),
                'maxwidth'  => $this->_getParam('maxwidth'),
                'maxheight' => $this->_getParam('maxheight'),
            ]);
        $this->view->thumb = $thumb;
        $this->render('weave/thumb', null, true);
    }

    public function contentAction()
    {
        $this->_getRequestedResearch(); // Important: rights to access the item will be granted in the research constructor

        $weave = $this->getRequestedWeave();
        if ($weave->getIframe()) {
            $this->view->items = [];
        } else {
            $this->view->items = $weave->getItems();
        }
    }

    public function worksAction()
    {
        $sharedWorks = [];
        if ($this->hasParam('work')) {
            try {
                $works[] = $this->_getRequestedWork();
            } catch (InsufficientRightsException $exception) {
                foreach ($this->_getRequestedResearch()->getShares() as $share) {
                    if ($share->getId() == $this->getParam('work')) {
                        $sharedWorks[] = $share;
                        break;
                    }
                }
            }
        } elseif ($this->hasParam('research')) {
            $works = array_merge(Identity::getUser()->getWorks(), Identity::getUser()->getCoAuthoredWorks());
            $sharedWorks = array_diff($this->_getRequestedResearch()->getShares(), $works);
        } else {
            $works = WorkRepository::findByPerson(Identity::getUser());
        }

        $this->view->works = $works;
        $this->view->sharedWorks = $sharedWorks;
    }

    public function workChildrenAction()
    {
        try {
            $this->view->work = $this->_getRequestedWork();
        } catch (InsufficientRightsException $exception) {
            foreach ($this->_getRequestedResearch()->getShares() as $share) {
                if ($share->getId() == $this->getParam('work')) {
                    $this->view->work = $share;
                    break;
                }
            }
        }
    }

    public function toolsAction()
    {
        $this->view->tools = ToolFactory::getTools();
        $this->view->research = $this->_getRequestedResearch();
    }

    public function popoversAction()
    {
        $this->_getRequestedResearch();

        $this->view->popovers = PopoverRepository::findByWeave($this->_getRequestedWeave());
    }

    public function weavesAction()
    {
        $this->view->weaves = $this->_getRequestedResearch()->getWeaves();
    }

    public function collaborationMessageAction()
    {
        $form = $this->_getForm($this->_getRequestedResearch());

        if ($form->isPostAndValid()) {
            foreach ($form->getValue('receiverid') as $receiverId) {
                $message = new \Rc\Domain\Model\Message();
                $message->setValues($form->getValues());
                $message->setReceiverId($receiverId);
                $message->setSenderId(Identity::getUserId());

                $this->_sendMessage($message);
            }
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @return Weave
     * @throws Exception
     */
    protected function getRequestedWeave()
    {
        try {
            return $this->_getRequestedWeave();
        } catch (Exception $exception) {
            try {
                return $this->_loadRequestedObject('popover', $this->_getParam('weave'));
            } catch (Exception $notUsed) {
                throw $exception;
            }
        }
    }
}
