<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;
use Rc\Domain\Model\User\Settings;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Metrics\Metrics;
use Rc\Privacy\AuditLogFactory;
use Rc\Service\Ingest\IngestService;
use Rc\Utility\TokenGenerator;

class PortalController extends BasicControllerAction
{

    public function init()
    {
        $this->initPortal();
    }

    public function indexAction()
    {
        $this->view->researchPager = $this->_getParam('rp', 0);
        $this->view->researches = ExpositionRepository::findPublicButNotPublished(['page' => $this->view->researchPager, 'limit' => 3]);

        $this->view->publishedResearchPager = $this->_getParam('prp', 0);
        $this->view->publishedResearches = ExpositionRepository::findPublished(['page' => $this->view->publishedResearchPager, 'limit' => 3]);

        $this->view->persons = UserRepository::findForLandingPage(['limit' => 9]);
    }

    public function searchAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (ResearchRepository::findByTitleOrKeyword($this->_getParam('value'), ['limit' => 10]) as $result) {
            foreach ($result->getKeywordList() as $keyword) {
                if (!preg_match('~' . preg_quote($this->_getParam('value')) . '.*~i', $keyword)) {
                    continue;
                }
                $retVal[strtolower($keyword)] = [
                    'value' => $result->getId(),
                    'label' => $keyword,
                ];
            };

            if (preg_match('~' . preg_quote($this->_getParam('value')) . '.*~i', $result->getTitle())) {
                $retVal[strtolower($result->getTitle())] = [
                    'value' => $result->getId(),
                    'label' => $result->getTitle()->__toString(),
                ];
            }
        }

        echo json_encode(array_values($retVal));
    }

    public function advancedSearchAction()
    {
        $form = $this->_getForm()->setDefaults($this->getAllParams());

        $this->view->form = $form;
    }

    public function searchResultAction()
    {
        if ($this->hasParam('resulttype')) {
            $form = PortalFormGenerator::getAdvancedSearch();
            $tab = $form->getTab($this->_getParam('resulttype')) ?: $form->getTab('exposition');
            $tab->isValid(); #init getValues()
            $params = $tab->getValues();

            if (!empty($params['statusprogress'])) {
                $params['status'][] = Exposition::STATUS_PROGRESS;
            }
            if (!empty($params['statuspublished'])) {
                $params['status'][] = Exposition::STATUS_PUBLISHED;
                $params['status'][] = Exposition::STATUS_REVIEW_PUBLISHED;
            }
        } else {
            $params['fulltext'] = $this->_getParam('fulltext');
        }
        $params['format'] = $this->_getParam('format');

        if (!empty($params['fulltext'])) {
            $params['fulltext'] = '"' . trim($params['fulltext'], '"') . '"';
        }

        $results = [];
        switch ($this->_getParam('resulttype')) {
            case 'research':
                $results['research'] = ExpositionRepository::search($params, ['page' => $this->_getParam('rp', 0), 'limit' => 30]);
                break;

            case 'user':
                $results['user'] = UserRepository::search($params, ['page' => $this->_getParam('up', 0), 'limit' => 30]);
                break;

            case 'work':
                $results['work'] = WorkRepository::search($params, ['page' => $this->_getParam('wp', 0), 'limit' => 30]);
                break;

            case 'project':
                $results['project'] = ProjectRepository::search($params, ['page' => $this->_getParam('pp', 0), 'limit' => 30]);
                break;

            case 'degree':
                $results['degree'] = DegreeRepository::search($params, ['page' => $this->_getParam('dp', 0), 'limit' => 30]);
                break;

            default:
                $results = [
                    'research' => ExpositionRepository::search($params, ['page' => $this->_getParam('rp', 0), 'limit' => 6]),
                    'user'     => UserRepository::search($params, ['page' => $this->_getParam('up', 0), 'limit' => 6]),
                    'work'     => WorkRepository::search($params, ['page' => $this->_getParam('wp', 0), 'limit' => 6]),
                    'project'  => ProjectRepository::search($params, ['page' => $this->_getParam('pp', 0), 'limit' => 6]),
                    'degree'   => DegreeRepository::search($params, ['page' => $this->_getParam('dp', 0), 'limit' => 6]),
                ];
                break;
        }

        $this->view->showReports = $this->_getParam('reports');
        $this->view->results = $results;
        $this->view->fulltext = $params['fulltext'];

        if ($params['format'] === 'json') {
            $this->getResponse()->setHeader('Content-Type', 'application/json');

            $this->_disableLayout();
            $this->render('search-result-json');
        }
    }

    public function registerAction()
    {
        $form = $this->_getForm();

        if (Identity::exists()) {
            $this->redirect('/settings');
        }

        if ($form->isPostAndValid()) {
            $user = $this->createBasicAccount($form);

            if (isset($user)) {
                AuditLogFactory::createAndPersist('create-account', $user);
                Metrics::increment('create-account');
            }

            $this->view->flashMessage = 'Your registration was successful. You will soon receive an email asking you to confirm your email address.';
        } else {
            $this->view->form = $form;
        }
    }

    public function confirmEmailAction()
    {
        $email = $this->getParam('email');
        $token = $this->getParam('token');

        if (Identity::exists() && Identity::getUser()->isApproved()) {
            $this->redirect('/');
            return;
        }

        if (Identity::exists() && $this->getRequest()->isPost()) {
            // send new confirmation mail

            $this->sendConfirmationMail(Identity::getUser());

            AuditLogFactory::createAndPersist('send-confirmation-mail', Identity::getUser());
            Metrics::increment('confirm-email', 1, ['resolution' => 'pending']);

            $this->_disableView();
        } else if (!empty($email) || !empty($token)) {
            // check confirmation of email

            if (TokenGenerator::generate($email) != $token) {
                $this->view->flashMessage = 'The given combination of email address and token is invalid.';
                return;
            }

            $user = UserRepository::findByEmail($email);

            if (isset($user)) {
                $user->setApproved(true);
                UserRepository::persist($user);

                AuditLogFactory::createAndPersist('confirm-email', $user);
                Metrics::increment('confirm-email', 1, ['resolution' => 'succeeded']);

                $this->view->flashMessage = 'Your email is approved! You can login now.';
            } else {
                Metrics::increment('confirm-email', 1, ['resolution' => 'failed']);

                $this->view->flashMessage = 'The email couldn\'t be approved, the token is not valid!';
            }
        } else if (!Identity::exists()) {
            $this->redirect('/');
            return;
        }
    }

    public function recentPublicationsAction()
    {
        $this->view->publishedResearches = ExpositionRepository::findPublished();
    }

    public function upgradeAction()
    {
        $form = $this->_getForm()->setDefaultsFrom(Identity::exists() ? Identity::getUser() : null);

        if ($form->isPostAndValid()) {
            $this->view->form = $form;
            $this->render('letter-agreement');
        } else {
            $this->view->form = $form;
        }
    }

    public function forgottenPasswordAction()
    {
        $form = $this->_getForm();

        if ($form->isPostAndValid()) {
            $user = UserRepository::findByEmail($form->getValue('email'));
            $this->view->lastKeyUsed = !(bool)$user->getPasswordForgottenKey();

            $user->setPasswordForgottenKey(md5(time() . $user->getId() . $user->getPassword()));
            UserRepository::persist($user);

            $mail = new Mail();
            $mail->setFrom(config('app.email'));
            $mail->addTo($form->getValue('email'));
            $mail->setSubject($this->view->translate('message.subject.forgotten.password'));
            $mail->setBodyText(
                sprintf(
                    $this->view->translate('message.content.forgotten.password.text'),
                    $user->getName(),
                    $user->getPasswordForgottenKey()
                ),
                'UTF-8'
            );
            $mail->setBodyHtml(
                sprintf(
                    $this->view->translate('message.content.forgotten.password.html'),
                    $user->getName(),
                    $user->getPasswordForgottenKey()
                ),
                'UTF-8'
            );
            $mail->send();
        } else {
            $this->view->form = $form;
        }
    }

    public function changePasswordAction()
    {
        if (!$this->_getParam('key')) {
            return false;
        }

        $user = UserRepository::findByPasswordForgottenKey($this->_getParam('key'));
        if (!$user) {
            return false;
        }

        $form = $this->_getForm();
        if ($form->isPostAndValid()) {
            $user->setValues($form->getValues());
            $user->setPasswordForgottenKey(null);
            UserRepository::persist($user);

            $this->redirect('/portal');
        } else {
            $this->view->form = $form;
        }
    }

    public function aboutAction()
    {
    }

    public function faqAction()
    {
    }

    public function membersAction()
    {
        $content = file_get_contents('http://www.societyforartisticresearch.org/membership/institutional-members/');
        preg_match('~<table id="logos">.*</table>~isU', $content, $matches);
        $logoTable = str_replace('/fileadmin/', 'http://www.societyforartisticresearch.org/fileadmin/', $matches[0]);

        $this->view->assign('logoTable', $logoTable);
    }

    public function portalAction()
    {
    }

    public function termsAction()
    {
    }

    public function privacyAction()
    {
    }

    public function contactAction()
    {
    }

    public function copyrightAction()
    {
    }

    public function copyrightConcernsAction()
    {
    }

    public function technicalInformationAction()
    {
    }

    public function feedAction()
    {
        $this->view->portalList = PortalRepository::findAll();
    }

    public function maintenanceAction()
    {
        $maintenance = config('app.maintenance');
        if ($maintenance) {
            $this->view->maintenance = $maintenance;
            $this->_disableLayout();
        } else {
            $this->redirect('/portal');
        }
    }

    public function announcementsAction()
    {
        $this->view->announcements = AnnouncementRepository::findAllApproved(0, 1e4);
    }

    public function announcementAction()
    {
        $announcement = $this->_getRequestedAnnouncement();
        $this->view->announcement = $announcement;
        $this->_disableLayout();
    }

    public function unsubscribeAction()
    {
        $email = $this->_getParam('email');
        if (!$email) {
            $this->redirect('/portal');
            return;
        }

        $user = UserRepository::findByEmail($email);
        if (!$user) {
            $this->redirect('/portal');
            return;
        }

        if (TokenGenerator::generate($user->getEmail()) != $this->_getParam('token')) {
            $this->redirect('/portal');
            return;
        }

        $form = $this->_getForm()->setDefaultsFrom($user->getSettings());

        if ($form->isPostAndValid()) {
            $user->getSettings()->setValues($form->getValues());
            UserRepository::persist($user);

            $this->view->form = $form;
        } else {
            $this->view->form = $form;
        }
    }

    public function loginAction()
    {
    }

    public function updatesAction()
    {
        $this->view->updates = UpdateRepository::findAll(0, 1e3);
    }

    /**
     * @param DefaultForm $form
     * @return User|null
     */
    protected function createBasicAccount(DefaultForm $form)
    {
        if (UserRepository::findByEmail($form->getValue('email'))) {
            return null;
        }

        $user = new User();
        $user->setValues($form->getValues());
        $user->addPortalId(PortalRepository::getDefault()->getId()); // always add user to default portal
        $user->setLimitedUser(true);
        $user->setName($form->getValue('firstname') . ' ' . $form->getValue('lastname'));
        $user->setMedia(IngestService::masterFromFile(APPLICATION_PATH . '/../public/rc/images/default-user.png'));

        $settings = new Settings();
        $settings->setValues($form->getValues());

        $user->setSettings($settings);

        UserRepository::persist($user);

        if ($settings->getAnnouncements() || $settings->getDigest() || $settings->getMessageNotification()) {
            AuditLogFactory::createAndPersist('change-settings', $user, [
                'announcements'        => $settings->getAnnouncements(),
                'digest'               => $settings->getDigest(),
                'message_notification' => $settings->getMessageNotification(),
            ]);
        }

        $this->sendConfirmationMail($user);

        return $user;
    }

    /**
     * @param User $user
     * @throws Zend_Exception
     * @throws Zend_Mail_Exception
     */
    protected function sendConfirmationMail(User $user)
    {
        if (empty($user->getEmail())) {
            return;
        }

        $mail = new Mail();
        $mail->setFrom(config('app.email'));
        $mail->addTo($user->getEmail());
        $mail->setSubject($this->view->translate('message.subject.account.confirm'));
        $mail->setBodyHtml(
            sprintf(
                $this->view->translate('message.content.account.confirm'),
                $user->getName(),
                config('app.url') . '/portal/confirm-email?token=' . TokenGenerator::generate($user->getEmail()) . '&email=' . $user->getEmail()
            ),
            'UTF-8'
        );
        $mail->send();
    }
}

?>