<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Message;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Utility\ResearchUtility;

class ProfileEntityController extends BasicControllerAction
{

    public function mediaAction()
    {
        try {
            $entity = $this->_getRequestedEntity();
        } catch (\Exception $e) {
        }

        if (empty($entity)) {
            return '';
        }

        $this->view->thumb = $this->view->profile()->getThumb($entity);
    }

    public function showAction()
    {
    }

    public function addAction()
    {
    }

    public function editAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        $isOwner = $entity->isOwner(Identity::getUser());
        $class = ucfirst($entity->getName()) . 'FormGenerator';
        /** @var DefaultForm $form */
        $form = $class::getEdit($entity, $isOwner)->setDefaultsFrom($entity);

        if ($entity instanceof Work) {
            $form->setDefaults([
                'rcauthors'       => !$entity->hasFreeTextAuthor() && $entity->getAuthor() ? [$entity->getAuthor()->getId()] : null,
                'freetextauthors' => $entity->getFreeTextAuthor(),
            ]);
        }

        if (!$entity->isMetaEditableBy(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            $oldCoAuthors = $entity->getCoAuthors();
            $oldSupervisors = $entity->getSupervisors();

            $entity->setValues($form->getValues());

            if ($entity instanceof Degree) {
                $entity->setStatus(Degree::STATUS_PROGRESS);
                $entity->setPortalId(null);
            } elseif ($entity instanceof Project) {
                $ids = $form->getTab('meta')->getValue('principalinvestigators');
                if (!empty($ids) && is_array($ids)) {
                    $entity->setPrincipalInvestigator(UserRepository::findById(reset($ids)));
                } else {
                    $entity->setPrincipalInvestigator($entity->getAuthor());
                }
            } elseif ($entity instanceof Work) {
                $ids = $form->getTab('meta')->getValue('rcauthors');
                if (!$entity->hasFreeTextAuthor() && !empty($ids)) {
                    $entity->setAuthor(UserRepository::findById(reset($ids)));
                } else {
                    $entity->setAuthor(null);
                }

                if ($isOwner) {
                    $this->_handleCollaboration($entity, $entity->getCoAuthors(), $oldCoAuthors, 'coauthor');
                }
            }

            if ($isOwner) {
                $this->_handleCollaboration($entity, $entity->getSupervisors(), $oldSupervisors, 'supervisor');
            }

            ResearchRepository::persist($entity);
            EntityChangeRepository::addChange($entity, EntityChange::TYPE_EDIT);
            KeywordRepository::addKeywords($entity->getKeywords());

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->entity = $entity;
        }
    }

    public function removeAction()
    {
        /** @var Research $research */
        $research = $this->_getRequestedEntity();
        if (!$research->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $usedAsTemplate = PortalRepository::isResearchUsedAsTemplate($research);

        $form = $this->_getForm($research);
        if ($form->isPostAndValid() && !$usedAsTemplate) {
            ResearchUtility::removeResearch($research);

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->view->usedAsTemplate = $usedAsTemplate;
        }
    }

    public function shareAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($entity)->setDefaultsFrom($entity);
        if ($form->isPostAndValid()) {
            $entity->setValues($form->getValues());

            ResearchRepository::persist($entity);
            EntityChangeRepository::addChange($entity, EntityChange::TYPE_SHARE);

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
        }
    }

    public function versioningAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();

        $form = $this->_getForm($entity)->setDefaultsFrom($entity);
        if ($form->isPostAndValid()) {
            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
        }
    }

    public function collaborateAction()
    {
        // @todo: fix permsisions
        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($this->_getParam('entity'), 'research');

        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if ($entity instanceof Application) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($entity->getExpositionIds(), 'research');
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($entity->getWorkIds(), 'research');
        }

        $user = Identity::getUser();
        switch ($this->getParam('decision')) {
            case 'acceptDisapprove':
                if ($this->getParam('type') === 'coauthor') {
                    $entity->removeCoAuthor($user);
                } elseif ($this->getParam('type') == 'contributor') {
                    $entity->removeContributor($user);
                } elseif ($this->getParam('type') == 'supervisor') {
                    $entity->removeSupervisor($user);
                }

                $entity->removeDisapprovedCollaborator($user->getId(), $this->getParam('type'));

                ResearchRepository::persist($entity);

                $this->view->message = 'info.collaboration.acceptedDisapprove';

                return;

            case 'declineDisapprove':
                if (!$entity->isDisapprovedCollaborator($user->getId(), $this->getParam('type'))) {
                    $this->view->message = 'error.no.collaboration.offer';
                    return;
                }

                $entity->removeDisapprovedCollaborator($user->getId(), $this->getParam('type'));
                ResearchRepository::persist($entity);

                $this->view->message = 'info.collaboration.declinedDisapprove';

                return;

            case 'leave':
                $form = ExpositionFormGenerator::getLeaveCollaboration($entity);

                if ($form->isPostAndValid()) {
                    if ($this->getParam('type') === 'collaborateur') {
                        $entity->removeCoAuthor($user);
                        $entity->removeContributor($user);
                    } elseif ($this->getParam('type') == 'supervisor') {
                        $entity->removeSupervisor($user);
                    } elseif ($this->getParam('type') == 'editor') {
                        $entity->removeEditor($user);
                    }

                    ResearchRepository::persist($entity);

                    if ($this->getParam('type') == 'collaborateur') {
                        $this->_sendCollaborationMessage($entity, [$user], null, $this->getParam('decision'));
                    }

                    $this->_disableView();
                } else {
                    $this->view->form = $form;
                }

                return;
        }

        if (!$entity->isCollaborateur($user) && !$entity->isInvitedAsCollaborator($user)) {
            $this->view->message = 'error.no.collaboration.offer';
            return;
        }

        switch ($this->getParam('decision')) {
            case 'accept':
                if (!$entity->isContributor($user)) {
                    $this->_sendCollaborationMessage($entity, [$user], null, $this->getParam('decision'));
                }

                $entity->removePendingInvites([$user->getId()], $this->getParam('type'));
                ResearchRepository::persist($entity);

                $this->view->message = 'info.collaboration.accepted';

                $user->followEntity($entity);
                break;

            case 'decline':
                if ($this->getParam('type') === 'coauthor') {
                    $entity->removeCoAuthor($user);
                } elseif ($this->getParam('type') == 'contributor') {
                    $entity->removeContributor($user);
                } elseif ($this->getParam('type') == 'supervisor') {
                    $entity->removeSupervisor($user);
                }

                $entity->removePendingInvites([$user->getId()], $this->getParam('type'));
                ResearchRepository::persist($entity);

                $this->_sendCollaborationMessage($entity, [$user], null, $this->getParam('decision'));

                $this->view->message = 'info.collaboration.declined';

                break;

            default:
                break;
        }
    }

    public function changeLayoutAction()
    {
        $entity = $this->_getRequestedEntity();
        $groupElement = ProfileGroupElementRepository::findByUserIdAndEntityId(Identity::getUserId(), $entity->getId());
        if ($groupElement->getLayout() === null) {
            $groupElement->setLayout($entity instanceof Exposition ? ProfileGroupElement::LAYOUT_2x2 : ProfileGroupElement::LAYOUT_1x1);
        }
        $form = $this->_getForm()->setDefaultsFrom($groupElement);

        if ($form->isPostAndValid()) {
            $groupElement->setValues($form->getValues());
            ProfileGroupElementRepository::update($groupElement);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function connectToPortalAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($entity)->setDefaultsFrom($entity);
        if ($form->isPostAndValid()) {
            $oldPartOfPortalRequestId = $entity->getPartOfPortalRequestId();
            $entity->setValues($form->getValues());

            if ($entity->getPartOfPortalRequestId() && $oldPartOfPortalRequestId != $entity->getPartOfPortalRequestId()) {
                $entity->setPartOfPortalId('');

                // create notification for portal admins
                foreach (PortalRepository::findById($entity->getPartOfPortalRequestId())->getAdminIds() as $adminId) {
                    $message = new Message();
                    $message->setSenderId(UserRepository::findSystemAccount()->getId());
                    $message->setReceiverId($adminId);
                    $message->setType(Message::TYPE_PARTOF_PORTAL_REQUEST);
                    $message->setContent([
                        'user'   => Identity::getUserId(),
                        'entity' => $entity->getId(),
                        'type'   => $entity->getName(),
                        'portal' => $entity->getPartOfPortalRequestId(),
                    ]);
                    $this->_sendMessage($message);
                }
            } elseif (!$entity->getPartOfPortalRequestId()) {
                $entity->setPartOfPortalId('');
            }
            ResearchRepository::persist($entity);

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
        }
    }

    public function updateThumbAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $file = SimpleMediaRepository::findById($this->getParam('file'));
        if (isset($file) && $file->getResearch()->getId() == $entity->getId()) {
            $entity->setThumbFileId($file->getId());
        } else {
            $entity->setThumbFileId(null);
        }
        ResearchRepository::persist($entity);

        $this->_setSuccessHeader();
        $this->_disableView();
    }

    public function fileUploadAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $form = FileFormGenerator::getAdd();
        if ($this->getRequest()->isPost()) {
            $usedForm = $this->_getSubmittedTab($form);
        } else {
            $usedForm = $form;
        }

        if ($usedForm->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($usedForm->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($entity);
            $simpleMedia->setValues($usedForm->getValues());
            SimpleMediaRepository::persist($simpleMedia);

            $entity->addFile($simpleMedia);
            ResearchRepository::persist($entity);

            $this->_setFormActionHeader('/file/edit?file=' . $simpleMedia->getId());
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function fileListAction()
    {
        /** @var Research $entity */
        $entity = $this->_getRequestedEntity();
        if (!$entity->isMetaEditableBy(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $this->view->entity = $entity;
    }

    public function updateFilePositionAction()
    {
        $entity = $this->_getRequestedEntity();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $entity->setFilePosition($this->getParam('file'), $this->getParam('position'));
        ResearchRepository::persist($entity);

        $this->_setSuccessHeader();
        $this->_disableView();
    }

    public function fileRemoveAction()
    {
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($simpleMedia);
        $entity = $simpleMedia->getResearch();
        if (!$entity->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        if ($form->isPostAndValid()) {
            if ($entity->getThumbFileId() == $simpleMedia->getId()) {
                $entity->setThumbFileId(null);
                ResearchRepository::persist($entity);
            }
            ResearchRepository::remove($simpleMedia);

            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function suggestRelationsAction()
    {
        $this->_disableView();

        $entities = [];
        foreach (ProfileGroupElementRepository::findByUserId(Identity::getUserId(), 0, 500) as $groupElement) {
            if ($groupElement->getEntityId() == $this->getParam('excluded_id')) {
                continue;
            }
            $entities[] = $groupElement->getEntityId();
        }

        $retVal = [];
        foreach (ResearchRepository::findByName($this->getParam('name'), ['ids' => $entities, 'limit' => 50]) as $entity) {
            $retVal[$entity->getName() . '_' . $entity->getTitle() . $entity->getId()] = [
                'value' => $entity->getId(),
                'label' => $entity->getTitle() . ' (' . $this->view->translate('entity.' . $entity->getName()) . ')',
            ];
        }

        ksort($retVal);

        echo json_encode(array_values($retVal));
    }

    protected function buildSuggestList(array $entityList, $type)
    {
        $retVal = [];
        foreach ($entityList as $entity) {
            $retVal[$type . '_' . $entity->getTitle() . $entity->getId()] = [
                'value' => $entity->getId(),
                'label' => $entity->getTitle() . ' (' . $type . ')',
            ];
        }
        return $retVal;
    }
}