<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Message;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Export\ExpositionHtmlExport;
use Rc\Service\CrossRefService;
use Rc\Utility\ResearchUtility;

class ResearchController extends BasicControllerAction
{
    public function init()
    {
        $this->initPortal();
    }

    public function testAction()
    {
        $this->_disableView();

        $exposition = ExpositionRepository::findById(293185);
        echo '<br/><br/><br/><br/><br/>';
        echo count($exposition->getWeaves());
    }

    /**
     * @throws InsufficientRightsException
     * @throws Exception
     */
    public function addAction()
    {
        $exposition = new Exposition();
        $form = $this->_getForm();

        if ($form->isPostAndValid() && !Identity::getUser()->isLimitedUser()) {
            if ($form->getTab('meta')->getValue('editortype') == Exposition::EDITOR_TYPE_TEXT_BASED) {
                $duplicateFacade = new \Rc\Utility\DuplicateFacade();

                $templateId = $form->getTab('meta')->getValue('template');
                Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($templateId);

                $exposition = $duplicateFacade->duplicateExposition(ExpositionRepository::getById($templateId), null, ['collaboration' => 'private']);
                $exposition->setOwner(Identity::getUser());
                $exposition->setValues($form->getValues());

                ExpositionRepository::persist($exposition);

                $weave = $exposition->getDefaultWeave();
            } else {
                $weave = new GraphicalWeave();

                $exposition->setOwner(Identity::getUser());
                $exposition->setValues($form->getValues());
            }

            $weave->setOwner(Identity::getUser());
            $weave->setTitle(new \Rc\Domain\Model\Translation(Weave::DEFAULT_TITLE));
            $weave->setExposition($exposition);

            ExpositionRepository::persist($exposition);
            WeaveRepository::persist($exposition);

            KeywordRepository::addKeywords($exposition->getKeywords());

            Identity::getUser()->followEntity($exposition);

            $this->_setFormActionHeader('/research/edit?research=' . $exposition->getId() . '&redirect=true');
            $this->_setSuccessHeader();
            $this->view->redirectUrl = $this->view->editorUrl($exposition);
        } else {
            $this->view->form = $form;
            $this->view->research = $exposition;
        }
    }

    public function exportAction()
    {
        try {
            if (!Identity::exists()) {
                throw new InsufficientRightsException();
            }

            if ($this->hasParam('application') && $this->_getRequestedApplication()->containsBoundExpositionId($this->_getParam('exposition'))) {
                Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($this->_getParam('exposition'));
            }

            $research = $this->_getRequestedResearch();

            $file = ExpositionHtmlExport::export($research);
            $filename = str_replace('/', '-', utf8_decode($research->getTitle())) . '.zip';

            header('X-Accel-Redirect: ' . sendfile($file));
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . $filename . '"');

            $this->_disableLayout();
            $this->_disableView();
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function editAction()
    {
        $exposition = $this->_getRequestedResearch();
        $isOwner = $exposition->isOwner(Identity::getUser());
        if ((!$isOwner && !$exposition->isCoAuthor(Identity::getUser())) || !$exposition->isStatus(Exposition::STATUS_PROGRESS)) {
            if (!$exposition->isStatus(Exposition::STATUS_REVIEW) || !$exposition->getPortal()->isAdmin(Identity::getUser())) {
                throw new InsufficientRightsException();
            }
        }
        $form = $this->_getForm($exposition, $isOwner)
            ->setDefaultsFrom($exposition, '!chapterlist')
            ->setDefaults($exposition->getChapterList(), 'chapterlist');

        $form->getTab('chapterlist')->getElement('displaymenubarpermanently')->setValue($exposition->getDisplayMenubarPermanently());
        $form->getTab('chapterlist')->getElement('tocalignment')->setValue($exposition->getTocAlignment());

        if ($form->isPostAndValid()) {
            $oldCoAuthors = $exposition->getCoAuthors();
            $oldContributors = $exposition->getContributors();
            $oldSupervisors = $exposition->getSupervisors();

            $exposition->setValues($form->getValues());

            if ($isOwner) {
                $this->_handleCollaboration($exposition, $exposition->getCoAuthors(), $oldCoAuthors, 'coauthor');
                $this->_handleCollaboration($exposition, $exposition->getContributors(), $oldContributors, 'contributor');
                $this->_handleCollaboration($exposition, $exposition->getSupervisors(), $oldSupervisors, 'supervisor');
            }

            $this->updateResearchLastModified($exposition);

            if ($isOwner && $exposition->getAuthor()->getId() != $form->getValues()['authorid']) {
                $this->_handleAuthorChange($exposition, UserRepository::getById($form->getValues()['authorid']));
            }

            if (Identity::isSwitchedUserAdmin() && $isOwner && $exposition->getOwner()->getId() != $form->getValues()['ownerid']) {
                $this->_handleOwnerChange($exposition, UserRepository::getById($form->getValues()['ownerid']));
            }

            ExpositionRepository::persist($exposition);

            KeywordRepository::addKeywords($exposition->getKeywords());

            EntityChangeRepository::addChange($exposition, EntityChange::TYPE_EDIT);

            $this->_setSuccessHeader();

            if ($this->_getParam('redirect') == true) {
                $this->view->redirectUrl = $this->view->editorUrl($exposition);
            } else {
                $this->_disableView();
            }
        } else {
            $this->view->form = $form;
        }
    }

    public function removeAction()
    {
        $research = $this->_getRequestedResearch();
        if (!$research->isOwner(Identity::getUser()) || !$research->isStatus(Exposition::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }

        $usedAsTemplate = PortalRepository::isResearchUsedAsTemplate($research);

        $form = $this->_getForm($research);
        if ($form->isPostAndValid() && !$usedAsTemplate) {
            ResearchUtility::removeExposition($research);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->usedAsTemplate = $usedAsTemplate;
        }
    }

    public function publishAction()
    {
        if (!config('app.allow.publishing')) {
            throw new InsufficientRightsException();
        }

        $research = $this->_getRequestedResearch();
        $form = $this->_getForm($research);
        if (!$research->isOwner(Identity::getUser()) || !$research->isStatus(Exposition::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }
        if ($form->isPostAndValid()) {
            $portal = PortalRepository::findById($form->getValue('portalid'));
            if ($portal->isDefault()) {
                $this->view->form = $form;
                return;
            }

            // create notification for portal admins
            foreach ($portal->getAdminIds() as $adminId) {
                $message = new Message();
                $message->setSenderId(UserRepository::findSystemAccount()->getId());
                $message->setReceiverId($adminId);
                $message->setType(Message::TYPE_SUBMIT_TO_PORTAL);
                $message->setContent([
                    'user'     => Identity::getUserId(),
                    'research' => $research->getId(),
                    'portal'   => $form->getValue('portalid'),
                ]);

                $this->_sendMessage($message);
            }

            $research->setPortalId($form->getValue('portalid'));
            $research->setPublishRange($form->getValue('publishrange'));
            $research->setStatus(Exposition::STATUS_REVIEW);
            ExpositionRepository::persist($research);

            $this->view->portal = $portal;
        } else {
            $this->view->form = $form;
        }
    }

    public function selfPublishAction()
    {
        if (!config('app.allow.publishing')) {
            throw new InsufficientRightsException();
        }

        $research = $this->_getRequestedResearch();
        $form = $this->_getForm($research);
        if (!$research->isOwner(Identity::getUser()) || !$research->isStatus(Exposition::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }
        if ($form->isPostAndValid()) {
            $research->setStatus(Exposition::STATUS_PUBLISHED);
            $research->setPortalId($form->getValue('portalid'));
            $research->setPublishRange($form->getValue('publishrange'));
            ExpositionRepository::persist($research);

            CrossRefService::deposit($research, true);
            ExpositionRepository::persist($research);

            $this->view->portal = PortalRepository::getDefault();
        } else {
            $this->view->form = $form;
        }
    }

    public function republishAction()
    {
        $research = $this->_getRequestedResearch();
        $form = $this->_getForm($research);
        if (!$research->isOwner(Identity::getUser()) || !$research->isStatus(Exposition::STATUS_PUBLISHED)) {
            throw new InsufficientRightsException();
        }
        if ($form->isPostAndValid()) {
            // create notification for portal admins
            foreach (PortalRepository::findById($form->getValue('portalid'))->getAdminIds() as $adminId) {
                $message = new Message();
                $message->setSenderId(Identity::getUserId());
                $message->setReceiverId($adminId);
                $message->setType(Message::TYPE_SUBMIT_TO_PORTAL);
                $message->setContent([
                    'user'     => Identity::getUserId(),
                    'research' => $research->getId(),
                    'portal'   => $form->getValue('portalid'),
                ]);
                $message->setDeletedForSender(true);

                $this->_sendMessage($message);
            }

            $research->setAdditionalPendingPortalId($form->getValue('portalid'));
            $research->setStatus(Exposition::STATUS_REVIEW_PUBLISHED);
            ExpositionRepository::persist($research);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function shareAction()
    {
        $research = $this->_getRequestedResearch();
        if (!$research->isOwner(Identity::getUser())) {
            throw new InsufficientRightsException();
        }
        $form = $this->_getForm($research)->setDefaultsFrom($research);

        if ($form->isPostAndValid()) {
            $research->setValues($form->getValues());

            EntityChangeRepository::addChange($research, EntityChange::TYPE_SHARE);

            ExpositionRepository::persist($research);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function orderAction()
    {
        $this->_disableView();

        $order = array_flip($this->_getParam('order'));
        foreach (Identity::getUser()->getResearches() as $oldOrder => $research) {
            if ($order['research-' . $research->getId()] !== $oldOrder) {
                $research->setOrder($order['research-' . $research->getId()]);
                ExpositionRepository::persist($research);
            }
        }
    }

    public function searchAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (ExpositionRepository::suggest($this->_getParam('property'), $this->_getParam('value'), ['limit' => 10]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch(),
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }
}

?>