<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\Media\Resource\PdfResource;
use Rc\Domain\Model\Media\Resource\VideoResource;
use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\TableOfContent;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Security\Text;
use Rc\Service\Ingest\IngestService;
use Rc\Service\Ingest\MediaTypes;
use Symfony\Component\Process\Process;

class TextEditorController extends BasicControllerAction
{

    public function simpleMediaAddAction()
    {
        $exposition = $this->_getRequestedResearch();
        $form = SimpleMediaFormGenerator::getAdd();

        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form) ?: $form;
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($formToUse->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($exposition);
            $simpleMedia->setValues($formToUse->getValues());

            SimpleMediaRepository::persist($simpleMedia);

            $retVal = $this->dumpSimpleMedia($simpleMedia);
        } else {
            $retVal = ['success' => false, 'errors' => $formToUse->getMessages()];
        }

        echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        exit;
    }

    public function simpleMediaEditAction()
    {
        $exposition = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedSimpleMedia();
        $form = SimpleMediaFormGenerator::getEdit($simpleMedia)->getTab($simpleMedia->getMediaType());

        if (!$simpleMedia->isOwner(Identity::getUser())) {
            $retVal = ['success' => false, 'errors' => 'No rights to edit simple media!'];
            echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
            exit;
        }

        if ($form->isPostAndValid(array_merge($form->getValues(), $this->_request->getParams()))) {
            $simpleMedia->setValues($form->getValues());
            SimpleMediaRepository::persist($simpleMedia);

            $retVal = $this->dumpSimpleMedia($simpleMedia);
        } else {
            $retVal = ['success' => false, 'errors' => $form->getMessages()];
        }

        echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        exit;
    }

    public function simpleMediaRemoveAction()
    {
        $exposition = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedSimpleMedia();

        if (!$simpleMedia->isOwner(Identity::getUser())) {
            $retVal = ['success' => false, 'errors' => 'No rights to delete simple media!'];
            echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
            exit;
        }

        if ($this->_request->isPost()) {
            SimpleMediaRepository::remove($simpleMedia);
            $retVal = ['success' => true];
        } else {
            $retVal = ['success' => false, 'errors' => 'Only method post is allowed!'];
        }

        echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        exit;
    }

    public function simpleMediaListAction()
    {
        $exposition = $this->_getRequestedResearch();

        $retVal = [];
        foreach (SimpleMediaRepository::findByResearch($exposition) as $simpleMedia) {
            $retVal[] = $this->dumpSimpleMedia($simpleMedia);
        }

        echo json_encode($retVal, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        exit;
    }

    public function simpleMediaResourceAction()
    {
        $this->handleTemporaryPermissions($this->_getParam('research'));

        $exposition = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedSimpleMedia();

        if ($simpleMedia->getMedia()->isPdf()) {
            $resource = $simpleMedia->getMedia()->getMaster();
        } else {
            $resource = $simpleMedia->getMedia()->getDefaultDerivative();
        }

        if (!isset($resource)) {
            http_response_code(404);
            exit;
        }

        header('X-Accel-Redirect: ' . sendfile($resource->getUri()));
        header('Content-Type: ' . MediaTypes::getMediaTypeFromFilename($resource->getFileName()));
        exit;
    }

    public function simpleMediaThumbAction()
    {
        $this->handleTemporaryPermissions($this->_getParam('research'));

        $exposition = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedSimpleMedia();
        $width = $this->getParam('width');
        $height = $this->getParam('height');

        if ($simpleMedia->getMedia()->isImage()) {
            $resource = $simpleMedia->getMedia()->getBestFittingSplashScreen($width, $height);
        } elseif ($simpleMedia->hasThumb()) {
            $resource = $simpleMedia->getThumb()->getBestFittingSplashScreen($width, $height);
        } elseif ($simpleMedia->hasMedia()) {
            $resource = $simpleMedia->getMedia()->getBestFittingSplashScreen($width, $height);
        } else {
            http_response_code(404);
            exit;
        }

        header('X-Accel-Redirect: ' . sendfile($resource->getUri()));
        header('Content-Type: ' . MediaTypes::getMediaTypeFromFilename($resource->getFileName()));
        exit;
    }

    public function saveAction()
    {
        $exposition = $this->_getRequestedResearch();
        /** @var TextWeave $weave */
        $weave = $this->_getRequestedWeave();

        if (!$this->hasParams(['markdown', 'html', 'style', 'media', 'metadata'])) {
            echo json_encode(['success' => false, 'errors' => 'Save failed: some required fields are missing!']);
            exit;
        }

        $weave->setValues([
            'html'     => $this->getSaveParam('html'),
            'markdown' => $this->getSaveParam('markdown'),
            'media'    => $this->getSaveParam('media'),
            'style'    => $this->getSaveParam('style'),
            'metadata' => $this->getSaveParam('metadata'),
        ]);
        WeaveRepository::persist($weave);

        $exposition->setTitle($exposition->getTitle()->set('en', $this->getParam('title')));
        $exposition->setTableOfContent($this->createTableOfContent(json_decode($this->getSaveParam('toc', '[]'), true), $weave));
        ExpositionRepository::persist($exposition);

        exit;
    }

    public function loadAction()
    {
        $exposition = $this->_getRequestedResearch();
        /** @var TextWeave $weave */
        $weave = $this->_getRequestedWeave();

        echo json_encode([
            'html'     => $weave->getHtml(),
            'markdown' => $weave->getMarkdown(),
            'media'    => $weave->getMedia(),
            'metadata' => $weave->getMetadata(),
            'style'    => $weave->getStyle(),
            'title'    => (string)$exposition->getTitle(),
        ]);
        exit;
    }

    public function importAction()
    {
        $exposition = $this->_getRequestedResearch();

        $adapter = new Zend_File_Transfer_Adapter_Http();
        if (!$adapter->receive('file')) {
            echo json_encode(['success' => false, 'errors' => 'File upload failed due to errors: ' . implode(PHP_EOL, $adapter->getMessages())]);
            exit;
        }

        $file = $adapter->getFileName('file');
        $directory = dirname($file);
        $tmpMediaFolder = uniqid('files', true);

        $process = new Process('pandoc "' . $file . '" --to=markdown-pipe_tables-grid_tables-multiline_tables-simple_tables --extract-media=' . $tmpMediaFolder, $directory);
        $process->run();

        if (!$process->isSuccessful()) {
            echo json_encode(['success' => false, 'errors' => 'Import failed due to an error: ' . $process->getErrorOutput()]);
            exit;
        }

        $media = [];
        $mediaDirectory = $directory . '/' . $tmpMediaFolder . '/media';

        if (is_dir($mediaDirectory)) {
            $errors = [];

            /** @var SplFileInfo $file */
            foreach (new FilesystemIterator($mediaDirectory) as $file) {
                if ($file->isDir() || !in_array($file->getExtension(), ['wmf'])) {
                    continue;
                }

                rename($file->getPathname(), substr($file->getPathname(), 0, -3) . 'emf');
            }

            /** @var SplFileInfo $file */
            foreach (new FilesystemIterator($mediaDirectory) as $file) {
                if ($file->isDir()) {
                    continue;
                }

                if ($file->getExtension() == 'emf') {
                    $pngFile = $file->getPath() . '/' . $file->getBasename($file->getExtension()) . 'png';

                    $convertEmfProcess = new Process('inkscape ' . $file->getPathname() . ' --export-png=' . $pngFile . ' --export-area-drawing', $file->getPath());
                    $convertEmfProcess->run();

                    $file = new SplFileInfo($pngFile);
                }

                try {
                    $simpleMedia = SimpleMediaFactory::create(MediaTypes::getContentTypeFromFilename($file->getFilename()));
                    $simpleMedia->setOwner(Identity::getUser());
                    $simpleMedia->setResearch($exposition);
                    $simpleMedia->setName($file->getBasename('.' . $file->getExtension()));
                    $simpleMedia->setMedia(IngestService::masterFromFile($file->getPathname()));

                    SimpleMediaRepository::persist($simpleMedia);

                    $media[] = $simpleMedia->getId();
                } catch (\Exception $exception) {
                    $errors[] = 'Failed to import "' . $file->getFilename() . '" as Simple Media due to an error: ' . $exception->getMessage();
                }
            }
        }

        $retVal = ['success' => true, 'markdown' => $process->getOutput(), 'media' => $media];

        if (!empty($errors)) {
            $retVal['errors'] = $errors;
        }

        echo json_encode($retVal);
        exit;
    }

    public function exportAction()
    {
        if (!Identity::exists()) {
            throw new InsufficientRightsException();
        }

        $type = $this->getParam('type');
        $markdown = $this->useExternalLinkForMediaResources($this->getParam('markdown'));

        $file = uniqid(time(), true) . '.' . $type;
        $directory = temp_path();

        if (!in_array($type, ['html', 'md', 'pdf', 'docx', 'odt', 'epub'])) {
            echo json_encode(['success' => false, 'errors' => 'Export type "' . $type . '" is not supported!']);
            exit;
        }

        $process = new Process('pandoc --self-contained --from=markdown+footnotes -o ' . $file, $directory);
        $process->setInput($markdown);
        $process->run();

        if (!$process->isSuccessful()) {
            echo json_encode(['success' => false, 'errors' => 'Export failed due to an error: ' . $process->getErrorOutput()]);
            exit;
        }

        header('Content-Type: application/octet-stream');
        header('Content-Transfer-Encoding: Binary');
        header('Content-disposition: attachment; filename="converted.' . $type . '"');
        echo file_get_contents($directory . '/' . $file);
        unlink($directory . '/' . $file);
        exit;
    }

    protected function dumpSimpleMedia(SimpleMediaInterface $simpleMedia)
    {
        $media = null;

        if ($simpleMedia->hasMedia()) {
            $mediaContainer = $simpleMedia->getMediaContainer();
            $master = $simpleMedia->getMedia()->getMaster();

            $sizeMedia = new My_View_Helper_SizeMedia();
            $sizeMedia->setView($this->view);

            if ($sizeMedia->canBeRendered($mediaContainer)) {
                $status = 'Transcoded';
            } else {
                $status = $sizeMedia->getErrorMessage($mediaContainer, true);
            }

            $media = [
                'type'   => $master->getContentType(),
                'status' => $status,
            ];

            if ($master instanceof AudioResource) {
                $media += [
                    'duration' => $master->getDuration(),
                ];
            } elseif ($master instanceof VideoResource) {
                $media += [
                    'duration' => $master->getDuration(),
                    'width'    => $master->getWidth(),
                    'height'   => $master->getHeight(),
                ];
            } elseif ($master instanceof PdfResource) {
                $media += [
                    'width'  => $master->getWidth(),
                    'height' => $master->getHeight(),
                    'pages'  => $master->getPages(),
                ];
            } elseif ($master instanceof ImageResource) {
                $media += [
                    'width'  => $master->getWidth(),
                    'height' => $master->getHeight(),
                ];
            }
        }

        return [
            'id'          => $simpleMedia->getId(),
            'name'        => $simpleMedia->getName(),
            'copyright'   => $simpleMedia->getCopyright(),
            'description' => $simpleMedia->getDescription(),
            'media'       => $media,
        ];
    }

    /**
     * @param array $params
     * @return bool
     */
    protected function hasParams(array $params)
    {
        foreach ($params as $param) {
            if (!$this->hasParam($param)) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param string $markdown
     * @return string
     * @throws Exception
     */
    protected function useExternalLinkForMediaResources($markdown)
    {
        $pattern = preg_quote(config('app.url') . '/text-editor/simple-media-resource?research=%s&simple-media=%s');
        preg_match_all('~' . sprintf($pattern, '[\d]+', '([\d]+)') . '~', $markdown, $matches);

        foreach ($matches[0] as $key => $match) {
            $simpleMedia = SimpleMediaRepository::findById($matches[1][$key]);

            $resource = $simpleMedia->getMedia()->getDefaultDerivative();
            if (!isset($resource)) {
                continue;
            }

            $markdown = str_replace($match, $resource->getExternalUri(), $markdown);
        }

        return $markdown;
    }

    /**
     * @param array $tableOfContent
     * @param Weave $weave
     * @return TableOfContent
     */
    protected function createTableOfContent(array $tableOfContent, Weave $weave)
    {
        foreach ($tableOfContent as $key => $entry) {
            $tableOfContent[$key]['weaveid'] = $weave->getId();
        }

        return new TableOfContent($tableOfContent);
    }

    /**
     * @param string $name
     * @param mixed $default
     * @return mixed
     */
    protected function getSaveParam($name, $default = null)
    {
        return Text::removeJavascriptTags($this->getParam($name, $default));
    }

    protected function handleTemporaryPermissions($expositionId)
    {
        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($expositionId);
        }
    }
}
