<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\WorkRepository;

class WorkController extends BasicControllerAction
{

    /**
     * @return void
     */
    public function addAction()
    {
        $work = new Work();
        $form = $this->_getForm($work);

        if ($form->isPostAndValid()) {
            $work->setValues($form->getValues())
                ->setOwner(Identity::getUser());

            WorkRepository::persist($work);
            KeywordRepository::addKeywords($work->getKeywords());

            Identity::getUser()->followEntity($work);

            $this->_setSuccessHeader();
            $this->view->work = $work;
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @return void
     */
    public function addProfileAction()
    {
        $work = new Work();
        $form = $this->_getForm($work);

        if ($form->isPostAndValid()) {
            $work->setValues($form->getValues())
                ->setOwner(Identity::getUser());

            WorkRepository::persist($work);
            KeywordRepository::addKeywords($work->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->render('profile-entity/add', null, true);
        }
    }

    /**
     * @return void
     */
    public function showAction()
    {
        $this->view->assign('work', $this->_getRequestedWork());
    }

    /**
     * @return void
     */
    public function editAction()
    {
        $work = $this->_getRequestedWork();
        $isOwner = $work->isOwner(Identity::getUser());
        $form = $this->_getForm($work, $isOwner)
            ->setDefaultsFrom($work)
            ->setDefaults([
                'rcauthors'       => !$work->hasFreeTextAuthor() && $work->getAuthor() ? [$work->getAuthor()->getId()] : null,
                'freetextauthors' => $work->getFreeTextAuthor(),
            ]);

        if ($form->isPostAndValid()) {
            $oldCoAuthors = $work->getCoAuthors();

            $work->setValues($form->getValues());

            if ($isOwner) {
                $this->_handleCollaboration($work, $work->getCoAuthors(), $oldCoAuthors, 'coauthor');
            }

            WorkRepository::persist($work);
            EntityChangeRepository::addChange($work, EntityChange::TYPE_EDIT);
            KeywordRepository::addKeywords($work->getKeywords());

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
            $this->view->work = $work;
        }
    }

    /**
     * @return void
     */
    public function addFileAction()
    {
        $work = $this->_getRequestedWork();
        $form = $this->_getForm($this->_getRequestedResearch());

        if ($form->isPostAndValid()) {
            foreach ($this->_getRequestedFiles() as $simpleMedia) {
                $simpleMedia->setResearch($work);
                SimpleMediaRepository::persist($simpleMedia);

                $work->addResource($simpleMedia);
                WorkRepository::persist($work);
            }

            EntityChangeRepository::addChange($work, EntityChange::TYPE_EDIT);
            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function removeFileAction()
    {
        $simpleMedia = $this->_getRequestedFile();
        $work = $simpleMedia->getResearch();
        $form = $this->_getForm($simpleMedia);

        if ($form->isPostAndValid()) {
            $simpleMedia->setResearch($this->_getRequestedResearch());
            SimpleMediaRepository::persist($simpleMedia);

            $work->removeResource($simpleMedia);
            WorkRepository::persist($work);

            EntityChangeRepository::addChange($work, EntityChange::TYPE_EDIT);

            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function uploadFileAction()
    {
        $work = $this->_getRequestedWork();
        $research = $this->_getRequestedResearch();
        $form = FileFormGenerator::getAdd();

        if ($this->getRequest()->isPost()) {
            $usedForm = $this->_getSubmittedTab($form);
        } else {
            $usedForm = $form;
        }

        if ($usedForm->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($usedForm->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($work);
            $simpleMedia->setValues($usedForm->getValues());

            SimpleMediaRepository::persist($simpleMedia);
            EntityChangeRepository::addChange($work, EntityChange::TYPE_EDIT);

            $work->addFile($simpleMedia);
            WorkRepository::persist($simpleMedia);

            $this->_setFormActionHeader('file/edit?file=' . $simpleMedia->getId());
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @return void
     */
    public function removeAction()
    {
        $files = [];
        $usedOnWeaves = [];
        $works = $this->_getRequestedWorks();
        $form = $this->_getForm();

        foreach ($works as $work) {
            if (!$work->isOwner(Identity::getUser())) {
                $this->view->insufficientRights = true;
                return;
            }

            $files[$work->getId()] = [];
            $usedOnWeaves[$work->getId()] = [];
            foreach ($work->getMediaList() as $simpleMedia) {
                $weaves = WeaveRepository::findBySimpleMedia($simpleMedia);
                if (count($weaves) > 0) {
                    $files[$work->getId()][$simpleMedia->getId()] = $simpleMedia;
                    $usedOnWeaves[$work->getId()][$simpleMedia->getId()] = $weaves;
                }
            }

            if (empty($usedOnWeaves[$work->getId()])) {
                unset($usedOnWeaves[$work->getId()]);
            }
        }

        if ($form->isPostAndValid() && empty($usedOnWeaves)) {
            WorkRepository::removeCollection($works);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->works = $works;
            $this->view->files = $files;
            $this->view->usedOnWeaves = $usedOnWeaves;
        }
    }

    public function listAction()
    {
        if ($this->hasParam('work')) {
            $this->view->works = $this->_getRequestedWorks();
        } else {
            $this->view->works = Identity::getUser()->getWorks();
        }
        $this->view->showFiles = $this->_getParam('files', false);
    }

    public function shareAction()
    {
        $work = $this->_getRequestedWork();
        if (!$work->isOwner(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($work)->setDefaultsFrom($work);
        if ($form->isPostAndValid()) {
            $work->setValues($form->getValues());
            WorkRepository::persist($work);

            $this->_setSuccessHeader();
            $this->forward('show-own', 'profile');
        } else {
            $this->view->form = $form;
        }
    }

    public function updateThumbFileAction()
    {
        $work = $this->_getRequestedWork();
        if (!$work->isOwner(Identity::getUser()) || Identity::getUser()->isLimitedUser()) {
            throw new InsufficientRightsException();
        }

        $file = SimpleMediaRepository::findById($this->_getParam('file'));
        if (isset($file) && $file->getResearch()->getId() == $work->getId()) {
            $work->setThumbFileId($file->getId());
        } else {
            $work->setThumbFileId(null);
        }
        WorkRepository::persist($work);

        $this->_setSuccessHeader();
        $this->_disableView();
    }

    public function searchAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (WorkRepository::suggest($this->_getParam('property'), $this->_getParam('value'), ['limit' => 10]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch(),
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }

}

?>