<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Research\Application;

class ApplicationFormGenerator extends BasicFormGenerator
{

    /**
     * @param ApplicationProgram $applicationProgram
     * @param Application $application
     * @param bool $save
     * @return ApplicationDefaultForm
     */
    public static function getAdd(
        ApplicationProgram $applicationProgram = null,
        Application $application = null,
        $save = false
    )
    {
        $application = $application ?: new Application();

        $form = new ApplicationDefaultForm();
        $form->setTitle('dialog.application.add');

        if (isset($applicationProgram)) {
            $formTab = $form->addTab('application_form');
            $formTab->addElement('html', 'info', [
                'value' => '<br/><b>' . $applicationProgram->getTitle() . '</b>',
            ]);
            $formTab->addElement('text', 'title', [
                'label'    => 'field.title',
                'required' => true,
            ]);

            foreach ($applicationProgram->getApplicationForm()->getElements() as $element) {
                $formOptions = $element->getFormOptions();
                if (!empty($formOptions['required'])) {
                    $formOptions['label'] .= ' *';
                }
                if (!$save) {
                    $formOptions['required'] = false;
                }
                $formTab->addElement($element->getFormType(), 'custom_elements_' . $element->getIdentifier(),
                    $formOptions);
            }

            $fileTab = $form->addTab('application_file');
            foreach ($applicationProgram->getRequiredDocuments(true) as $key => $label) {
                $fileTab->addElement('file', 'custom_files_' . $label, [
                    'label'      => str_replace('_', ' ', str_replace('custom_file_', '', $label)) . ' *',
                    'required'   => $save,
                    'validators' => [
                        ['MimeType', false, ['application/pdf', 'application/x-pdf']],
                    ],
                ]);
            }
            if ($applicationProgram->getAllowAnyDocuments()) {
                $fileTab->addElement('html', 'spacer', [
                    'value' => '<hr/>
                        <i class="ui-icon ui-icon-info"></i> Here you can upload any number of files of all common types like documents, media files or archives.<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                        If we have missed a file type please contact <a href="mailto:support@researchcatalogue.net">support@researchcatalogue.net</a>.',
                ]);
                $count = count($application->getOptionalDocuments()) + 10;
                for ($i = 1; $i <= $count ; $i++) {
                    $fileTab->addElement('file', 'custom_optional-files_' . $i, [
                        'label'      => $i. '. optional document',
                    ]);
                }
            }
            if (empty($fileTab->getElements())) {
                $form->disableTab('application_file');
            }

            if ($applicationProgram->getRequiredExpositions() || $applicationProgram->getRequiredWorks()) {
                $researchTab = $form->addTab('application_research');
                if ($applicationProgram->getRequiredExpositions()) {
                    $researchTab->addElement('multiselect', 'expositionids', [
                        'label'                     => 'expositions *',
                        'required'                  => $save,
                        'multiOptions'              => static::_getLazyExpositionList(static::_getParam('application_research.expositionids',
                            $application->getExpositionIds())),
                        'data-autocomplete'         => true,
                        'data-autocomplete-options' => json_encode([
                            'url'         => '/application/search-exposition',
                            'placeholder' => 'Add exposition...',
                            'maxcount'    => $applicationProgram->getRequiredExpositions(),
                        ]),
                    ]);
                }
                if ($applicationProgram->getRequiredWorks()) {
                    $researchTab->addElement('multiselect', 'workids', [
                        'label'                     => 'works *',
                        'required'                  => $save,
                        'multiOptions'              => static::_getLazyWorkList(static::_getParam('application_research.workids',
                            $application->getWorkIds())),
                        'data-autocomplete'         => true,
                        'data-autocomplete-options' => json_encode([
                            'url'         => '/application/search-work',
                            'placeholder' => 'Add work...',
                            'maxcount'    => $applicationProgram->getRequiredWorks(),
                        ]),
                    ]);
                }
            }

            $contacts = [];
            foreach ($applicationProgram->getContactPersons() as $contact) {
                $contacts[] = '
                <a href="/profile/?person=' . $contact->getId() . '" target="_blank">' . $contact->getName() . '</a>
                <a href="javascript:addMessageDialog(' . $contact->getId() . ')"><i class="ui-icon ui-icon-mail-closed"></i></a>';
            }
            $contacts = '<p><b>' . self::_getTranslate()->translate('field.contact-persons') . ': </b>' . implode('; ',
                    $contacts) . '</p>';

            if (!empty($applicationProgram->getDescription())) {
                $description = '<p>
                    <b>' . self::_getTranslate()->translate('field.description') . ': </b><br/>
                    <span style="white-space: pre-wrap;">' . $applicationProgram->getDescription() . '</span>
                </p>';
            } else {
                $description = '';
            }

            if (!empty($applicationProgram->getFurtherInformation())) {
                $furtherInformation = '<p><b>' . self::_getTranslate()->translate('field.further-information') . ': </b><a href="' . $applicationProgram->getFurtherInformation() . '" target="_blank">Open</a></p>';
            } else {
                $furtherInformation = '';
            }

            if (!empty($applicationProgram->getGuidelines())) {
                $guidelines = '<p><b>' . self::_getTranslate()->translate('field.extended-guidelines') . ': </b><a href="' . $applicationProgram->getGuidelines()->getExternalUri() . '" target="_blank">Open</a></p>';
            } else {
                $guidelines = '';
            }

            $infoTab = $form->addTab('application_info');
            $infoTab->addElement('html', 'info-description', [
                'value' => $description . $furtherInformation . $guidelines . $contacts,
            ]);

            $collaborationTab = self::_addCollaborationTab($form, $application);
            $collaborationTab->getElement('supervisors')->setLabel('field.adviser');
            $collaborationTab->removeElement('contributors');
            if (!$application->isOwner(Identity::getUser())) {
                $collaborationTab->removeElement('coauthors');
            }

            $form->addTab('application_status');

            $form->addCancelButton()->addSubmitButton('save');
        } else {
            $form->addElement('select', 'application', [
                'multiOptions' => ['' => ''] + static::_getApplicationProgramList(),
                'onchange'     => 'addApplicationDialog($(this).val()); $(this).val("")',
            ]);
            $form->addCancelButton()->addSubmitButton();
        }

        return $form;
    }

    /**
     * @param Application $application
     * @param bool $save
     * @return ApplicationDefaultForm
     */
    public static function getEdit(Application $application, $save = false)
    {
        $form = self::getAdd($application->getProgram(), $application, $save);
        $form->setTitle('dialog.application.edit');

        return $form;
    }

    public static function getPublish()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.application.publish');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.application.publish'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getReview(Application $application, $save = false)
    {
        $reviewForm = $application->getProgram()->getReviewForm();

        $form = new ApplicationDefaultForm();
        $form->setTitle('dialog.review.add');

        $formTab = $form->addTab('review_form');
        foreach ($reviewForm->getElements() as $element) {
            $formOptions = $element->getFormOptions();
            if (!$save) {
                $formOptions['required'] = false;
            }
            $formTab->addElement($element->getFormType(), 'custom_elements_' . $element->getIdentifier(), $formOptions);
        }

        $form->addCancelButton()->addSubmitButton()->addSubmitButton('save');

        return $form;
    }

    /**
     * @return DefaultForm
     */
    public static function getSubmitReview()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.review.submit');
        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.review.confirmation'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }
}
