<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Repository\PortalRepository;

class ExpositionFormGenerator extends BasicFormGenerator
{

    public static function getAdd(Exposition $research = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.add');

        if (!isset($research)) {
            $form->getView()->jQuery()->addOnLoad('initExpositionEditorType(\'#frm-meta-editortype\');');
        }

        $form->addTab('meta')
            ->addElement('languageBar', 'language-bar')
            ->addElement('select', 'editortype', [
                'label'        => 'field.editor-type',
                'multiOptions' => Exposition::getEditorTypes(),
            ])
            ->addElement('select', 'template', [
                'label'        => 'field.template',
                'multiOptions' => self::getTextExpositionTemplates(),
            ])
            ->addElement('file', 'preview', [
                'label' => 'field.mediathumb',
            ])
            ->addMultiLanguageElement('text', 'title', [
                'required'   => true,
                'label'      => 'field.title',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Exposition::VAL_TITLE_LENGTH_MIN,
                            Research::VAL_TITLE_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('datePicker', 'date', [
                'required' => true,
                'value'    => date(static::_getLocale()->getDefaultDateFormatForPhp()),
                'label'    => 'field.date',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'required'   => true,
                'label'      => 'field.abstract',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Research::VAL_DESCRIPTION_LENGTH_MIN,
                            Research::VAL_DESCRIPTION_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'affiliation', [
                'label' => 'field.affiliation',
            ])
            ->addElement('text', 'referenceurl', [
                'label' => 'field.reference-url',
            ])
            ->addElement('text', 'keywords', [
                'label'       => 'field.keywords',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ]);

        if (isset($research)) {
            $multiOptionsForWeave = [];
            foreach ($research->getWeaves() as $weave) {
                $multiOptionsForWeave[$weave->getId()] = $weave->getTitle();
            }

            $form->getTab('meta')->addElement('select', 'defaultweaveid', [
                'label'        => 'field.default.weave',
                'multiOptions' => $multiOptionsForWeave,
            ]);

            $tab = $form->addTab('chapterlist');
            $tab->addElement('checkbox', 'displaymenubarpermanently', [
                'label'       => 'field.display-menubar-permanently',
                'description' => 'description.display-menubar-permanently',
            ])
                ->addElement('select', 'tocalignment', [
                    'label'        => 'field.toc-alignment',
                    'multiOptions' => [
                        'vertical'   => 'field.vertical',
                        'horizontal' => 'field.horizontal',
                    ],
                ]);

            $chapterList = $research->getChapterList();
            $weaves = $research->getWeaves();

            $count = max(max(count($weaves), count($chapterList)) + 3, 10);
            for ($i = 0; $i < $count; $i++) {
                $accordion = $tab->addAccordion('chapter-' . ($i + 1), [], [
                    'sortable' => true,
                    'title'    => sprintf(static::_getTranslate()->translate('accordion.chapterlist.entry'), $i + 1),
                ]);
                $accordion->setElementsBelongTo('chapterlist[' . $i . ']')
                    ->addElement('select', 'weaveid', [
                        'label'        => 'field.weave',
                        'multiOptions' => ['' => ''] + $multiOptionsForWeave,
                    ])
                    ->addElement('text', 'title', [
                        'label' => 'field.title',
                    ])
                    ->addElement('number', 'coordx', [
                        'label' => 'field.coord.x',
                    ])
                    ->addElement('number', 'coordy', [
                        'label' => 'field.coord.y',
                    ]);
            }

            self::_addCollaborationTab($form, $research);
            self::_addRelationTab($form, $research);
        }

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getImport(Exposition $research = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.import');

        $form->addElement('file', 'archive', [
            'label'      => 'field.research.archive',
            'required'   => true,
            'validators' => [
                'IsCompressed',
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getEdit(Exposition $research, $isOwner)
    {
        $form = static::getAdd($research);
        $form->setTitle('dialog.research.edit', $research);

        $form->getTab('meta')->removeElement('editortype');
        $form->getTab('meta')->removeElement('template');

        if ($research->getDefaultWeave() instanceof TextWeave) {
            $form->disableTab('chapterlist');
        }

        if (!$isOwner) {
            $form->clearAndDisableTab([
                'collaboration',
            ]);
        }

        return $form;
    }

    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.research.remove');

        return $form;
    }

    public static function getLeaveCollaboration($research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.collaboration.leave', $research);

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.collaboration.leave'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getPublish(Exposition $research)
    {
        $defaultPortalId = key(static::_getDefaultPortal());

        $form = new DefaultForm();
        $form->setTitle('dialog.research.publish', $research);

        if ($research->getDefaultWeave() instanceof TextWeave && !$research->getDefaultWeave()->getMarkdown()) {
            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.no.content.on.weave'),
            ]);
            $form->addCancelButton();
        } else if ($research->getDefaultWeave() instanceof GraphicalWeave && !$research->hasItems()) {
            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.no.tools.on.weave'),
            ]);
            $form->addCancelButton();
        } else {
            $hasExternalContent = $research->hasExternalContent();

            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.publication'),
            ]);
            if ($hasExternalContent) {
                $form->addElement('html', 'info-external-content', [
                    'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.externalContent'),
                ]);
            }
            if (Identity::getUser()->isMemberOfNonDefaultPortal()) {
                $portals = [];
                $portalsAllowingPublishing = static::_getPortalsWhichAllowsLimitedPublishing('exposition', true, $hasExternalContent);
                foreach (Identity::getUser()->getPortalIds() as $portalId) {
                    if (isset($portalsAllowingPublishing[$portalId])) {
                        $portals[$portalId] = $portalsAllowingPublishing[$portalId];
                    }
                }

                if (!empty($portals)) {
                    $form
                        ->addElement('radio', 'submitto-limited', [
                            'value'        => '',
                            'multioptions' => ['submittolimited' => 'field.submit.to.limited'],
                            'onclick'      => 'togglePublishState(this);$("#' . $form->getElementsPrefix() . 'portalid").val($("#frm-portal-limited").val());$("#' . $form->getElementsPrefix() . 'publishrange").val("limited")',
                        ])
                        ->addElement('select', 'portal-limited', [
                            'label'        => 'field.portal',
                            'disabled'     => 'disabled',
                            'multioptions' => ['' => ''] + $portals,
                            'onchange'     => '$("#' . $form->getElementsPrefix() . 'portalid").val($(this).val())',
                        ]);
                }
            }
            $form->addElement('radio', 'submitto-unlimited', [
                'value'        => '',
                'multioptions' => ['submittounlimited' => 'field.submit.to.unlimited'],
                'onclick'      => 'togglePublishState(this);$("#' . $form->getElementsPrefix() . 'portalid").val($("#frm-portal-unlimited").val());$("#' . $form->getElementsPrefix() . 'publishrange").val("unlimited")',
            ])
                ->addElement('select', 'portal-unlimited', [
                    'label'        => 'field.portal',
                    'disabled'     => 'disabled',
                    'multioptions' => ['' => ''] + static::_getPortalsWhichAllowsPublishing('exposition', true, $hasExternalContent),
                    'onchange'     => '$("#' . $form->getElementsPrefix() . 'portalid").val($(this).val())',
                ])
                ->addElement('hidden', 'publishrange', [
                    'required' => true,
                    'value'    => 'unlimited',
                ])
                ->addElement('hidden', 'portalid', [
                    'required' => true,
                    'value'    => $defaultPortalId,
                ]);

            $form->addCancelButton()->addSubmitButton();
        }

        return $form;
    }

    public static function getSelfPublish(Exposition $research)
    {
        $defaultPortalId = key(static::_getDefaultPortal());

        $form = new DefaultForm();
        $form->setTitle('dialog.research.self-publish', $research);

        if ($research->getDefaultWeave() instanceof TextWeave && !$research->getDefaultWeave()->getMarkdown()) {
            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.no.content.on.weave'),
            ]);
            $form->addCancelButton();
        } else if ($research->getDefaultWeave() instanceof GraphicalWeave && !$research->hasItems()) {
            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.no.tools.on.weave'),
            ]);
            $form->addCancelButton();
        } else {
            $form->addElement('html', 'info', [
                'value' => '<br/><span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.publication'),
            ])
                ->addElement('hidden', 'publishrange', [
                    'required' => true,
                    'value'    => 'unlimited',
                ])
                ->addElement('hidden', 'portalid', [
                    'required' => true,
                    'value'    => $defaultPortalId,
                ]);

            $form->addCancelButton()->addSubmitButton();
        }

        return $form;
    }

    public static function getRepublish(Exposition $research)
    {
        $portalList = [];
        foreach (static::_getPortalList() as $portalId => $portalName) {
            if (!in_array($portalId, array_merge((array)$research->getPortalId(), $research->getAdditionalPortalIds()))) {
                $portalList[$portalId] = $portalName;
            }
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.research.publish', $research);

        $form->addElement('html', 'info', [
            'value' => '<br/><span style="float: left; margin-right: .3em;" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.publication'),
        ])
            ->addElement('select', 'portalid', [
                'required'     => true,
                'label'        => 'field.portal',
                'multioptions' => ['' => ''] + $portalList,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @return array
     */
    protected static function getTextExpositionTemplates()
    {
        /** @var Portal[] $portals */
        $portals = array_unique(array_merge([PortalRepository::getDefault()], Identity::getUser()->getPortals()));

        $templates = [];
        foreach ($portals as $portal) {
            foreach ($portal->getTextExpositionTemplates() as $exposition) {
                $templates[$exposition->getId()] = $exposition->getTitle();
            }
        }
        return $templates;
    }
}