<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Application\ApplicationFavorite;
use Rc\Domain\Model\Application\ApplicationForm;
use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Message;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Snapshot;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Application\ApplicationFavoriteRepository;
use Rc\Domain\Repository\Application\ApplicationFormRepository;
use Rc\Domain\Repository\Application\ApplicationProgramRepository;
use Rc\Domain\Repository\MessageRepository;
use Rc\Domain\Repository\Portal\IssueRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\Research\SnapshotRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Metrics\Metrics;
use Rc\Utility\Strings;

/**
 * @property ViewInterface $view
 * @method ApplicationProgram|null _findRequestedApplicationProgram()
 * @method Application _getRequestedApplication()
 * @method ApplicationProgram _getRequestedApplicationProgram()
 * @method ApplicationFavorite _getRequestedApplicationFavorite()
 * @method ApplicationForm _getRequestedApplicationForm()
 * @method Project _getRequestedProject()
 * @method Degree _getRequestedDegree()
 * @method Work _getRequestedWork()
 * @method Work[] _getRequestedWorks($throwIsEmptyException = true)
 * @method Exposition _getRequestedResearch()
 * @method SimpleMedia _getRequestedSimpleMedia()
 * @method Tool _getRequestedItem()
 * @method Tool _getRequestedTool()
 * @method Tool[] _getRequestedItems($throwIsEmptyException = true)
 * @method Tool[] _getRequestedTools($throwIsEmptyException = true)
 * @method Weave _getRequestedWeave()
 * @method Snapshot _getRequestedSnapshot()
 * @method SimpleMedia _getRequestedFile()
 * @method SimpleMedia[] _getRequestedFiles($throwIsEmptyException = true)
 * @method User _getRequestedPerson()
 * @method User _getRequestedUser()
 * @method Popover _getRequestedPopover()
 * @method ProfileGroup _getRequestedProfileGroup()
 * @method ProfileGroupElement _getRequestedProfileGroupElement()
 * @method Portal _getRequestedPortal()
 * @method Portal\Issue _getRequestedIssue()
 * @method ProfileShortcut _getRequestedProfileShortcut()
 * @method Comment _getRequestedComment()
 * @method Message _getRequestedMessage()
 * @method Message[] _getRequestedMessages()
 * @method Zend_Controller_Request_Http getRequest()
 * @method Zend_Controller_Response_Http getResponse()
 * @property Zend_Controller_Request_Http $_request
 * @property Zend_Controller_Response_Http $_response
 */
class BasicControllerAction extends Zend_Controller_Action
{
    const MAINTENANCE_COOKIE_NAME = 'rc_maintenance';

    /**
     * @var array
     */
    private $objectContainer = [];

    /**
     * provides an easy access to passed domain objects (from request referenced by id)
     *
     * @param string $method
     * @param array $args
     * @throws \Exception
     * @return array
     */
    public function __call($method, $args)
    {
        if (substr($method, 0, 13) == '_getRequested') {
            if (substr($method, -1) == 's') {
                return $this->_getRequestedObjects(substr($method, 13, -1), isset($args[0]) ? $args[0] : true);
            } else {
                return $this->_getRequestedObject(substr($method, 13));
            }
        } elseif (substr($method, 0, 14) == '_findRequested') {
            return $this->_findRequestedObject(substr($method, 14));
        }

        parent::__call($method, $args);
    }

    public function init()
    {
        Metrics::increment('requests', 1, ['controller' => $this->_getParam('controller'), 'action' => $this->_getParam('action')]);

        $this->initBasic();
    }

    public function initBasic()
    {
        if (!config('app.maintenance')) {
            return;
        }

        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            return;
        }

        if ($this->_getParam('controller') == 'portal' && $this->_getParam('action') == 'maintenance') {
            return;
        }

        if (isset($_COOKIE[self::MAINTENANCE_COOKIE_NAME]) && $_COOKIE[self::MAINTENANCE_COOKIE_NAME] == 'ignore') {
            return;
        }

        if ($this->_getParam('controller') == 'session') {
            return;
        }

        $this->redirect('portal/maintenance');
    }

    public function initPortal()
    {
        $this->initBasic();

        if (!$this->_suppressLayout()) {
            Zend_Layout::startMvc(
                [
                    'layout'     => 'portal',
                    'layoutPath' => APPLICATION_PATH . '/views/layouts',
                ]
            );
            $this->layout = Zend_Layout::getMvcInstance();

            $basic = new stdClass();
            $basic->seo = null;
            $basic->fulltext = $this->_getParam('fulltext');
            $basic->searchedFor = $_SERVER['QUERY_STRING'];
            $basic->isLoggedIn = Identity::exists();
            $basic->username = $this->_getParam('username');
            $basic->password = $this->_getParam('password');
            $basic->portals = PortalRepository::findByAllowProfile();
            if (Identity::exists()) {
                $basic->currentUser = Identity::getUser();
                $basic->unreadMessages = MessageRepository::findUnreadByReceiver(Identity::getUser(), ['limit' => 5]);
            } else {
                $basic->currentUser = null;
                $basic->unreadMessages = [];
            }

            $this->layout->basic = $basic;
        }
    }

    protected function _suppressLayout()
    {
        return $this->getRequest()->isXmlHttpRequest() || $this->getRequest()->getParam('iframe-submit');
    }

    protected function _disableView()
    {
        $this->_helper->viewRenderer->setNoRender(true);
    }

    protected function _disableLayout()
    {
        $this->_helper->layout->disableLayout();
    }

    protected function _setSuccessHeader()
    {
        header('Form-Validation: 1');
        $this->view->formIsValid = true;
        if (is_object(Zend_Layout::getMvcInstance())) {
            Zend_Layout::getMvcInstance()->formIsValid = true; // for requests via iframe
        }
    }

    protected function _setFormActionHeader($action)
    {
        if (is_object($action) && $action instanceof \Rc\Domain\Model\AbstractModel) {
            $url = '/' . $action->getName() . '/edit?' . $action->getName() . '=' . $action->getId();
        } elseif (is_string($action)) {
            $url = $action;
        } else {
            throw new Exception('UNKNOWN PARAMETER TYPE');
        }

        header('Form-Action: ' . $url);
        $this->view->formAction = true;
        if (is_object(Zend_Layout::getMvcInstance())) {
            Zend_Layout::getMvcInstance()->formAction = $url; // for requests via iframe
        }
    }

    /**
     * @return Research
     * @throws \Exception
     */
    protected function _getRequestedEntity()
    {
        $id = $this->getRequest()->getParam('entity', 0);
        return $this->_loadRequestedObject('ProfileEntity', $id);
    }

    /**
     * @param string $objectName
     * @return object
     */
    protected function _findRequestedObject($objectName)
    {
        try {
            return $this->_getRequestedObject($objectName);
        } catch (Exception $e) {
            return null;
        }
    }

    protected function _getRequestedObject($objectName)
    {
        $id = $this->getRequest()->getParam(strtolower($objectName), 0);
        if (empty($id)) {
            $id = $this->getRequest()->getParam(Strings::toSnakeCase($objectName, '-'), 0);
        }

        return $this->_loadRequestedObject($objectName, $id);
    }

    protected function _getRequestedObjects($objectName, $throwIsEmptyException = true)
    {
        $ids = (array)$this->getRequest()->getParam(strtolower($objectName), []);
        if (empty($ids)) {
            $ids = $this->getRequest()->getParam(Strings::toSnakeCase($objectName, '-'), []);
        }

        if (count($ids) == 0 && $throwIsEmptyException) {
            throw new Exception('INVALID_' . strtoupper($objectName) . '_IDS', 1492580755);
        }

        $retVal = [];
        foreach ($ids as $id) {
            $retVal[$id] = $this->_loadRequestedObject($objectName, $id);
        }
        return $retVal;
    }

    protected function _loadRequestedObject($objectName, $id)
    {
        $repository = ucfirst($objectName) . 'Repository';
        switch ($repository) {
            case 'ResearchRepository':
                $repository = ExpositionRepository::class;
                break;
            case 'ProjectRepository':
                $repository = ProjectRepository::class;
                break;
            case 'DegreeRepository':
                $repository = DegreeRepository::class;
                break;
            case 'WorkRepository':
                $repository = WorkRepository::class;
                break;
            case 'ProfileEntityRepository':
                $repository = ResearchRepository::class;
                break;
            case 'WeaveRepository':
                $repository = WeaveRepository::class;
                break;
            case 'PopoverRepository':
                $repository = PopoverRepository::class;
                break;
            case 'PortalRepository':
                $repository = PortalRepository::class;
                break;
            case 'IssueRepository':
                $repository = IssueRepository::class;
                break;
            case 'FileRepository':
            case 'SimpleMediaRepository':
                $repository = SimpleMediaRepository::class;
                break;
            case 'UserRepository':
            case 'PersonRepository':
                $repository = UserRepository::class;
                break;
            case 'ProfileGroupRepository':
                $repository = ProfileGroupRepository::class;
                break;
            case 'CommentRepository':
                $repository = CommentRepository::class;
                break;
            case 'ReportRepository':
                $repository = ReportRepository::class;
                break;
            case 'ItemRepository':
            case 'ToolRepository':
                $repository = ToolRepository::class;
                break;
            case 'MessageRepository':
                $repository = MessageRepository::class;
                break;
            case 'ApplicationRepository':
                $repository = ApplicationRepository::class;
                break;
            case 'ApplicationFavoriteRepository':
                $repository = ApplicationFavoriteRepository::class;
                break;
            case 'ApplicationFormRepository':
                $repository = ApplicationFormRepository::class;
                break;
            case 'ApplicationProgramRepository':
                $repository = ApplicationProgramRepository::class;
                break;
            case 'AnnouncementRepository':
                $repository = AnnouncementRepository::class;
                break;
            case 'UpdateRepository':
                $repository = UpdateRepository::class;
                break;
            case 'NewsletterRepository':
                $repository = NewsletterRepository::class;
                break;
            case 'SnapshotRepository':
                $repository = SnapshotRepository::class;
                break;
            default:
                throw new Exception('Unsupported dilps repository ' . $repository . '! Migrate it...');
        }

        if (!isset($this->objectContainer[$id])) {
            if (strlen($id) != 24 && (!ctype_digit((string)$id) || (int)$id <= 0)) {
                throw new Exception('INVALID_' . strtoupper($objectName) . '_ID [' . $id . ']', 1492580754);
            }
            $this->objectContainer[$id] = $repository::getById($id);
        }

        if (method_exists($repository, 'getModelName') && !is_a($this->objectContainer[$id], $repository::getModelName())) {
            throw new Exception('INCORRECT_REPOSITORY_FOR_DOMAIN_OBJECT [' . $repository . ' - ' . get_class($this->objectContainer[$id]) . ']');
        }

        return $this->objectContainer[$id];
    }

    /**
     * @return DefaultForm
     * @throws Exception
     */
    protected function _getForm()
    {
        $className = substr(get_called_class(), 0, -strlen('Controller')) . 'FormGenerator';
        if (!class_exists($className)) {
            throw new Exception('unknown form generator class "' . $className . '".');
        }

        // convert xxx-yyy to XxxYyy
        $formMethod = 'get' . Strings::toCamelCase($this->getRequest()->getActionName(), '-');

        if (!method_exists($className, $formMethod)) {
            throw new Exception('unknown method "' . $formMethod . '" in form generator class "' . $className . '".');
        }

        if (func_num_args()) {
            return call_user_func_array([$className, $formMethod], func_get_args());
        } else {
            return $className::$formMethod();
        }
    }

    /**
     * @param DefaultForm $form
     * @return DefaultForm|null
     */
    protected function _getSubmittedTab(DefaultForm $form)
    {
        foreach ($form->getTabs() as $tab) {
            $tabName = substr($tab->getName(), 3);
            $tabParams = $this->_getParam($tabName);
            if (isset($tabParams['submitbutton']) || $this->_getParam('mediatype') == $tabName) {
                return $tab;
            }
        }
        return null;
    }

    protected function _sendMessage(Message $message)
    {
        // redirect administrator (6) messages to rc user support (45585)
        if (app_env('production') && $message->getReceiverId() == 6 && $message->getSenderId() != 5) {
            $message->setReceiver(UserRepository::findById('45585'));
        }

        MessageRepository::persist($message);

        if (config('app.allow.messaging') && $message->getReceiver()->isApproved() && $message->getReceiver()->getSettings()->getMessageNotification()) {
            try {
                $mail = new Mail();
                $mail->setFrom(config('app.email'));
                $mail->addTo($message->getReceiver()->getEmail());
                $mail->setSubject(sprintf($this->view->translate('message.subject.unread.message'), $message->getSubject(),
                    $message->getSender()->getName()));
                $mail->setBodyHtml(
                    sprintf(
                        $this->view->translate('message.content.unread.message.html'),
                        $message->getReceiver()->getName(),
                        $message->getSender()->getName(),
                        preg_replace('~<div.*</div>~is', sprintf($this->view->translate('message.mail.placeholder'), $message->getId()),
                            $message->getContent()),
                        $message->getId()
                    ),
                    'UTF-8'
                );
                foreach ($message->getAttachments() as $name => $attachment) {
                    $part = new Zend_Mime_Part(file_get_contents($attachment->getMaster()->getUri()));
                    $part->type = $attachment->getMaster()->getMimeType();
                    $part->disposition = Zend_Mime::DISPOSITION_ATTACHMENT;
                    $part->encoding = Zend_Mime::ENCODING_BASE64;
                    $part->filename = $name;

                    $mail->addAttachment($part);
                }
                $mail->send();
            } catch (Exception $e) {
            }
        }
    }

    /**
     * @param User $user
     * @param array $portalIds
     */
    protected function _handlePortalMembership(User $user, array $portalIds)
    {
        $defaultPortalId = PortalRepository::getDefault()->getId();
        $oldPendingPortalIds = $user->getPendingPortalIds();

        $user->setPortalIds(array_merge([$defaultPortalId], array_intersect($user->getPortalIds(), $portalIds)));
        $user->setPendingPortalIds(array_diff($portalIds, $user->getPortalIds()));

        $addedPortalIds = array_diff($user->getPendingPortalIds(), $oldPendingPortalIds);
        foreach (PortalRepository::findByIds($addedPortalIds) as $portal) {
            foreach ($portal->getAdminIds() as $adminId) {
                $message = new Message();
                $message->setSenderId($user->getId());
                $message->setReceiverId($adminId);
                $message->setType(Message::TYPE_PORTAL_ADD);
                $message->setContent([
                    'portal' => $portal->getId(),
                    'user'   => $user->getId(),
                    'type'   => 'add',
                ]);
                $message->setDeletedForSender(true);

                $this->_sendMessage($message);
            }
        }
    }

    protected function _handleCollaboration(Research $entity, $new, $old, $type)
    {
        $addedContributors = array_diff($new, $old);
        $removedContributors = array_diff($old, $new);
        $canceledContributors = [];

        foreach ($removedContributors as $key => $userId) {
            $user = UserRepository::findById($userId);
            if (!isset($user)) {
                unset($removedContributors[$key]);
                continue;
            }

            if ($entity->isInvitePending($userId, $type)) {
                unset($removedContributors[$key]);
                $canceledContributors[] = $userId;
            } elseif ($type === 'supervisor') {
                $entity->addDisapprovedCollaborator($userId, $type);
            }
        }

        $entity->addPendingInvites($addedContributors, $type);
        $entity->removePendingInvites($removedContributors, $type);

        $this->_sendCollaborationMessage($entity, $addedContributors, $type, 'add');
        $this->_sendCollaborationMessage($entity, $removedContributors, $type, $type === 'supervisor' ? 'disapprove' : 'remove');
        $this->_sendCollaborationMessage($entity, $canceledContributors, $type, 'cancel');
    }

    protected function _handleAuthorChange(Exposition $entity, User $newAuthor)
    {
        $oldAuthor = $entity->getAuthor();

        $entity->setAuthor($newAuthor);
        $entity->removeCoAuthor($newAuthor);
        $entity->setCoAuthors(array_merge([$oldAuthor->getId()], $entity->getCoAuthors()));

        $newAuthor->followEntity($entity);
    }

    protected function _handleOwnerChange(Exposition $entity, User $newOwner)
    {
        $oldOwner = $entity->getAuthor();

        $entity->setAuthor($newOwner);
        $entity->setOwner($newOwner);
        $entity->removeCoAuthor($newOwner);
        $entity->setCoAuthors(array_merge([$oldOwner->getId()], $entity->getCoAuthors()));
    }

    protected function _sendCollaborationMessage(Research $entity, array $users, $type, $action)
    {
        foreach ($users as $user) {
            if (is_object($user)) {
                $userId = $user->getId();
            } else {
                $userId = $user;
            }

            $message = new Message();

            if ($action == 'admin.add' || $action == 'admin.remove') {
                $message->setSenderId(Identity::getUserId());
                $message->setReceiverId($userId);
            } elseif ($action == 'decline' || $action == 'accept' || $action == 'leave') {
                $message->setSenderId($userId);
                $message->setReceiverId($entity->getAuthor()->getId());
            } else {
                $message->setSenderId($entity->getAuthor()->getId());
                $message->setReceiverId($userId);
            }

            $message->setType(constant('Message::TYPE_COLLABORATION_' . str_replace('.', '_', strtoupper($action))));
            $message->setContent([
                'entity'      => $entity->getId(),
                'entity_type' => $entity->getName(),
                'user'        => $userId,
                'type'        => $type,
            ]);

            $this->_sendMessage($message);
        }
    }

    /**
     * @param Exposition|null $research
     * @param bool $skipEntityChange
     */
    protected function updateResearchLastModified(Exposition $research = null, $skipEntityChange = false)
    {
        if (!isset($research) || $research->isPublished()) {
            return;
        }

        if ($research->trackChangesForLastModified()) {
            $research->setLastModified();
            ExpositionRepository::persist($research);
        }

        if (!$skipEntityChange) {
            EntityChangeRepository::addChange($research, EntityChange::TYPE_EDIT);
        }
    }

    /**
     * @return bool
     */
    protected function isFormSubmit()
    {
        return $this->getParam('submitbutton') == 'submitbutton';
    }

    /**
     * @param string $message
     */
    protected function _handleInsufficientRights($message = 'You do not have sufficient rights to see this page.')
    {
        if (Identity::exists()) {
            $this->initPortal();
            $this->view->assign('insufficientRights', '<b>' . $message . '</b>');
        } else {
            $this->forward('login', 'portal');
        }
    }

    /**
     * @return Exposition|null
     */
    protected function _getRequestedResearchAndCheckForInvitation()
    {
        $defaultErrorMessage = 'You do not have permissions to access this research! Please check if you have entered a correct url, the research still exists or if you are still a collaborator. If you think it\'s a bug please contact the <a href="mailto:support@researchcatalogue.net">RC User Support</a>';

        try {
            return $this->_getRequestedResearch();
        } catch (InsufficientRightsException $exception) {
            $researchId = $this->_getParam('research');
            if (Zend_Registry::get('RC_DILPS_AUTH')->hasTemporaryReadGroupFor($researchId)) {
                return $this->_handleInsufficientRights($defaultErrorMessage);
            }

            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($researchId);
            $exposition = $this->_getRequestedResearchAndCheckForInvitation();
            if (!isset($exposition)) {
                return $this->_handleInsufficientRights($defaultErrorMessage);
            }

            $exposition = $this->_getRequestedResearch();
            if (!$exposition->isInvitePending(Identity::getUserId())) {
                return $this->_handleInsufficientRights($defaultErrorMessage);
            }

            $message = MessageRepository::findLastPendingInvite(Identity::getUser(), $exposition);
            if (!isset($message)) {
                return $this->_handleInsufficientRights($defaultErrorMessage);
            }

            return $this->_handleInsufficientRights('You do not have permissions to access this research, but we have found an active collaboration invitation which grants you access to this research. <a href="javascript:showMessageDialog(' . $message->getId() . ')">Display message</a>');
        }
    }
}

?>
