<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;

class ProfileGroupService
{
    /**
     * @var array
     */
    protected static $defaultGroupCache = [];

    /**
     * @var array
     */
    protected static $profileGroups = [
        ProfileGroupRepository::GROUP_NAME_EXPOSITION,
        ProfileGroupRepository::GROUP_NAME_WORK,
        ProfileGroupRepository::GROUP_NAME_ACTIVITY,
        ProfileGroupRepository::GROUP_NAME_EXPOSITION_COLLABORATED,
        ProfileGroupRepository::GROUP_NAME_EXPOSITION_SUPERVISED,
        ProfileGroupRepository::GROUP_NAME_WORK_COLLABORATED,
        ProfileGroupRepository::GROUP_NAME_WORK_SUPERVISED,
        ProfileGroupRepository::GROUP_NAME_ACTIVITY_COLLABORATED,
        ProfileGroupRepository::GROUP_NAME_ACTIVITY_SUPERVISED,
        ProfileGroupRepository::GROUP_NAME_APPLICATION,
        ProfileGroupRepository::GROUP_NAME_APPLICATION_COLLABORATED,
        ProfileGroupRepository::GROUP_NAME_APPLICATION_SUPERVISED,
    ];

    /**
     * @param int $userId
     * @param array $elements
     */
    public static function assignElementsToDefaultGroup($userId, array $elements)
    {
        $entityList = self::getEntityList($elements);
        self::assignToDefaultGroup($userId, $entityList);
    }

    /**
     * @param User $user
     */
    public static function assignGrouplessElementsToDefaultGroup(User $user)
    {
        self::assignGrouplessElementsToDefaultProfileGroups($user->getId(), ResearchRepository::findByPerson($user));
    }

    /**
     * @param User $user
     */
    public static function createDefaultGroupsIfNecessary(User $user)
    {
        foreach (self::$profileGroups as $key => $value) {
            if (!empty(ProfileGroupRepository::countByUserIdAndName($user->getId(), $value))) {
                continue;
            }
            self::createAndStoreGroup($user->getId(), $value, $key);
        }
    }

    /**
     * @param int $userId
     * @param array $elements
     */
    protected static function assignGrouplessElementsToDefaultProfileGroups($userId, array $elements)
    {
        $entityList = self::getEntityList($elements);

        $missingEntityIds = ProfileGroupElementRepository::filterGroupLessEntityIds($userId, array_keys($entityList));

        $missingEntities = array_intersect_key($entityList, array_flip($missingEntityIds));

        self::assignToDefaultGroup($userId, $missingEntities);
    }

    /**
     * @param array $elements
     * @return \Rc\Domain\Model\ModelInterface[]
     */
    protected static function getEntityList(array $elements)
    {
        $list = [];
        foreach ($elements as $element) {
            if ($element instanceof ProfileGroupElement) {
                $list[$element->getEntityId()] = ResearchRepository::findById($element->getEntityId());
            } else if ($element instanceof \Rc\Domain\Model\ModelInterface) {
                $list[$element->getId()] = $element;
            } else {
                throw new InvalidArgumentException('Object type "' . get_class($element) . '" is not supported.');
            }
        }

        return $list;
    }

    /**
     * @param int $userId
     * @param Research[] $entityList
     */
    protected static function assignToDefaultGroup($userId, array $entityList)
    {
        if (empty($entityList)) {
            return;
        }

        ProfileGroupElementRepository::removeByUserIdAndEntityIds($userId, array_keys($entityList));

        foreach ($entityList as $entity) {
            $groupElement = new ProfileGroupElement();
            $groupElement->setUserId($userId);
            $groupElement->setEntityId($entity->getId());
            $groupElement->setGroupId(self::getDefaultGroup($userId, $entity)->getId());
            $groupElement->setLayout($entity instanceof Exposition ? ProfileGroupElement::LAYOUT_2x2 : ProfileGroupElement::LAYOUT_1x1);
            ProfileGroupElementRepository::add($groupElement);
        }
    }

    /**
     * @param int $userId
     * @param Research $entity
     * @param bool $createMissingGroup
     * @throws \Exception
     * @return \ProfileGroup
     */
    protected static function getDefaultGroup($userId, Research $entity, $createMissingGroup = true)
    {
        $person = UserRepository::findById($userId);
        if ($entity instanceof Exposition) {
            if ($entity->isSupervisor($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_EXPOSITION_SUPERVISED;
            } else if ($entity->isCollaborateur($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_EXPOSITION_COLLABORATED;
            } else {
                $name = ProfileGroupRepository::GROUP_NAME_EXPOSITION;
            }
        } else if ($entity instanceof Work) {
            if ($entity->isSupervisor($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_WORK_SUPERVISED;
            } else if ($entity->isCollaborateur($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_WORK_COLLABORATED;
            } else {
                $name = ProfileGroupRepository::GROUP_NAME_WORK;
            }
        } else if ($entity instanceof Degree || $entity instanceof Project) {
            if ($entity->isSupervisor($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_ACTIVITY_SUPERVISED;
            } else if ($entity->isCollaborateur($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_ACTIVITY_COLLABORATED;
            } else {
                $name = ProfileGroupRepository::GROUP_NAME_ACTIVITY;
            }
        } else if ($entity instanceof Application) {
            if ($entity->isSupervisor($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_APPLICATION_SUPERVISED;
            } else if ($entity->isCollaborateur($person)) {
                $name = ProfileGroupRepository::GROUP_NAME_APPLICATION_COLLABORATED;
            } else {
                $name = ProfileGroupRepository::GROUP_NAME_APPLICATION;
            }
        } else {
            throw new InvalidArgumentException('Object type "' . get_class($entity) . '" is not supported.');
        }

        $cacheEntry =& self::$defaultGroupCache[$userId . '.' . $name];
        if (isset($cacheEntry)) {
            return $cacheEntry;
        }

        $cacheEntry = ProfileGroupRepository::findByUserIdAndName($userId, $name);
        if (isset($cacheEntry)) {
            return $cacheEntry;
        }

        if (!$createMissingGroup) {
            throw new Exception('Group "' . $name . '" for user ' . $userId . ' is missing.');
        }

        self::createDefaultGroupsIfNecessary($person);
        return self::getDefaultGroup($userId, $entity, false);
    }

    protected static function createAndStoreGroup($userId, $groupName, $position)
    {
        $group = new ProfileGroup();
        $group->setPosition($position);
        $group->setName($groupName);
        $group->setUserId($userId);
        $group->setRemovable(false);
        ProfileGroupRepository::add($group);
    }

}
