<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition\Weave\Tool;

/**
 * @author Thomas Mühlschlegel - thomas.muehlschlegel@web.de - Karlsruhe, Germany
 *
 * @param $list : supposed an object with the following structure:
 *    Array
 *            (
 *                [0] => Array
 *                        (
 *                        [id] => Array
 *                            (
 *                                [entityID] => 44
 *                                [collectionID] => 32
 *                            )
 *                        [x] => 627
 *                        [y] => 33
 *                        [width] => 512
 *                        [height] => 347
 *                    )
 *                [1] => ...
 *                ...
 *            )
 *        )
 *
 * @param $conf : config array. for more information check the private configuration array
 *
 */
class WeaveThumb
{

    private $image = null;
    private $scaleFactor = 1;
    private $conf = array(
        'width' => null, // use null for auto width
        'height' => null, // use null for auto height
        'maxwidth' => null, // scale image if width greater than maxwidth
        'maxheight' => null, // scale image if height greater than maxheight
        'padding' => array(
            'top' => 0,
            'right' => 0,
            'bottom' => 0,
            'left' => 0,
        ),
        'transparent' => true,
        'drawText' => false,
        'quality' => 0, // Compression level: from 0 (no compression) to 9.
        'destination' => null, // path + filename
        'fontSize' => 2, // Can be 1-5 for built-in fonts in latin2 encoding (where higher numbers corresponding to larger fonts)
        'fontColor' => '000000',
        'borderColor' => '000000',
        'backgroundColor' => 'FFFFFF',
    );

    /**
     * WeaveThumb constructor.
     *
     * @param Iterator|array $items
     * @param array $conf
     */
    public function __construct($items, array $conf = array())
    {
        $this->items = $items;
        $this->conf = $this->convertColorStringsToRgbArray($conf, array('fontColor', 'borderColor', 'backgroundColor')) + $this->conf;
        $this->createImage();
    }

    public function read($destroy = true)
    {
        imagepng($this->image);
        if ($destroy) {
            imagedestroy($this->image);
        }
    }

    public function save($destroy = true)
    {
        if (!isset($this->conf['destination'])) {
            throw new Exception('output file is not set!');
        }
        if (!imagepng($this->image, $this->destination)) {
            throw new Exception('image could not be saved!');
        }
        if ($destroy) {
            imagedestory($this->image);
        }
        return $this->destination;
    }

    private function convertColorStringsToRgbArray($conf, $keysToEdit)
    {
        foreach ($keysToEdit as $key) {
            if (isset($conf[$key]) && is_string($conf[$key])) {
                $pieces = str_split($conf[$key], 2);
                $conf[$key] = array(
                    hexdec($pieces[0]),
                    hexdec($pieces[1]),
                    hexdec($pieces[2]),
                );
            }
        }
        return $conf;
    }

    private function drawItems()
    {
        $fontWidth = imagefontwidth($this->conf['fontSize']);
        $fontHeight = imagefontheight($this->conf['fontSize']);

        /** @var Tool $item */
        foreach ($this->items as $item) {
            $this->drawRectangle(
                ($item->getLeft() + $this->conf['padding']['left']) * $this->scaleFactor,
                ($item->getTop() + $this->conf['padding']['top']) * $this->scaleFactor,
                $item->getOuterWidth() * $this->scaleFactor,
                $item->getOuterHeight() * $this->scaleFactor,
                $item->getRotate()
            );

            if ($this->conf['drawText']) {
                $this->drawText(
                    ($item->getLeft() + $item->getOuterWidth()) * $this->scaleFactor - $fontWidth * strlen($item->getId()),
                    ($item->getTop() + $item->getOuterHeight()) * $this->scaleFactor - $fontHeight, $item->getId()
                );
            }
        }
    }

    private function getMaxImageSize()
    {
        $size = (object)array('width' => 0, 'height' => 0);

        foreach ($this->items as $item) {
            $size->width = max($size->width, $item->getLeft() + $item->getOuterWidth() + $this->conf['padding']['left'] + $this->conf['padding']['right']);
            $size->height = max($size->height, $item->getTop() + $item->getOuterHeight() + $this->conf['padding']['top'] + $this->conf['padding']['bottom']);
        }

        return $size;
    }

    private function isSetImageSize()
    {
        return isset($this->conf['width']) && isset($this->conf['height']);
    }

    private function setImageSize($size)
    {
        if (!isset($this->conf['width'])) {
            $this->conf['width'] = $size->width;
        }
        if (!isset($this->conf['height'])) {
            $this->conf['height'] = $size->height;
        }
    }

    private function createImage()
    {
        if (!$this->isSetImageSize()) {
            $size = $this->getMaxImageSize();
            $this->setImageSize($size);
        }

        $this->setScaleFactorAndAdjustImageSize();

        $this->image = imagecreate($this->conf['width'], $this->conf['height']);

        if ($this->conf['transparent']) {
            $this->setBackgroundTransparent();
        } else {
            $this->setBackgroundColor();
        }

        $this->drawItems();
    }

    private function setScaleFactorAndAdjustImageSize()
    {
        $this->setScaleFactor('width');
        $this->setScaleFactor('height');

        $this->adjustImageSize('width');
        $this->adjustImageSize('height');
    }

    private function setScaleFactor($side)
    {
        if (isset($this->conf['max' . $side]) && $this->conf[$side] > $this->conf['max' . $side]) {
            $this->scaleFactor = min($this->scaleFactor, $this->conf['max' . $side] / $this->conf[$side]);
        }
    }

    private function adjustImageSize($side)
    {
        $this->conf[$side] = $this->scaleFactor * $this->conf[$side];
        $this->conf[$side] += 1; // to show border of container at the image border pixels is added
    }

    private function setBackgroundTransparent()
    {
        imagecolortransparent($this->image, imagecolorallocate($this->image, 0x00, 0xFF, 0x00));
    }

    private function setBackgroundColor()
    {
        $bgColor = imagecolorallocate($this->image, $this->conf['backgroundColor'][0], $this->conf['backgroundColor'][1], $this->conf['backgroundColor'][2]);

        imagefilledrectangle($this->image, 0, 0, $this->conf['width'], $this->conf['height'], $bgColor);
    }

    /**
     * @param $centerX
     * @param $centerY
     * @param $angle
     * @param $point
     * @see http://stackoverflow.com/questions/2259476/rotating-a-point-about-another-point-2d
     */
    private function rotate($centerX, $centerY, $angle, $point)
    {
        $radiant = $angle * acos(-1) / 180;

        $s = sin($radiant);
        $c = cos($radiant);

        // translate point back to origin:
        $point->x -= $centerX;
        $point->y -= $centerY;

        // rotate point
        $xnew = $point->x * $c - $point->y * $s;
        $ynew = $point->x * $s + $point->y * $c;

        // translate point back:
        $point->x = $xnew + $centerX;
        $point->y = $ynew + $centerY;
    }

    private function drawRectangle($x, $y, $width, $height, $rotate)
    {
        $a = (object)array('x' => $x, 'y' => $y);
        $b = (object)array('x' => $x + $width, 'y' => $y);
        $c = (object)array('x' => $x + $width, 'y' => $y + $height);
        $d = (object)array('x' => $x, 'y' => $y + $height);

        if ($rotate != 0) {
            $rotateX = $x + ($width / 2);
            $rotateY = $y + ($height / 2);
            $this->rotate($rotateX, $rotateY, $rotate, $a);
            $this->rotate($rotateX, $rotateY, $rotate, $b);
            $this->rotate($rotateX, $rotateY, $rotate, $c);
            $this->rotate($rotateX, $rotateY, $rotate, $d);
        }

        $borderColor = ImageColorAllocate($this->image, $this->conf['borderColor'][0], $this->conf['borderColor'][1], $this->conf['borderColor'][2]);

        imageline($this->image, $a->x, $a->y, $b->x, $b->y, $borderColor);
        imageline($this->image, $b->x, $b->y, $c->x, $c->y, $borderColor);
        imageline($this->image, $c->x, $c->y, $d->x, $d->y, $borderColor);
        imageline($this->image, $d->x, $d->y, $a->x, $a->y, $borderColor);
    }

    private function drawText($x, $y, $text)
    {
        $fontColor = ImageColorAllocate($this->image, $this->conf['fontColor'][0], $this->conf['fontColor'][1], $this->conf['fontColor'][2]);

        imagestring($this->image, $this->conf['fontSize'], $x, $y, $text, $fontColor);
    }

}

?>