<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\AbstractDilpsModel;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

class EntityChangeRepository extends MySqlRepository
{
    /**
     * @param ModelInterface $entity
     * @param string $type
     * @param User $identity
     */
    public static function addChange(ModelInterface $entity, $type, $identity = null)
    {
        if (!Identity::exists()) {
            return;
        }
        if (!isset($identity)) {
            $identity = Identity::getUser();
        }

        if ($entity instanceof Exposition && !$entity->isAuthor($identity) && !$entity->isCoAuthor($identity)) {
            return;
        }

        $accessLevel = $entity->getAccessLevel();
        if ($entity instanceof Exposition && $entity->isPublished()) {
            $accessLevel = AbstractDilpsModel::ACCESS_LEVEL_PUBLIC;
        }

        $values = [
            'type' => static::_escape($type),
            'entityid' => $entity->getId(),
            'userid' => $identity->getId(),
            'ownerid' => $entity->getAuthor() ? $entity->getAuthor()->getId() : null,
            'publishedinid' => method_exists($entity, 'getPortalId') && $entity->getPortalId() ? $entity->getPortalId() : null,
            'connectedtoid' => $entity->getPartOfPortalId() ?: null,
            'created' => date('Y-m-d H:i'),
            'accesslevel' => $accessLevel,
        ];
        foreach ($values as $key => &$value) {
            if (isset($value)) {
                $value = '`' . $key . '` = "' . static::_escape($value) . '"';
            } else {
                $value = '`' . $key . '` = NULL';
            }
        }

        static::_query('
            INSERT ' . static::_getJoinedTables() . '
            SET ' . implode(', ', $values) . '
            ON DUPLICATE KEY UPDATE ' . implode(', ', $values)
        );

        static::_query('
            UPDATE ' . static::_getJoinedTables() . '
            SET accesslevel = ' . $entity->getAccessLevel() . '
            WHERE entityid = ' . $entity->getId()
        );
    }

    /**
     * @param User $user
     * @param array $options
     * @return EntityChange[]
     */
    public static function findByUser(User $user, array $options = array())
    {
        $options += ['limit' => 15, 'page' => 0];
        $accessLevel = [AbstractDilpsModel::ACCESS_LEVEL_PUBLIC, AbstractDilpsModel::ACCESS_LEVEL_LIMITED];

        $result = static::_query('
            SELECT *
            FROM (
                SELECT `change`.*
                FROM `rc_userfeed` `userfeed`
                JOIN `rc_profilegroup` `group`
                  ON `userfeed`.`targetid` = `group`.`userid`
                JOIN `rc_profilegroupelement` `element`
                  ON `element`.`groupid` = `group`.`id` AND `element`.`userid` = `userfeed`.`targetid`
                JOIN `rc_entitychange` `change`
                  ON `change`.`entityid` = `element`.`entityid`
                WHERE `userfeed`.`userid` = ' . $user->getId() . '
                  AND `group`.`hide` = 0
                  AND `change`.`accesslevel` IN (' . implode(',', $accessLevel) . ')
                  AND (
                    (`userfeed`.`receiveShared` = 1 AND `change`.`type` = "share") OR
                    (`userfeed`.`receiveEdited` = 1 AND `change`.`type` = "edit") OR
                    (`userfeed`.`receiveCommented` = 1 AND `change`.`type` = "comment") OR
                    (`userfeed`.`receivePublished` = 1 AND `change`.`type` = "publish")
                  )
            UNION
                SELECT `change`.*
                FROM `rc_userfeed` `userfeed`
                JOIN `rc_entitychange` `change`
                  ON `change`.`userid` = `userfeed`.`targetid`
                WHERE `userfeed`.`userid` = ' . $user->getId() . '
                  AND `change`.`accesslevel` IN (' . implode(',', $accessLevel) . ')
                  AND (
                    (`userfeed`.`receiveShared` = 1 AND `change`.`type` = "share") OR
                    (`userfeed`.`receiveEdited` = 1 AND `change`.`type` = "edit") OR
                    (`userfeed`.`receiveCommented` = 1 AND `change`.`type` = "comment") OR
                    (`userfeed`.`receivePublished` = 1 AND `change`.`type` = "publish")
                  )
            UNION
                SELECT `change`.*
                FROM `rc_entityfeed` `entityfeed`
                JOIN `rc_entitychange` `change`
                  ON `change`.`entityid` = `entityfeed`.`targetid`
                WHERE `entityfeed`.`userid` = ' . $user->getId() . '
                  AND `change`.`accesslevel` IN (' . implode(',', $accessLevel) . ')
                  AND (
                    (`entityfeed`.`receiveShared` = 1 AND `change`.`type` = "share") OR
                    (`entityfeed`.`receiveEdited` = 1 AND `change`.`type` = "edit") OR
                    (`entityfeed`.`receiveCommented` = 1 AND `change`.`type` = "comment") OR
                    (`entityfeed`.`receivePublished` = 1 AND `change`.`type` = "publish")
                  )
            UNION
                SELECT `change`.*
                FROM `rc_portalfeed` `portalfeed`
                JOIN `rc_entitychange` `change`
                  ON `change`.`publishedinid` = `portalfeed`.`targetid` OR (
                    `change`.`connectedtoid` = `portalfeed`.`targetid` AND `change`.`accesslevel` IN (' . implode(',', $accessLevel) . ')
                  )
                WHERE `portalfeed`.`userid` = ' . $user->getId() . '
                  AND (
                    (`portalfeed`.`receiveShared` = 1 AND `change`.`type` = "share") OR
                    (`portalfeed`.`receiveEdited` = 1 AND `change`.`type` = "edit") OR
                    (`portalfeed`.`receiveCommented` = 1 AND `change`.`type` = "comment") OR
                    (`portalfeed`.`receivePublished` = 1 AND `change`.`type` = "publish")
                  )
            ) tmp
            GROUP BY `entityid`
            ORDER BY `created` DESC
            LIMIT ' . (int)($options['page'] * $options['limit']) . ',' . (int)$options['limit'] . '
        ');

        return static::_fetchAndFreeResults($result);
    }
}
