<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

/**
 * @method static ProfileGroupElement findById($id, $limit = null)
 */
class ProfileGroupElementRepository extends MySqlRepository
{

    /**
     * @param int $groupId
     * @param int $page
     * @param int $limit
     * @param array $orderBy
     * @return ProfileGroupElement[]
     */
    public static function findByGroupId($groupId, $page = null, $limit = null, $orderBy = ['position' => 'ASC'])
    {
        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                groupid = ' . static::_escape($groupId) . '
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    /**
     * @param int $userId
     * @param int $entityId
     * @return ProfileGroupElement
     * @throws \Exception
     */
    public static function findByUserIdAndEntityId($userId, $entityId)
    {
        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                entityid = ' . static::_escape($entityId) . '
        ');

        return static::_fetchAndFreeResult($result);
    }

    /**
     * @param int $userId
     * @param array $entityIds
     * @return int[]
     * @throws \Exception
     */
    public static function filterGroupLessEntityIds($userId, array $entityIds)
    {
        if (empty($entityIds)) {
            return [];
        }

        $result = static::_query('
            SELECT
                entityid
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                entityid IN (' . static::_escape(implode(',', $entityIds)) . ')
        ');

        $existingEntityIds = [];
        while ($row = $result->fetch_array()) {
            $existingEntityIds[] = $row['entityid'];
        }

        return array_diff($entityIds, $existingEntityIds);
    }

    /**
     * @param int $userId
     * @param array $entityIds
     * @return ProfileGroupElement[]
     * @throws \Exception
     */
    public static function findByUserIdAndEntityIds($userId, array $entityIds)
    {
        if (empty($entityIds)) {
            return [];
        }

        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                entityid IN (' . static::_escape(implode(',', $entityIds)) . ')
        ');

        return static::_fetchAndFreeResults($result);
    }

    /**
     * @param int $userId
     * @return ProfileGroupElement[]
     * @throws \Exception
     */
    public static function findByUserId($userId, $page = null, $limit = NUL)
    {
        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    public static function getMaximumPosition($groupId)
    {
        $result = static::_query('
            SELECT
                MAX(position)
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                groupid = ' . static::_escape($groupId) . '
        ')->fetch_array();

        return (int)reset($result);
    }

    public static function countByGroupIdAndName($groupId, $name)
    {
        $result = static::_query('
            SELECT
                COUNT(*)
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                groupid = ' . static::_escape($groupId) . '
                name = "' . static::_escape($name) . '"
        ');

        return (int)reset($result->fetch_array());
    }

    public static function add(MySqlEntity $object)
    {
        if ($object instanceof ProfileGroupElement && $object->getPosition() === null) {
            $object->setPosition(self::getMaximumPosition($object->getGroupId()) + 1);
        }
        return parent::add($object);
    }

    public static function removeByGroupId($groupId)
    {
        static::_query('
            DELETE FROM
                ' . static::_getJoinedTables() . '
            WHERE
                groupid = ' . static::_escape($groupId) . '
        ');
    }

    public static function removeByUserIdAndEntityIds($userId, array $entityIds)
    {
        if (empty($entityIds)) {
            return;
        }

        $values = [];
        foreach ($entityIds as $entityId) {
            $values[] = static::_escape($entityId);
        }

        static::_query('
            DELETE FROM
                ' . static::_getJoinedTables() . '
            WHERE
                userid = ' . static::_escape($userId) . ' AND
                entityid IN (' . implode(',', $values) . ')
        ');
    }

}