<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\Media\Resource\VideoResource;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\MediaContainerAwareInterface;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Service\Ingest\ImageIngestService;

class My_View_Helper_SizeMedia extends My_View_Helper_HtmlElement
{

    /**
     * Temporary thumbnail cache
     *
     * @var MediaInterface
     */
    protected $thumb;

    /**
     * @var array
     */
    private $_defaultOptions = [
        'size'        => Tool::SIZE_CONTAIN,
        'position'    => Tool::POSITION_CENTER_CENTER,
        'class'       => null,
        'url'         => null,
        'aspectRatio' => null,
        'media'       => 'media-only',
        'thumb'       => false,
        'raw'         => false,
        'styling'     => true,
    ];

    /**
     * Merge two arrays together.
     * If an integer key exists in both arrays and preserveNumericKeys is false, the value
     * from the second array will be appended to the first array. If both values are arrays, they
     * are merged together, else the value of the second array overwrites the one of the first array.
     *
     * @param  array $a
     * @param  array $b
     * @param  bool $preserveNumericKeys
     * @return array
     */
    protected static function merge(array $a, array $b, $preserveNumericKeys = false)
    {
        foreach ($b as $key => $value) {
            if (isset($a[$key]) || array_key_exists($key, $a)) {
                if (!$preserveNumericKeys && is_int($key)) {
                    $a[] = $value;
                } elseif (is_array($value) && is_array($a[$key])) {
                    $a[$key] = self::merge($a[$key], $value, $preserveNumericKeys);
                } else {
                    $a[$key] = $value;
                }
            } else {
                $a[$key] = $value;
            }
        }
        return $a;
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return null|string
     */
    public function auto($mediaContainer, $width, $height, array $options = [])
    {
        if ($mediaContainer instanceof MediaContainerAwareInterface) {
            $mediaContainer = $mediaContainer->getMediaContainer();
        }
        if (!$mediaContainer->hasMedia()) {
            return null;
        }

        $dimension = $mediaContainer->getMediaResolution() ?: new Dimension($width, $height);
        $dimension->fit($width, $height);

        return $this->sizeMedia($mediaContainer, $dimension->getWidth(), $dimension->getHeight(), $options);
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return null|string
     */
    public function sizeMedia($mediaContainer, $width, $height, array $options = [])
    {
        if ($mediaContainer instanceof MediaContainerAwareInterface) {
            $mediaContainer = $mediaContainer->getMediaContainer();
        }
        if (!isset($mediaContainer) || !$mediaContainer->hasMedia()) {
            return null;
        }

        $options += $this->_defaultOptions;

        if ($mediaContainer->isImage() || $mediaContainer->isPdf()) {
            return $this->renderImage($mediaContainer, $width, $height, $options);
        } elseif ($mediaContainer->isVideo()) {
            return $this->renderVideo($mediaContainer, $width, $height, $options);
        } elseif ($mediaContainer->isAudio()) {
            return $this->renderAudio($mediaContainer, $width, $height, $options);
        } elseif ($mediaContainer->isArchive()) {
            return $this->renderArchive($mediaContainer, $width, $height, $options);
        } else {
            return $this->renderUnknown($mediaContainer, $width, $height, $options);
        }
    }

    /**
     * @param ImageResource $poster
     * @param int $width
     * @param int $height
     * @param array $options
     * @return null|string
     */
    public function sizeResource(ImageResource $poster = null, $width, $height, array $options = [])
    {
        $options += $this->_defaultOptions;

        $size = $this->calculateContainerSize(
            isset($poster) ? $poster->getDimension() : new Dimension(null, null),
            $width,
            $height,
            $options['size']
        );

        if (!$options['styling']) {
            return $this->htmlElement(
                'img',
                null,
                ['src' => isset($poster) ? $this->url($poster, $size) : $options['url']],
                ['shortTag' => true]
            );
        }

        $attributes = [
            'style'         => [
                'width'  => $size->getWidth() . 'px',
                'height' => $size->getHeight() . 'px',
            ],
            'data-position' => json_encode([
                'my'        => $options['position'],
                'at'        => $options['position'],
                'collision' => 'none',
            ]),
        ];

        if (isset($poster) && $poster->getFileExtension() == 'svg' && !$options['raw']) {
            $scaleX = round($size->getWidth() / $poster->getDimension()->getWidth(), 2);
            $scaleY = round($size->getHeight() / $poster->getDimension()->getHeight(), 2);
            $containerAttributes = [
                'class'            => $options['class'],
                'data-size'        => $options['size'],
                'data-aspectRatio' => $options['aspectRatio'],
            ];
            $objectAttributes = [
                'data'   => $this->url($poster, $size),
                'type'   => $poster->getMimeType(),
                'width'  => $poster->getDimension()->getWidth() . 'px',
                'height' => $poster->getDimension()->getHeight() . 'px',
                'style'  => [
                    'transform-origin' => 'left top',
                    'transform'        => 'scale(' . $scaleX . ', ' . $scaleY . ')',
                ],
            ];
            $content = $this->htmlElement('div', $this->htmlElement('object', null, $objectAttributes), $containerAttributes + $attributes, ['render' => false]);
        } else {
            $imageAttributes = [
                'class'            => $options['class'],
                'src'              => isset($poster) ? $this->url($poster, $size) : $options['url'],
                'href'             => isset($options['imageUrl']) ? $options['imageUrl'] : null,
                'data-size'        => $options['size'],
                'data-aspectRatio' => $options['aspectRatio'],
            ];
            $content = $this->htmlElement('img', null, array_filter($attributes + $imageAttributes), ['shortTag' => true]);
        }
        $linkAttributes = [
            'href'   => $options['url'],
            'target' => '_blank',
        ];

        if (!isset($options['url'])) {
            return $content;
        }
        return $this->htmlElement('a', $content, $attributes + $linkAttributes, ['render' => false]);
    }

    /**
     * @param ImageResource $poster
     * @param int $width
     * @param int $height
     * @param array $options
     * @return null|string
     */
    public function sizeThumb(ImageResource $poster = null, $width, $height, array $options = [])
    {
        if (!isset($poster)) {
            return null;
        }

        $options += $this->_defaultOptions;
        $dimension = $this->getBestFittingSizeFor(
            $poster->getWidth(),
            $poster->getHeight(),
            $width,
            $height,
            $options['size']
        );

        $attribs = [
            'class' => $options['class'],
            'src'   => $options['url'] ?: $this->url($poster, $dimension),
            'style' => [
                'width'  => $dimension->getWidth() . 'px',
                'height' => $dimension->getHeight() . 'px',
            ],
            'data'  => [
                'position'    => json_encode([
                    'my'        => $options['position'],
                    'at'        => $options['position'],
                    'collision' => 'none',
                ]),
                'size'        => $options['size'],
                'aspectRatio' => $options['aspectRatio'],
            ],
        ];

        return $this->htmlElement('img', '', $attribs, ['shortTag' => true]);
    }

    /**
     * @param int $originalWidth
     * @param int $originalHeight
     * @param int $fitToWidth
     * @param int $fitToHeight
     * @param string $sizing
     * @return Dimension
     */
    public function getBestFittingSizeFor($originalWidth, $originalHeight, $fitToWidth, $fitToHeight, $sizing)
    {
        $mediaDimension = new Dimension($originalWidth ?: $fitToWidth, $originalHeight ?: $fitToHeight);
        $mediaWidth = $mediaDimension->getWidth();
        $mediaHeight = $mediaDimension->getHeight();

        $widthFactor = $fitToWidth / $mediaWidth;
        $heightFactor = $fitToHeight / $mediaHeight;

        switch ($sizing) {
            case Tool::SIZE_CONTAIN:
                $factor = ($widthFactor < $heightFactor) ? $widthFactor : $heightFactor;
                $width = min($fitToWidth, ceil($factor * $mediaWidth));
                $height = min($fitToHeight, ceil($factor * $mediaHeight));
                break;
            case Tool::SIZE_COVER:
                $factor = ($widthFactor > $heightFactor) ? $widthFactor : $heightFactor;
                $width = ceil($factor * $mediaWidth);
                $height = ceil($factor * $mediaHeight);
                break;
            case Tool::SIZE_FIT_WIDTH:
                $width = $fitToWidth;
                $height = ceil($fitToHeight * $widthFactor);
                break;
            case Tool::SIZE_FIT_HEIGHT:
                $width = ceil($fitToWidth * $heightFactor);
                $height = $fitToHeight;
                break;
            case Tool::SIZE_FIT:
                $width = $fitToWidth;
                $height = $fitToHeight;
                break;
            case Tool::SIZE_AUTO:
                $width = $mediaWidth;
                $height = $mediaHeight;
                break;
            default:
                $factor = ($widthFactor < $heightFactor) ? $widthFactor : $heightFactor;
                $width = ceil($factor * $mediaWidth);
                $height = ceil($factor * $mediaHeight);
                break;
        }
        return new Dimension($width, $height);
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param bool $textOnly
     * @return string
     */
    public function getErrorMessage(MediaContainerInterface $mediaContainer, $textOnly = false)
    {
        $media = $mediaContainer->getMedia();
        if (!isset($media)) {
            $key = 'error.no.media.file.selected';
        } elseif (!$media->hasMaster()) {
            $key = 'error.media.files.missing';
        } elseif ($this->fileExists($media, 'input')) {
            $key = 'error.media.files.transcoding.incoming';
        } elseif ($this->fileExists($media, 'process')) {
            $key = 'error.media.files.transcoding.process';
        } elseif ($this->fileExists($media, 'error')) {
            $key = 'error.media.files.transcoding.error';
        } else {
            $key = 'error.media.files.missing';
        }

        $message = sprintf($this->view->translate($key), $mediaContainer->getName(), $mediaContainer->getId());
        if ($textOnly) {
            return $message;
        }

        return '<div style="background:white;width:100%;height:100%">' . $message . '</div>';
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @return bool
     */
    public function canBeRendered(MediaContainerInterface $mediaContainer)
    {
        if (!$mediaContainer->hasMedia()) {
            return false;
        }

        if ($mediaContainer->isAudio() || $mediaContainer->isVideo()) {
            return $mediaContainer->getMedia()->hasDerivatives();
        }

        return true;
    }

    /**
     * @param Dimension $mediaSize
     * @param int $fitToWidth
     * @param int $fitToHeight
     * @param string $sizing
     * @return Dimension
     */
    public function calculateContainerSize($mediaSize, $fitToWidth, $fitToHeight, $sizing = Tool::SIZE_CONTAIN)
    {
        $mediaWidth = $mediaSize->getWidth() ?: $fitToWidth;
        $mediaHeight = $mediaSize->getHeight() ?: $fitToHeight;

        $widthFactor = $fitToWidth / $mediaWidth;
        $heightFactor = $fitToHeight / $mediaHeight;

        $newSize = new \stdClass();
        switch ($sizing) {
            case Tool::SIZE_CONTAIN:
                $factor = ($widthFactor < $heightFactor) ? $widthFactor : $heightFactor;
                $newSize->width = min($fitToWidth, ceil($factor * $mediaWidth));
                $newSize->height = min($fitToHeight, ceil($factor * $mediaHeight));
                break;
            case Tool::SIZE_COVER:
                $factor = ($widthFactor > $heightFactor) ? $widthFactor : $heightFactor;
                $newSize->width = ceil($factor * $mediaWidth);
                $newSize->height = ceil($factor * $mediaHeight);
                break;
            case Tool::SIZE_FIT_WIDTH:
                $newSize->width = $fitToWidth;
                $newSize->height = ceil($fitToHeight * $widthFactor);
                break;
            case Tool::SIZE_FIT_HEIGHT:
                $newSize->width = ceil($fitToWidth * $heightFactor);
                $newSize->height = $fitToHeight;
                break;
            case Tool::SIZE_FIT:
                $newSize->width = $fitToWidth;
                $newSize->height = $fitToHeight;
                break;
            case Tool::SIZE_AUTO:
                $newSize->width = $mediaWidth;
                $newSize->height = $mediaHeight;
                break;
            default:
                $factor = ($widthFactor < $heightFactor) ? $widthFactor : $heightFactor;
                $newSize->width = ceil($factor * $mediaWidth);
                $newSize->height = ceil($factor * $mediaHeight);
                break;
        }

        return new Dimension($newSize->width, $newSize->height);
    }

    /**
     * @param \Rc\Domain\Model\Media\Resource|null $resource
     * @param Dimension $dimension
     * @return string|null
     */
    public function url(\Rc\Domain\Model\Media\Resource $resource = null, $dimension = null)
    {
        if (!isset($resource)) {
            return null;
        }

        if (isset($dimension) && $resource instanceof ImageResource) {
            $cachedResource = ImageIngestService::loadCachedResource($resource, $dimension, true);
            if (isset($cachedResource)) {
                return $cachedResource->getExternalUri();
            }
        }

        return $resource->getExternalUri();
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return string
     */
    protected function renderArchive(MediaContainerInterface $mediaContainer, $width, $height, array $options = [])
    {
        $media = $mediaContainer->getMedia();
        $thumb = $mediaContainer->getThumb();
        $size = $this->calculateContainerSize(
            isset($thumb) ? $thumb->getDimension() : new Dimension(null, null),
            $width,
            $height,
            $options['size']
        );

        $attribs = [
            'id'               => 'objectviewer-' . $media->getUuid(),
            'data-type'        => $mediaContainer->getMimeType(),
            'data-levels'      => isset($options['levels']) ? $options['levels'] : 0,
            'data-start-image' => isset($options['startLevel']) ? $options['startLevel'] : 1,
            'data-auto-rotate' => isset($options['autoRotate']) ? $options['autoRotate'] : 0,
            'src'              => $this->getSplashScreenUrl($thumb),
            'style'            => [
                'width'  => $size->getWidth() . 'px',
                'height' => $size->getHeight() . 'px',
            ],
            'class'            => $options['class'],
        ];
        $params = [
            'shortTag' => true,
            'render'   => false,
        ];

        if (empty($options['thumb']) && !empty($options['research']) && !empty($options['simpleMedia'])) {
            $attribs['data-resource-uri'] = '/profile/load-archive-urls?work=' . $options['research']->getId() . '&file=' . $options['simpleMedia']->getId();

            $this->view->jQuery()->addOnLoad('
                    $("#objectviewer-' . $media->getUuid() . '").click(function () {
                        var container = $(this);
                        if (container.attr("data-resources")) {
                            initObjectViewer(container);
                        } else {
                            $("<div></div>").attr("class", "container-loading").appendTo(container.parent());
                            $.get(container.attr("data-resource-uri"), {}, function(data) {
                                container.attr("data-resources", data.join(","));
                                if (data.length > 0) {
                                    initObjectViewer(container);
                                }
                                $(".container-loading").remove();
                            }, "json");
                        }
                    });'
            );
        }

        return $this->htmlElement('img', null, $attribs, $params);
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return string
     */
    protected function renderAudio(MediaContainerInterface $mediaContainer, $width, $height, array $options = [])
    {
        if (!$this->canBeRendered($mediaContainer)) {
            return $this->getErrorMessage($mediaContainer);
        }

        $media = $mediaContainer->getMedia();
        $this->view->jQuery()->addOnLoad('
            jwplayer("audio-' . $media->getUuid() . '").setup({
                image: "' . $this->getSplashScreenUrl($mediaContainer->getThumb(), new Dimension($width, $height)) . '",
                file: "' . $this->getDerivativeUrl($media, AudioResource::DERIVATIVE_TYPE_MP3) . '",
                width: ' . $width . ',
                height: ' . $height . ',
                volume: 100,
                skin: {name: "minimal"},
                primary: "html5"
            });'
        );

        return '<div id="audio-' . $media->getUuid() . '" width="' . $width . '" height="' . $height . '"></div>';
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return string
     */
    protected function renderVideo(MediaContainerInterface $mediaContainer, $width, $height, array $options = [])
    {
        if (!$this->canBeRendered($mediaContainer)) {
            return $this->getErrorMessage($mediaContainer);
        }

        $media = $mediaContainer->getMedia();
        $uuid = uniqid($media->getUuid());

        $this->view->jQuery()->addOnLoad('
            jwplayer("video-' . $uuid . '").setup({
                image: "' . $this->getSplashScreenUrl($mediaContainer->getThumb(), new Dimension($width, $height)) . '",
                file: "' . $this->getDerivativeUrl($media, VideoResource::DERIVATIVE_TYPE_MP4) . '",
                width: ' . $width . ',
                height: ' . $height . ',
                volume: 100,
                timeSliderAbove: false,
                skin: {name: "minimal"},
                primary: "html5",
                stretching: "uniform"
            });'
        );

        return '<div id="video-' . $uuid . '" width="' . $width . '" height="' . $height . '"></div>';
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return string
     */
    protected function renderImage(MediaContainerInterface $mediaContainer, $width, $height, array $options = [])
    {
        if (!$mediaContainer->hasThumb()) {
            return 'no thumb';
        }

        if (!empty($options['thumb'])) {
            $options['imageUrl'] = $this->url($mediaContainer->getMedia()->getSplashScreen());
        }
        return $this->sizeResource($mediaContainer->getThumb(), $width, $height, $options);
    }

    /**
     * @param MediaContainerInterface $mediaContainer
     * @param int $width
     * @param int $height
     * @param array $options
     * @return string
     */
    protected function renderUnknown(MediaContainerInterface $mediaContainer, $width, $height, array $options = [])
    {
        $media = $mediaContainer->getMedia();
        return '<span style="width:' . $width . ';height:' . $height . ';vertical-align:center;text-align:center">Show [' . $media->getMaster()->getFileExtension() . ']</span>';
    }

    /**
     * @param MediaInterface $media
     * @param string $type
     * @return bool
     */
    protected function fileExists(MediaInterface $media, $type)
    {
        $master = $media->getMaster();
        $directory = config('services.zencoder.directories.' . $type);
        $filename = $master->getCheckSum() . '.' . $master->getFileExtension();

        return file_exists($directory . '/' . $filename);
    }

    /**
     * @param ImageResource $resource
     * @param Dimension $dimension
     * @return null|string
     */
    protected function getSplashScreenUrl(ImageResource $resource = null, $dimension = null)
    {
        return $this->url($resource, $dimension);
    }

    /**
     * @param MediaInterface $media
     * @param string $type
     * @return string|null
     */
    protected function getDerivativeUrl(MediaInterface $media, $type)
    {
        return $this->url($media->getDerivative($type), new Dimension(null, null));
    }
}
