<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application;

use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\Media;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 */
class ApplicationProgram extends AbstractModel
{

    const VAL_DESCRIPTION_LENGTH_MIN = 300;
    const VAL_DESCRIPTION_LENGTH_MAX = 7000;

    const ACCESS_GROUP_RC_USER = 'rc';
    const ACCESS_GROUP_PORTAL_USER = 'portal';
    const ACCESS_GROUP_APPLICANT_USER = 'applicant';

    const STATUS_CLOSED = 'closed';
    const STATUS_OPEN = 'open';
    const STATUS_TESTING = 'testing';

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $portal;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $title;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $description;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $accessGroup;

    /**
     * @var ArrayCollection|Portal[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $applicantPortals;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $startDateTime;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $endDateTime;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $contactPersons;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $admins;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $requiredDocuments;

    /**
     * Allow attachment of any number of files
     *
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $allowAnyDocuments;

    /**
     * @var integer
     * @ODM\Field(type="integer")
     */
    protected $requiredExpositions;

    /**
     * @var integer
     * @ODM\Field(type="integer")
     */
    protected $requiredWorks;

    /**
     * @var ApplicationForm
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Application\ApplicationForm", inversedBy="programs", storeAs="dbRef")
     */
    protected $applicationForm;

    /**
     * @var ApplicationForm
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Application\ApplicationForm", inversedBy="reviewPrograms", storeAs="dbRef")
     */
    protected $reviewForm;

    /**
     * @var Media
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Media", cascade={"persist"}, storeAs="dbRef")
     */
    protected $guidelines;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $publishReview;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $furtherInformation;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $status;

    public function __construct()
    {
        parent::__construct();

        $this->contactPersons = new ArrayCollection();
        $this->admins = new ArrayCollection();
        $this->applicantPortals = new ArrayCollection();
    }

    /**
     * @return array
     */
    public static function getAccessGroups()
    {
        return [
            self::ACCESS_GROUP_APPLICANT_USER => 'access-group.applicant-user',
            self::ACCESS_GROUP_PORTAL_USER    => 'access-group.portal-user',
            self::ACCESS_GROUP_RC_USER        => 'access-group.rc-user',
        ];
    }

    /**
     * @return array
     */
    public static function getStatuses()
    {
        return [
            self::STATUS_CLOSED  => 'application-program.status.closed',
            self::STATUS_OPEN    => 'application-program.status.open',
            self::STATUS_TESTING => 'application-program.status.testing',
        ];
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     */
    public function setPortal($portal)
    {
        $this->portal = $portal;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return string
     */
    public function getAccessGroup()
    {
        return $this->accessGroup;
    }

    /**
     * @param string $accessGroup
     */
    public function setAccessGroup($accessGroup)
    {
        $this->accessGroup = $accessGroup;
    }

    /**
     * @return Portal[]
     */
    public function getApplicantPortals()
    {
        return $this->applicantPortals->toArray();
    }

    /**
     * @param Portal[] $applicantPortals
     */
    public function setApplicantPortals(array $applicantPortals = null)
    {
        $this->applicantPortals = new ArrayCollection($applicantPortals);
    }

    /**
     * @return bool
     */
    public function hasApplicantPortals()
    {
        return !$this->applicantPortals->isEmpty() && $this->accessGroup == self::ACCESS_GROUP_APPLICANT_USER;
    }

    /**
     * @return array
     */
    public function getApplicantPortalIds()
    {
        return $this->createIdList($this->applicantPortals);
    }

    /**
     * @param array $applicantPortalIds
     */
    public function setApplicantPortalIds($applicantPortalIds)
    {
        $this->applicantPortals = $this->createReferenceList(Portal::class, $applicantPortalIds);
    }

    /**
     * @return DateTime
     */
    public function getStartDateTime()
    {
        return $this->startDateTime;
    }

    /**
     * @param DateTime $date
     */
    public function setStartDateTime($date)
    {
        $this->startDateTime = $date;
    }

    /**
     * @param string $startDateTime
     */
    public function setStartDate($startDateTime)
    {
        $this->startDateTime = $this->castToDateTime($startDateTime);
    }

    /**
     * @param string|bool $format
     * @return DateTime|string
     */
    public function getStartDate($format = true)
    {
        return $this->formatDateTime($this->startDateTime, $format);
    }

    /**
     * @param string $startTime format: hh:mm:ss
     */
    public function setStartTime($startTime)
    {
        if (!isset($this->startDateTime)) {
            return;
        }
        call_user_func_array([$this->startDateTime, 'setTime'], explode(':', $startTime));
    }

    /**
     * @return string
     */
    public function getStartTime()
    {
        if ($this->startDateTime) {
            return $this->startDateTime->format('H:i:s');
        } else {
            return '00:00:00';
        }
    }

    /**
     * @return DateTime
     */
    public function getEndDateTime()
    {
        return $this->endDateTime;
    }

    /**
     * @param DateTime $date
     */
    public function setEndDateTime($date)
    {
        $this->endDateTime = $date;
    }

    /**
     * @param string $endDateTime
     */
    public function setEndDate($endDateTime)
    {
        $this->endDateTime = $this->castToDateTime($endDateTime);
    }

    /**
     * @param string|bool $format
     * @return DateTime|string
     */
    public function getEndDate($format = true)
    {
        return $this->formatDateTime($this->endDateTime, $format);
    }

    /**
     * @param string $endTime format: hh:mm:ss
     */
    public function setEndTime($endTime)
    {
        if (!isset($this->endDateTime)) {
            return;
        }
        call_user_func_array([$this->endDateTime, 'setTime'], explode(':', $endTime));
    }

    /**
     * @return string
     */
    public function getEndTime()
    {
        if ($this->endDateTime) {
            return $this->endDateTime->format('H:i:s');
        } else {
            return '00:00:00';
        }
    }

    /**
     * @return User[]
     */
    public function getContactPersons()
    {
        return $this->contactPersons;
    }

    /**
     * @param User[] $contactPersons
     */
    public function setContactPersons($contactPersons)
    {
        $this->contactPersons = $contactPersons;
    }

    /**
     * @return integer[]
     */
    public function getContactPersonIds()
    {
        return $this->createIdList($this->contactPersons);
    }

    /**
     * @param integer[] $contactPersonIds
     */
    public function setContactPersonIds($contactPersonIds)
    {
        $this->contactPersons = $this->createReferenceList(User::class, $contactPersonIds);
    }

    /**
     * @return User[]
     */
    public function getAdmins()
    {
        return $this->admins->toArray();
    }

    /**
     * @param User[] $admins
     */
    public function setAdmins(array $admins)
    {
        $this->admins = $admins;
    }

    /**
     * @return integer[]
     */
    public function getAdminIds()
    {
        return $this->createIdList($this->admins);
    }

    /**
     * @param integer[] $adminIds
     */
    public function setAdminIds(array $adminIds)
    {
        $this->admins = $this->createReferenceList(User::class, $adminIds);
    }

    /**
     * @param bool $asArray
     * @return string|array
     */
    public function getRequiredDocuments($asArray = false)
    {
        if ($asArray) {
            return array_map(function ($value) {
                return 'custom_file_' . preg_replace('~[^a-z0-9_]~', '_', strtolower($value));
            }, array_filter(explode(',', $this->requiredDocuments)));
        }
        return $this->requiredDocuments;
    }

    /**
     * @param string $requiredDocuments
     */
    public function setRequiredDocuments($requiredDocuments)
    {
        $this->requiredDocuments = $requiredDocuments;
    }

    /**
     * @return bool
     */
    public function getAllowAnyDocuments()
    {
        return $this->allowAnyDocuments;
    }

    /**
     * @param bool $allowAnyDocuments
     */
    public function setAllowAnyDocuments($allowAnyDocuments)
    {
        $this->allowAnyDocuments = $allowAnyDocuments;
    }

    /**
     * @return int
     */
    public function getRequiredExpositions()
    {
        return $this->requiredExpositions;
    }

    /**
     * @param int $requiredExpositions
     */
    public function setRequiredExpositions($requiredExpositions)
    {
        $this->requiredExpositions = $requiredExpositions;
    }

    /**
     * @return int
     */
    public function getRequiredWorks()
    {
        return $this->requiredWorks;
    }

    /**
     * @param int $requiredWorks
     */
    public function setRequiredWorks($requiredWorks)
    {
        $this->requiredWorks = $requiredWorks;
    }

    /**
     * @return ApplicationForm
     */
    public function getApplicationForm()
    {
        return $this->applicationForm;
    }

    /**
     * @param ApplicationForm|int $applicationForm
     */
    public function setApplicationForm($applicationForm)
    {
        $this->applicationForm = $applicationForm;
    }

    /**
     * @return integer
     */
    public function getApplicationFormId()
    {
        if (!isset($this->applicationForm)) {
            return null;
        }
        return $this->applicationForm->getId();
    }

    /**
     * @param integer $applicationFormId
     */
    public function setApplicationFormId($applicationFormId)
    {
        $this->applicationForm = $this->createReference(ApplicationForm::class, $applicationFormId);
    }

    /**
     * @return ApplicationForm
     */
    public function getReviewForm()
    {
        return $this->reviewForm;
    }

    /**
     * @param Media $reviewForm
     */
    public function setReviewForm($reviewForm)
    {
        $this->reviewForm = $reviewForm;
    }

    /**
     * @return integer
     */
    public function getReviewFormId()
    {
        if (!isset($this->reviewForm)) {
            return null;
        }
        return $this->reviewForm->getId();
    }

    /**
     * @param integer $reviewFormId
     */
    public function setReviewFormId($reviewFormId)
    {
        $this->reviewForm = $this->createReference(ApplicationForm::class, $reviewFormId);
    }

    /**
     * @return Media
     */
    public function getGuidelines()
    {
        try {
            if (isset($this->guidelines)) {
                $media = $this->guidelines->getName();
            }
        } catch (DocumentNotFoundException $exception) {
            trigger_error($exception->getMessage(), E_USER_WARNING);

            $this->guidelines = null;
        }
        return $this->guidelines;
    }

    /**
     * @param Media $guidelines
     */
    public function setGuidelines($guidelines)
    {
        $this->guidelines = $guidelines;
    }

    /**
     * @return bool
     */
    public function isPublishReview()
    {
        return $this->publishReview;
    }

    /**
     * @param bool $publishReview
     */
    public function setPublishReview($publishReview)
    {
        $this->publishReview = $publishReview;
    }

    /**
     * @return string
     */
    public function getFurtherInformation()
    {
        return $this->furtherInformation;
    }

    /**
     * @param string $furtherInformation
     */
    public function setFurtherInformation($furtherInformation)
    {
        $this->furtherInformation = (string)$furtherInformation;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

}