<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application;

use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Core\DateTimeTrait;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Work;

/**
 * @ODM\EmbeddedDocument
 */
class ApplicationSnapshot
{
    use DateTimeTrait;

    /**
     * @var ArrayCollection|Exposition[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition", cascade={"all"}, storeAs="dbRef")
     */
    protected $expositions;

    /**
     * @var ArrayCollection|Work[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Work", cascade={"all"}, storeAs="dbRef")
     */
    protected $works;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $date;

    /**
     * @var bool
     */
    private $new = false;

    public function __construct()
    {
        $this->expositions = new ArrayCollection();
        $this->works = new ArrayCollection();
        $this->date = new DateTime();

        $this->new = true;
    }

    /**
     * @return Exposition[]
     */
    public function getExpositions()
    {
        return $this->expositions->toArray();
    }

    /**
     * @param Exposition[] $expositions
     */
    public function setExpositions(array $expositions)
    {
        $this->expositions = new ArrayCollection($expositions);
    }

    /**
     * @param Exposition $exposition
     */
    public function addExposition(Exposition $exposition)
    {
        $this->expositions->add($exposition);
    }

    /**
     * @return Work[]
     */
    public function getWorks()
    {
        return $this->works->toArray();
    }

    /**
     * @param Work[] $works
     */
    public function setWorks(array $works)
    {
        $this->works = new ArrayCollection($works);
    }

    /**
     * @param Work $work
     */
    public function addWork(Work $work)
    {
        $this->works->add($work);
    }

    /**
     * @return Research[]
     */
    public function getResearches()
    {
        return array_merge($this->expositions->toArray(), $this->works->toArray());
    }

    /**
     * @param Research $research
     */
    public function addResearch(Research $research)
    {
        if ($research instanceof Exposition) {
            $this->addExposition($research);
        } elseif ($research instanceof Work) {
            $this->addWork($research);
        } else {
            throw new \InvalidArgumentException('Only ' . Exposition::class . ' and ' . Work::class . ' are supported, ' . get_class($research) . ' given!');
        }
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getDate($format = true)
    {
        return $this->formatDateTime($this->date, $format);
    }

    /**
     * @param string $date
     * @return void
     */
    public function setDate($date)
    {
        $this->date = $this->castToDateTime($date);
    }

    /**
     * @return bool
     */
    public function isNew()
    {
        return $this->new;
    }
}
