<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application\Generic;

use Doctrine\Common\Collections\ArrayCollection;
use Rc\Domain\Model\Application\ApplicationForm;
use Rc\Domain\Model\Application\Form\FormElement;
use Rc\Domain\Model\Application\Form\FormElement\CheckboxFormElement;
use Rc\Domain\Model\Application\Form\FormElement\CheckboxListFormElement;
use Rc\Domain\Model\Application\Form\FormElementValueContainer;
use Rc\Domain\Model\Application\Form\LayoutElement;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\MediaContainer;

trait MetaDataAwareTrait
{

    /**
     * @var ArrayCollection|FormElementValueContainer[]
     * @ODM\EmbedMany(targetDocument="Rc\Domain\Model\Application\Form\FormElementValueContainer", strategy="set")
     */
    protected $metaData;

    /**
     * @param bool $asArray
     * @return FormElementValueContainer[]|array
     */
    public function getMetaData($asArray = true)
    {
        if (!$asArray) {
            return $this->metaData->toArray();
        }

        $metaData = [];
        foreach ($this->metaData as $key => $container) {
            $metaData[$key] = $container->getValue();
        }
        return $metaData;
    }

    /**
     * @param string $keyIdentifier
     * @param ApplicationForm $form
     * @return array
     */
    public function getFormattedMetaData($keyIdentifier = MetaDataAwareInterface::KEY_IDENTIFIER_DEFAULT, ApplicationForm $form = null)
    {
        $metaData = [];
        foreach ($this->metaData as $key => $container) {
            $element = $this->getProgramElement($key);
            if (!isset($element)) {
                continue;
            }

            if ($keyIdentifier == MetaDataAwareInterface::KEY_IDENTIFIER_LABEL) {
                $key = $container->getElement()->getLabel();
            }

            if ($container->getValue() instanceof MediaInterface) {
                $value = new MediaContainer($container->getValue());
            } elseif ($container->getElement() instanceof CheckboxFormElement) {
                $value = $container->getValue() ? 'yes' : 'no';
            } elseif ($container->getElement() instanceof CheckboxListFormElement) {
                $value = [];
                foreach ($container->getValue() as $label) {
                    $value[] = $label . ': yes';
                }
            } else {
                $value = $container->getValue();
            }

            $metaData[$key] = $value;
        }

        if (isset($form)) {
            foreach ($form->getElements() as $key => $element) {
                if (!$element instanceof LayoutElement) {
                    continue;
                }

                $metaData[$key] = print_r($element->getFormOptions(), true);
            }
        }

        return $metaData;
    }

    /**
     * @param ApplicationForm $form
     * @return array
     */
    public function getFormattedMetaDataWithLayout(ApplicationForm $form)
    {
        $metaData = [];
        foreach ($form->getElements() as $key => $element) {
            if ($element instanceof LayoutElement) {
                $metaData[count($metaData)] = $element->getFormOptions()['value'];
            } elseif (isset($this->metaData[$key])) {
                $metaData[$this->metaData[$key]->getElement()->getLabel()] = null;
            }
        }

        $metaData = array_merge($metaData, $this->getFormattedMetaData(MetaDataAwareInterface::KEY_IDENTIFIER_LABEL));

        return $metaData;
    }

    /**
     * @param array $metaData
     * @param bool $preserveFiles
     */
    public function setMetaData(array $metaData, $preserveFiles = true)
    {
        foreach ($metaData as $identifier => $value) {
            $element = $this->getProgramElement($identifier);
            if (!isset($element) || $value === '') {
                unset($metaData[$identifier]);
                continue;
            }

            if ($value instanceof MediaInterface) {
                $value->setEmbedded(true);
            }

            $metaData[$identifier] = new FormElementValueContainer($element, $value);
        }

        if ($preserveFiles) {
            foreach ($this->metaData as $key => $container) {
                if ($container->getElement()->requiresUpload() && !isset($metaData[$key])) {
                    $metaData[$key] = $container;
                }
            }
        }

        $this->metaData = new ArrayCollection($metaData);
    }

    /**
     * @param string $identifier
     * @return null|string
     */
    public function getLabelForMetaDataIdentifier($identifier)
    {
        if (!isset($this->metaData[$identifier])) {
            return null;
        }

        return $this->metaData[$identifier]->getElement()->getLabel();
    }

    /**
     * @param string $identifier
     * @return FormElement|null
     */
    protected function getProgramElement($identifier)
    {
        foreach ($this->getMetadataForm()->getFormElements() as $element) {
            if ($element->getIdentifier() == $identifier) {
                return $element;
            }
        }
        return null;
    }

    /**
     * @return ApplicationForm $form
     */
    abstract protected function getMetadataForm();
}
