<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\Dimension;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Media\Resource;
use Rc\Domain\Model\Media\Resource as MediaResource;
use Rc\Domain\Model\Media\Resource\AudioResource;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\Media\Resource\VideoResource;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\DimensionAwareInterface;
use Rc\Service\Ingest\ImageIngestService;

/**
 * @ODM\Document
 * @ODM\Index(name="checksum", unique=true, keys={"master.checkSum"="text"})
 */
class Media extends AbstractModel implements MediaInterface
{
    /**
     * @var int
     */
    protected $id = false;

    /**
     * @var MediaResource
     * @ODM\EmbedOne()
     */
    protected $master;

    /**
     * @var ArrayCollection|Resource[]
     * @ODM\EmbedMany(strategy="set")
     */
    protected $derivativeList;

    /**
     * @var ImageResource
     * @ODM\EmbedOne()
     */
    protected $splashScreen;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $contentType;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $expires;

    /**
     * @var boolean
     */
    protected $embedded;

    public function __construct()
    {
        parent::__construct();
        $this->derivativeList = new ArrayCollection();
    }

    /**
     * @return boolean
     */
    public function hasMaster()
    {
        return isset($this->master);
    }

    /**
     * @return MediaResource
     */
    public function getMaster()
    {
        if (isset($this->master)) {
            return $this->master;
        }

        $derivative = $this->getDerivative(VideoResource::DERIVATIVE_TYPE_MP4);
        if (isset($derivative)) {
            return $derivative;
        }

        $derivative = $this->getDerivative(AudioResource::DERIVATIVE_TYPE_MP3);
        if (isset($derivative)) {
            return $derivative;
        }

        return null;
    }

    /**
     * @param MediaResource $master
     */
    public function setMaster(MediaResource $master)
    {
        $this->master = $master;
        $this->contentType = $master->getContentType();
    }

    /**
     * @return MediaResource
     */
    public function getPrimaryDerivative()
    {
        $derivative = $this->getDerivative(VideoResource::DERIVATIVE_TYPE_MP4);
        if (isset($derivative)) {
            return $derivative;
        }

        $derivative = $this->getDerivative(AudioResource::DERIVATIVE_TYPE_MP3);
        if (isset($derivative)) {
            return $derivative;
        }

        return $this->master;
    }

    /**
     * @inheritDoc
     */
    public function getUri($relative = false)
    {
        return $this->master->getUri($relative);
    }

    /**
     * @inheritDoc
     */
    public function getExternalUri($relative = false)
    {
        return $this->master->getExternalUri($relative);
    }

    /**
     * @return boolean
     */
    public function hasSplashScreen()
    {
        return isset($this->splashScreen);
    }

    /**
     * @param int $width
     * @param int $height
     * @return ImageResource|null
     */
    public function getBestFittingSplashScreen($width, $height)
    {
        if (!$this->hasSplashScreen()) {
            return null;
        }

        return ImageIngestService::loadCachedResource($this->getSplashScreen(), new Dimension($width, $height), true);
    }

    /**
     * @return ImageResource
     */
    public function getSplashScreen()
    {
        return $this->splashScreen;
    }

    /**
     * @param ImageResource|null $splashScreen
     */
    public function setSplashScreen($splashScreen = null)
    {
        $this->splashScreen = $splashScreen;
    }

    /**
     * @return Dimension|null
     */
    public function getResolution()
    {
        foreach (array_merge([$this->master], $this->getDerivativeList()) as $resource) {
            if (!isset($resource) || !$resource instanceof DimensionAwareInterface) {
                continue;
            }
            if (!$resource->getHeight()) {
                continue;
            }
            return new Dimension($resource->getWidth(), $resource->getHeight());
        }
        return null;
    }

    /**
     * @return bool
     */
    public function hasResolution()
    {
        return $this->getResolution() !== null;
    }

    /**
     * @return MediaResource[]
     */
    public function getDerivativeList()
    {
        return $this->derivativeList->toArray();
    }

    /**
     * @param MediaResource[] $derivativeList
     */
    public function setDerivativeList(array $derivativeList)
    {
        $this->derivativeList->clear();
        foreach ($derivativeList as $item) {
            $this->addDerivative($item);
        }
    }

    /**
     *
     */
    public function clearDerivativeList()
    {
        $this->setDerivativeList([]);
    }

    /**
     * @return bool
     */
    public function hasDerivatives()
    {
        return !$this->derivativeList->isEmpty();
    }

    /**
     * @param string $type
     * @return MediaResource|null
     */
    public function getDerivativeByType($type)
    {
        return $this->getDerivative($type);
    }

    /**
     * @param string $type
     * @return MediaResource|null
     */
    public function getDerivative($type)
    {
        return $this->derivativeList->offsetGet($type);
    }

    /**
     * @return MediaResource|null
     */
    public function getDefaultDerivative()
    {
        if ($this->isImage() || $this->isPdf()) {
            return $this->getSplashScreen();
        } elseif ($this->isVideo()) {
            return $this->getDerivative(VideoResource::DERIVATIVE_TYPE_MP4);
        } elseif ($this->isAudio()) {
            return $this->getDerivative(AudioResource::DERIVATIVE_TYPE_MP3);
        } else {
            return null;
        }
    }

    /**
     * @param string $type
     * @return bool
     */
    public function hasDerivative($type)
    {
        return !empty($this->getDerivative($type));
    }

    /**
     * @param MediaResource $derivative
     */
    public function addDerivative(MediaResource $derivative)
    {
        $this->derivativeList->offsetSet($derivative->getFileExtension(), $derivative);
    }

    /**
     * @return MediaResource[]
     */
    public function getResourceList()
    {
        $retVal = $this->derivativeList->toArray();
        $retVal[] = $this->splashScreen;
        $retVal[] = $this->master;
        return array_reverse(array_filter($retVal));
    }

    /**
     * @return string|null
     */
    public function getContentType()
    {
        return $this->contentType;
    }

    /**
     * @internal
     * @param string $type
     */
    public function setContentType($type)
    {
        $this->contentType = $type;
    }

    /**
     * @return bool
     */
    public function isImage()
    {
        return $this->getContentType() == MediaType::IMAGE;
    }

    /**
     * @return bool
     */
    public function isVideo()
    {
        return $this->getContentType() == MediaType::VIDEO;
    }

    /**
     * @return bool
     */
    public function isAudio()
    {
        return $this->getContentType() == MediaType::AUDIO;
    }

    /**
     * @return bool
     */
    public function isPdf()
    {
        return $this->getContentType() == MediaType::PDF;
    }

    /**
     * @return bool
     */
    public function isArchive()
    {
        return $this->getContentType() == MediaType::ARCHIVE;
    }

    /**
     * @return \DateTime
     */
    public function getExpires()
    {
        return $this->expires;
    }

    /**
     * @param \DateTime $expires
     */
    public function setExpires($expires)
    {
        $this->expires = $expires;
    }

    /**
     * @return bool
     */
    public function isEmbedded()
    {
        return $this->embedded;
    }

    /**
     * @param bool $embedded
     */
    public function setEmbedded($embedded)
    {
        $this->embedded = $embedded;
    }

    /**
     * Fix duplicate key exception
     * @return void
     */
    public function __clone()
    {
        if ($this->embedded) {
            return;
        }

        parent::__clone();
    }

}