<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Media;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\Resource\Type\RoleType;
use Rc\Service\Ingest\MediaTypes;
use Rc\Utility\TokenGenerator;

/**
 * @ODM\EmbeddedDocument
 * @ODM\InheritanceType("SINGLE_COLLECTION")
 */
class Resource
{

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $path;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $fileName;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $fileSize;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $contentType;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $mimeType;

    /**
     * @var RoleType
     * @ODM\Field(type="string")
     */
    protected $role;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $pathSuffix;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $checkSum;

    /**
     * @return bool
     */
    public function isImage()
    {
        return $this->contentType == MediaType::IMAGE;
    }

    /**
     * @return bool
     */
    public function isVideo()
    {
        return $this->contentType == MediaType::VIDEO;
    }

    /**
     * @return bool
     */
    public function isAudio()
    {
        return $this->contentType == MediaType::AUDIO;
    }

    /**
     * @return bool
     */
    public function isPdf()
    {
        return $this->contentType == MediaType::PDF;
    }

    /**
     * @return string
     */
    public function getFileExtension()
    {
        return MediaTypes::getFilenameExtensionFromMediaType($this->mimeType);
    }

    /**
     * @return string
     */
    public function getPath()
    {
        return $this->path;
    }

    /**
     * @param string $path
     */
    public function setPath($path)
    {
        $this->path = (string)$path;
    }

    /**
     * @return string
     */
    public function getFileName()
    {
        return $this->fileName;
    }

    /**
     * @param string $fileName
     */
    public function setFileName($fileName)
    {
        $this->fileName = $fileName;
    }

    /**
     * @return int
     */
    public function getFileSize()
    {
        return $this->fileSize;
    }

    /**
     * @param int $fileSize
     */
    public function setFileSize($fileSize)
    {
        $this->fileSize = $fileSize;
    }

    /**
     * @return string
     */
    public function getContentType()
    {
        return $this->contentType;
    }

    /**
     * @param string $contentType
     */
    public function setContentType($contentType)
    {
        $this->contentType = $contentType;
    }

    /**
     * @return string
     */
    public function getMimeType()
    {
        return $this->mimeType;
    }

    /**
     * @param string $mimeType
     */
    public function setMimeType($mimeType)
    {
        $this->mimeType = $mimeType;
    }

    /**
     * @return RoleType
     */
    public function getRole()
    {
        return $this->role;
    }

    /**
     * @param RoleType $role
     */
    public function setRole($role)
    {
        $this->role = $role;
    }

    /**
     * @return string
     */
    public function getCheckSum()
    {
        return $this->checkSum;
    }

    /**
     * @param string $checkSum
     */
    public function setCheckSum($checkSum)
    {
        $this->checkSum = $checkSum;
    }

    /**
     * @return string
     */
    public function getPathSuffix()
    {
        return $this->pathSuffix;
    }

    /**
     * @param string $pathSuffix
     */
    public function setPathSuffix($pathSuffix)
    {
        $this->pathSuffix = trim((string)$pathSuffix, '/') . '/';
    }

    /**
     * @param bool $relative
     * @return string
     */
    public function getUri($relative = false)
    {
        $uri = $this->getPathSuffix() . $this->getPrefix() . '/' . $this->getFileName();

        if (!$relative) {
            $uri = ($this->path ?: config('media.storage_path')) . '/' . $uri;
        }

        return $uri;
    }

    /**
     * @param bool $relative
     * @return string
     */
    public function getExternalUri($relative = false)
    {
        $uri = $this->getPathSuffix() . $this->getPrefix() . '/' . $this->getFileName();

        if (!$relative) {
            $time = round(time() / 300) * 300 + 86400;
            $uri = config('media.url') . '/' . $uri . '?t=' . TokenGenerator::generate('/' . $uri . $time) . '&e=' . $time;
        }

        return $uri;
    }

    /**
     * @return string
     */
    protected function getPrefix()
    {
        return substr($this->getFileName(), 0, 4);
    }
}