<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use NewsletterRepository;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Zend_Registry;

/**
 * @ODM\Document
 * @ODM\Index(keys={"sender"="asc", "deletedForSender"="asc"}),
 * @ODM\Index(keys={"receiver"="asc", "deletedForReceiver"="asc"})
 */
class Message extends AbstractModel
{
    const TYPE_COLLABORATION_ADMIN_ADD = 'collaboration.admin.add';
    const TYPE_COLLABORATION_ADMIN_REMOVE = 'collaboration.admin.remove';
    const TYPE_COLLABORATION_ADD = 'collaboration.add';
    const TYPE_COLLABORATION_ADD_EDITOR = 'collaboration.add.editor';
    const TYPE_COLLABORATION_REMOVE = 'collaboration.remove';
    const TYPE_COLLABORATION_DISAPPROVE = 'collaboration.disapprove';
    const TYPE_COLLABORATION_CANCEL = 'collaboration.cancel';
    const TYPE_COLLABORATION_ACCEPT = 'collaboration.accept';
    const TYPE_COLLABORATION_DECLINE = 'collaboration.decline';
    const TYPE_COLLABORATION_LEAVE = 'collaboration.leave';

    const TYPE_PORTAL_ADD = 'portal.add';
    const TYPE_SUBMIT_TO_PORTAL = 'submit.to.portal';
    const TYPE_SUBMISSION_ACCEPTED = 'submission.accepted';
    const TYPE_SUBMISSION_DECLINED = 'submission.declined';
    const TYPE_PARTOF_PORTAL_REQUEST = 'partof.request';
    const TYPE_PARTOF_ACCEPTED = 'partof.accepted';
    const TYPE_PARTOF_DECLINED = 'partof.declined';

    const TYPE_MEDIA_TRANSCODED = 'media.transcoded';

    const TYPE_RESEARCH_TO_REVIEW = 'review.new.research';

    const TYPE_APPLICATION_TO_REVIEW = 'review.new.application';
    const TYPE_APPLICATION_ACCEPTED = 'application.accepted';
    const TYPE_APPLICATION_DECLINED = 'application.declined';
    const TYPE_APPLICATION_REWORK = 'application.rework';
    const TYPE_APPLICATION_FORWARDED = 'application.forwarded';
    const TYPE_APPLICATION_SUBMITTED = 'application.submitted';
    const TYPE_APPLICATION_REVIEW_MAIL = 'application.reviewmail';

    const TYPE_NEWSLETTER_TO_APPROVE = 'approve.new.newsletter';

    const TYPE_TASK_ADD = 'task.add';

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $sender;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $receiver;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $type;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $subject;

    /**
     * @var string
     * @ODM\Field(type="raw")
     */
    protected $content;

    /**
     * @var MediaInterface[]|ArrayCollection
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Media", strategy="set", cascade={"persist"}, storeAs="dbRef")
     */
    protected $attachments;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $deletedForSender;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $deletedForReceiver;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $read = false;

    public function __construct()
    {
        parent::__construct();
        $this->attachments = new ArrayCollection();
    }

    /**
     * @param int $userId
     */
    public function setSenderId($userId)
    {
        $this->sender = $this->createReference(User::class, $userId);
    }

    /**
     * @param int $userId
     */
    public function setReceiverId($userId)
    {
        $this->receiver = $this->createReference(User::class, $userId);
    }

    /**
     * @return User
     */
    public function getSender()
    {
        return $this->sender;
    }

    /**
     * @param User $sender
     */
    public function setSender($sender)
    {
        $this->sender = $sender;
    }

    /**
     * @return User
     */
    public function getReceiver()
    {
        return $this->receiver;
    }

    /**
     * @param User $receiver
     */
    public function setReceiver($receiver)
    {
        $this->receiver = $receiver;
    }

    /**
     * @return int
     */
    public function getDeleted()
    {
        return $this->deletedForSender;
    }

    /**
     * @param int $deletedForSender
     */
    public function setDeletedForSender($deletedForSender)
    {
        $this->deletedForSender = $deletedForSender;
    }

    /**
     * @return int
     */
    public function getDeletedForReceiver()
    {
        return $this->deletedForReceiver;
    }

    /**
     * @param int $deletedForReceiver
     */
    public function setDeletedForReceiver($deletedForReceiver)
    {
        $this->deletedForReceiver = $deletedForReceiver;
    }

    /**
     * @return boolean
     */
    public function getRead()
    {
        return $this->read;
    }

    /**
     * @return boolean
     */
    public function isRead()
    {
        return $this->read;
    }

    /**
     * @param boolean $read
     */
    public function setRead($read)
    {
        $this->read = $read;
    }

    /**
     * @return string
     */
    public function getSubject()
    {
        if ($this->getType() === self::TYPE_COLLABORATION_ADD) {
            $data = json_decode($this->content, true);
            $type = $data['type'];

            $retVal = $this->_translate('message.subject.' . $this->getType() . '.' . $type);
        } elseif ($this->getType() == self::TYPE_APPLICATION_REVIEW_MAIL) {
            $data = json_decode($this->content, true);

            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($data['entity']);

            /** @var Application $entity */
            $entity = MongoRepository::findById($data['entity']);
            $retVal = sprintf($this->_translate('message.subject.' . $this->getType()), $entity->getTitle(), $entity->getAuthor()->getName());
        } elseif ($this->getType()) {
            $retVal = $this->_translate('message.subject.' . $this->getType());
        } else {
            $retVal = $this->subject;
        }

        return $retVal;
    }

    /**
     * @param string $subject
     */
    public function setSubject($subject)
    {
        $this->subject = $subject;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getContent()
    {
        switch ($this->getType()) {
            case self::TYPE_COLLABORATION_ADMIN_ADD:
            case self::TYPE_COLLABORATION_ADMIN_REMOVE:
            case self::TYPE_COLLABORATION_ADD:
            case self::TYPE_COLLABORATION_REMOVE:
            case self::TYPE_COLLABORATION_DISAPPROVE:
            case self::TYPE_COLLABORATION_CANCEL:
            case self::TYPE_COLLABORATION_ACCEPT:
            case self::TYPE_COLLABORATION_DECLINE:
            case self::TYPE_COLLABORATION_LEAVE:
                $retVal = $this->_parseCollaborationContent();
                break;
            case self::TYPE_MEDIA_TRANSCODED:
                $retVal = $this->_parseTranscodedMediaContent();
                break;
            case self::TYPE_PARTOF_PORTAL_REQUEST:
            case self::TYPE_SUBMIT_TO_PORTAL:
                $retVal = $this->_parseSubmitToPortalContent();
                break;
            case self::TYPE_PARTOF_ACCEPTED:
            case self::TYPE_PARTOF_DECLINED:
            case self::TYPE_SUBMISSION_ACCEPTED:
            case self::TYPE_SUBMISSION_DECLINED:
            case self::TYPE_APPLICATION_ACCEPTED:
            case self::TYPE_APPLICATION_DECLINED:
                $retVal = $this->_parseSubmissionContent();
                break;
            case self::TYPE_RESEARCH_TO_REVIEW:
            case self::TYPE_APPLICATION_TO_REVIEW:
                $retVal = $this->_parseReviewContent();
                break;
            case self::TYPE_APPLICATION_SUBMITTED:
                $retVal = $this->_parseSubmittedContent();
                break;
            case self::TYPE_NEWSLETTER_TO_APPROVE:
                $retVal = $this->_parseApproveNewsletterContent();
                break;
            case self::TYPE_TASK_ADD:
                $retVal = $this->_parseTaskContent();
                break;
            case self::TYPE_PORTAL_ADD:
                $retVal = $this->_parsePortalContent();
                break;
            case self::TYPE_APPLICATION_REVIEW_MAIL:
                $retVal = null;
                break;
            default:
                $retVal = $this->content;
        }

        return $retVal;
    }

    /**
     * @param string|array $content
     */
    public function setContent($content)
    {
        if (is_array($content)) {
            $content = json_encode($content);
        }

        $this->content = $content;
    }

    /**
     * @return ArrayCollection|Media[]
     */
    public function getAttachments()
    {
        return $this->attachments;
    }

    /**
     * @param ArrayCollection|Media[] $attachments
     */
    public function setAttachments($attachments)
    {
        $this->attachments = $attachments;
    }

    /**
     * @param Media $attachment
     * @param string $name
     */
    public function addAttachment(Media $attachment, $name = null)
    {
        if (isset($name)) {
            $this->attachments->offsetSet(str_replace('.', '_', $name), $attachment);
        } else {
            $this->attachments->add($attachment);
        }
    }

    /**
     * @return int
     */
    public function getSenderId()
    {
        return $this->sender->getId();
    }

    /**
     * @return int
     */
    public function getReceiverId()
    {
        return $this->receiver->getId();
    }

    /**
     * @param string $key
     * @return string
     */
    protected function _translate($key)
    {
        return Zend_Registry::get('Zend_Translate')->translate($key);
    }

    /**
     * @return string
     */
    protected function _parseCollaborationContent()
    {
        $data = json_decode($this->content, true);

        if (isset($data['research'])) {
            $data['entity'] = $data['research'];
            $data['entity_type'] = 'exposition';
        }

        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($data['entity']);

        $sender = UserRepository::findById($this->getSenderId());
        $receiver = UserRepository::findById($this->getReceiverId());
        $entity = ResearchRepository::findById($data['entity']);

        if (!isset($entity)) {
            return 'This invitation is deprecated.';
        }

        $sprintfParams = [
            $receiver->getName(),
            $this->_entityUrl($sender),
            strtolower($data['entity_type']),
            $entity->getTitle(),
            $entity->getId(),
            $data['type'],
        ];

        $translationKey = 'message.content.' . $this->getType();
        $content = $this->_translate($translationKey . '.' . $data['type']);
        if ($content === $translationKey . '.' . $data['type']) {
            $content = $this->_translate($translationKey);
        }

        return vsprintf($content, $sprintfParams);
    }

    /**
     * @return string
     */
    protected function _parseTranscodedMediaContent()
    {
        $data = json_decode($this->content, true);
        $entity = ExpositionRepository::findById($data['entity']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            isset($entity) ? $entity->getTitle() : '<i>Unknown</i>',
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parseSubmitToPortalContent()
    {
        $data = json_decode($this->content, true);

        $portal = PortalRepository::findById($data['portal']);
        if (isset($data['research'])) {
            $data['entity'] = $data['research'];
            $data['type'] = 'exposition';
        }

        $receiver = UserRepository::findById($this->getReceiverId());
        $sender = UserRepository::findById(isset($data['user']) ? $data['user'] : $this->getSenderId());

        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($data['entity']);
        $entity = ResearchRepository::findById($data['entity']);

        $sprintfParams = [
            $receiver->getName(),
            $this->_entityUrl($sender),
            strtolower($data['type']),
            $entity->getTitle(),
            $portal->getName(),
            $data['type'] === 'application' ? 'application' : 'reviewing',
            $data['portal'],
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parseSubmissionContent()
    {
        $data = json_decode($this->content, true);

        $portal = PortalRepository::findById($data['portal']);
        if (isset($data['research'])) {
            $data['entity'] = $data['research'];
            $data['type'] = 'exposition';
        }
        $entity = ResearchRepository::findById($data['entity']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            $portal->getName(),
            strtolower($data['type']),
            $entity->getTitle(),
        ];

        $content = vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
        if (!empty($data['content'])) {
            $content .= '<br/><br/>' . sprintf($this->_translate('info.additional.notes'), $data['content']);
        }

        return $content;
    }

    protected function _parseSubmittedContent()
    {
        $data = json_decode($this->content, true);

        $entity = ApplicationRepository::findById($data['application']);
        $portal = PortalRepository::findById($data['portal']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            $entity->getTitle(),
            $portal->getName(),
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parseReviewContent()
    {
        $data = json_decode($this->content, true);

        $portal = PortalRepository::findById($data['portal']);
        $entity = ResearchRepository::findById(isset($data['research']) ? $data['research'] : $data['entity']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            isset($entity) ? $entity->getTitle() : 'Unknown',
            $portal->getName(),
            isset($entity) ? $entity->getId() : null,
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parseApproveNewsletterContent()
    {
        $data = json_decode($this->content, true);

        $portal = PortalRepository::findById($data['portal']);
        $newsletter = NewsletterRepository::findById($data['newsletter']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            $newsletter->getSubject(),
            $portal->getName(),
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parsePortalContent()
    {
        $data = json_decode($this->content, true);

        $user = UserRepository::findById($data['user']);
        $portal = PortalRepository::findById($data['portal']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            $this->_entityUrl($user),
            $this->_entityUrl($portal),
            $user->getId(),
            $portal->getId(),
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    protected function _parseTaskContent()
    {
        $data = json_decode($this->content, true);

        $research = ExpositionRepository::findById($data['research']);
        $user = UserRepository::findById($data['user']);
        $receiver = UserRepository::findById($this->getReceiverId());

        $sprintfParams = [
            $receiver->getName(),
            $this->_entityUrl($user),
            $research->getId(),
            $data['weave'],
            $research->getTitle(),
            $data['deadline'],
            $data['content'],
        ];

        return vsprintf($this->_translate('message.content.' . $this->getType()), $sprintfParams);
    }

    /**
     * @param User|Portal|Application $entity
     * @return string
     */
    protected function _entityUrl($entity)
    {
        return '<a href="' . (new \My_View_Helper_Url())->entityUrl($entity) . '">' . $entity->getTitle() . '</a>';
    }
}
