<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Core\Acl\AclGroup;
use Rc\Domain\Model\Core\Acl\AclHelper;
use Rc\Domain\Model\Media\MediaAwareInterface;
use Rc\Domain\Model\Media\MediaAwareTrait;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research\DoiAwareInterface;
use Rc\Domain\Model\Research\DoiAwareTrait;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Repository\PortalRepository;

/**
 * @ODM\Document
 */
class Portal extends AbstractModel implements MediaAwareInterface, DoiAwareInterface
{
    use MediaAwareTrait;
    use DoiAwareTrait;

    const FEATURE_MEMBERSHIP_SELECTION = 'portalMembershipSelection';
    const FEATURE_EXPOSITION_DOI_DEPOSIT = 'expositionDoiDeposit';
    const FEATURE_EXPOSITION_CONNECTING = 'expositionConnecting';
    const FEATURE_EXPOSITION_PUBLISHING = 'expositionPublishing';
    const FEATURE_LIMITED_EXPOSITION_PUBLISHING = 'limitedExpositionPublishing';
    const FEATURE_PROJECT_CONNECTING = 'projectConnecting';
    const FEATURE_DEGREE_CONNECTING = 'degreeConnecting';
    const FEATURE_DEGREE_PUBLISHING = 'degreePublishing';
    const FEATURE_WORK_CONNECTING = 'workConnecting';
    const FEATURE_APPLICATION_PUBLISHING = 'applicationPublishing';
    const FEATURE_PROFILE = 'profile';
    const FEATURE_LINK_EXTERNAL = 'linkExternal';
    const FEATURE_APPLICATION_PROGRAM = 'applicationProgram';
    const FEATURE_EXTERNAL_CONTENT = 'externalContent';

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $name;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $shortName;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $doiAbbreviation;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $issn;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $url;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $default;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", cascade={"persist"}, storeAs="dbRef")
     */
    protected $admins;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", cascade={"persist"}, storeAs="dbRef")
     */
    protected $reviewers;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $contactPersons;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", mappedBy="portals", storeAs="dbRef")
     */
    protected $members;

    /**
     * @var ArrayCollection|Issue[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal\Issue", mappedBy="portal")
     */
    protected $issues;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $externalContactPersons;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $description;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $longDescription;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $viewText;

    /**
     * @var array
     * @ODM\Field(type="collection")
     */
    protected $issueList = [];

    /**
     * @var Exposition[]|ArrayCollection
     * @ODM\ReferenceMany(targetDocument="\Rc\Domain\Model\Research\Exposition")
     */
    protected $textExpositionTemplates;

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $features = [
        self::FEATURE_EXPOSITION_CONNECTING         => false,
        self::FEATURE_EXPOSITION_DOI_DEPOSIT        => false,
        self::FEATURE_EXPOSITION_PUBLISHING         => false,
        self::FEATURE_LIMITED_EXPOSITION_PUBLISHING => false,
        self::FEATURE_PROJECT_CONNECTING            => false,
        self::FEATURE_DEGREE_CONNECTING             => false,
        self::FEATURE_DEGREE_PUBLISHING             => false,
        self::FEATURE_WORK_CONNECTING               => false,
        self::FEATURE_APPLICATION_PUBLISHING        => false,
        self::FEATURE_PROFILE                       => false,
        self::FEATURE_LINK_EXTERNAL                 => false,
        self::FEATURE_APPLICATION_PROGRAM           => false,
        self::FEATURE_MEMBERSHIP_SELECTION          => false,
        self::FEATURE_EXTERNAL_CONTENT              => false,
    ];

    public function __construct()
    {
        parent::__construct();
        $this->admins = new ArrayCollection();
        $this->reviewers = new ArrayCollection();
        $this->contactPersons = new ArrayCollection();
        $this->members = new ArrayCollection();
        $this->mediaList = new ArrayCollection();
        $this->textExpositionTemplates = new ArrayCollection();
        $this->issues = new ArrayCollection();
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->getName();
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return $this
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     * @return $this
     */
    public function setDescription($description)
    {
        $this->description = $description;
        return $this;
    }

    /**
     * @return string
     */
    public function getShortName()
    {
        return $this->shortName;
    }

    /**
     * @param string $shortName
     * @return $this
     */
    public function setShortName($shortName)
    {
        $this->shortName = $shortName;
        return $this;
    }

    /**
     * @return string
     */
    public function getDoiAbbreviation()
    {
        return $this->doiAbbreviation;
    }

    /**
     * @param string $doiAbbreviation
     */
    public function setDoiAbbreviation($doiAbbreviation)
    {
        $this->doiAbbreviation = $doiAbbreviation;
    }

    /**
     * @return string
     */
    public function getIssn()
    {
        return $this->issn;
    }

    /**
     * @param string $issn
     */
    public function setIssn($issn)
    {
        $this->issn = $issn;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @param string $url
     * @return $this
     */
    public function setUrl($url)
    {
        $this->url = $url;
        return $this;
    }

    /**
     * @return bool
     */
    public function isDefault()
    {
        return $this->default;
    }

    /**
     * @param bool $default
     * @return $this
     */
    public function setDefault($default)
    {
        $this->default = $default;
        return $this;
    }

    /**
     * @return string
     */
    public function getLongDescription()
    {
        return $this->longDescription;
    }

    /**
     * @param string $longDescription
     * @return $this
     */
    public function setLongDescription($longDescription)
    {
        $this->longDescription = $longDescription;
        return $this;
    }

    /**
     * @return string
     */
    public function getViewText()
    {
        return $this->viewText;
    }

    /**
     * @param string $viewText
     * @return $this
     */
    public function setViewText($viewText)
    {
        $this->viewText = $viewText;
        return $this;
    }

    /**
     * @return User[]
     */
    public function getMembers()
    {
        return $this->members;
    }

    /**
     * @param \Rc\Domain\Model\User $user
     * @return bool
     */
    public function isMember(User $user)
    {
        return $this->members->contains($user);
    }

    /**
     * @return int
     */
    public function getMemberCount()
    {
        return $this->members->count();
    }

    /**
     * @return User[]
     */
    public function getAdmins()
    {
        return $this->admins->toArray();
    }

    /**
     * @return array
     */
    public function getAdminIds()
    {
        return $this->createIdList($this->admins);
    }

    /**
     * @param array $userList
     */
    public function setAdminIds(array $userList)
    {
        $oldAdminIds = $this->getAdminIds();

        AclHelper::addUserGroup(AclGroup::PORTAL_ADMIN($this->getId()), $userList);
        AclHelper::removeUserGroup(AclGroup::PORTAL_ADMIN($this->getId()), array_diff($oldAdminIds, $userList));

        $this->admins = new ArrayCollection($this->createReferenceList(User::class, $userList));
    }

    /**
     * @param \Rc\Domain\Model\User $user
     * @return bool
     */
    public function isAdmin(User $user)
    {
        return $this->admins->contains($user);
    }

    /**
     * @return string
     */
    public function getExternalContactPersons()
    {
        return $this->externalContactPersons;
    }

    /**
     * @param string $externalContactPersons
     */
    public function setExternalContactPersons($externalContactPersons)
    {
        $this->externalContactPersons = $externalContactPersons;
    }

    /**
     * @return bool
     */
    public function hasContactPersons()
    {
        return count($this->getContactPersonIds()) > 0 || count($this->getExternalContactPersonsArray()) > 0;
    }

    /**
     * @return array
     */
    public function getContactPersonIds()
    {
        return $this->createIdList($this->contactPersons);
    }

    /**
     * @return User[]
     */
    public function getContactPersons()
    {
        return $this->contactPersons->toArray();
    }

    /**
     * @param array $userList
     */
    public function setContactPersonIds(array $userList)
    {
        $this->contactPersons = new ArrayCollection($this->createReferenceList(User::class, $userList));
    }

    /**
     * @return array
     */
    public function getExternalContactPersonsArray()
    {
        $retVal = [];
        foreach (preg_split('~\n~i', $this->externalContactPersons . PHP_EOL) as $contactPerson) {
            if (preg_match('~(?<name>.*) (?<email><.*>)~i', $contactPerson, $matches) == 0) {
                continue;
            }
            $retVal[] = [
                'name'  => ucwords($matches['name']),
                'email' => $matches['email'],
            ];
        }
        return $retVal;
    }

    /**
     * @return User[]
     */
    public function getReviewers()
    {
        foreach ($this->reviewers as $reviewer) {
            try {
                $reviewer->getName();
            } catch (DocumentNotFoundException $exception) {
                trigger_error($exception->getMessage(), E_USER_WARNING);

                $this->reviewers->removeElement($reviewer);
                PortalRepository::persist($this);
            }
        }
        return $this->reviewers->toArray();
    }

    /**
     * @return array
     */
    public function getReviewerIds()
    {
        return $this->createIdList($this->reviewers);
    }

    /**
     * @param array $userList
     */
    public function setReviewerIds(array $userList)
    {
        $oldReviewerIds = $this->getReviewerIds();

        AclHelper::addUserGroup(AclGroup::PORTAL_REVIEWER($this->id), $userList);
        AclHelper::removeUserGroup(AclGroup::PORTAL_REVIEWER($this->id), array_diff($oldReviewerIds, $userList));

        $this->reviewers = new ArrayCollection($this->createReferenceList(User::class, $userList));
    }

    /**
     * @param array $userList
     */
    public function addReviewerIds(array $userList)
    {
        $this->setReviewerIds(array_merge($this->getReviewerIds(), $userList));
    }

    /**
     * @return Issue[]
     */
    public function getIssues()
    {
        return $this->issues->toArray();
    }

    /**
     * @param ArrayCollection|Issue[] $issues
     */
    public function setIssues($issues)
    {
        foreach ($issues as $issue) {
            $issue->setPortal($this);
        }
        $this->issues = new ArrayCollection($issues);
    }

    /**
     * @return Issue[]
     */
    public function getPublishedIssues()
    {
        $issues = [];
        foreach ($this->issues as $issue) {
            if (!$issue->isPublished()) {
                continue;
            }

            $issues[] = $issue;
        }

        return $issues;
    }

    /**
     * @return array
     * @deprecated
     */
    public function getIssueList()
    {
        return $this->issueList;
    }

    /**
     * @param array $issueList
     * @deprecated
     */
    public function setIssueList(array $issueList)
    {
        $this->issueList = array_unique($issueList);
    }

    /**
     * @return Exposition[]
     */
    public function getTextExpositionTemplates()
    {
        return $this->textExpositionTemplates->toArray();
    }

    /**
     * @param Exposition[] $expositions
     */
    public function setTextExpositionTemplates(array $expositions)
    {
        $this->textExpositionTemplates = new ArrayCollection($expositions);
    }

    /**
     * @return int[]
     */
    public function getTextExpositionTemplateIds()
    {
        return $this->createIdList($this->textExpositionTemplates);
    }

    /**
     * @param array $ids
     */
    public function setTextExpositionTemplateIds(array $ids)
    {
        $this->textExpositionTemplates = $this->createReferenceList(Exposition::class, $ids);
    }

    /**
     * @return bool
     */
    public function getAllowExpositionDoiDeposit()
    {
        return $this->getFeature(self::FEATURE_EXPOSITION_DOI_DEPOSIT);
    }

    /**
     * @param bool $value
     */
    public function setAllowExpositionDoiDeposit($value)
    {
        $this->setFeature(self::FEATURE_EXPOSITION_DOI_DEPOSIT, $value);
    }

    /**
     * @return bool
     */
    public function getAllowExpositionConnecting()
    {
        return $this->getFeature(self::FEATURE_EXPOSITION_CONNECTING);
    }

    /**
     * @param bool $value
     */
    public function setAllowExpositionConnecting($value)
    {
        $this->setFeature(self::FEATURE_EXPOSITION_CONNECTING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowProjectConnecting()
    {
        return $this->getFeature(self::FEATURE_PROJECT_CONNECTING);
    }

    /**
     * @param bool $value
     */
    public function setAllowProjectConnecting($value)
    {
        $this->setFeature(self::FEATURE_PROJECT_CONNECTING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowDegreeConnecting()
    {
        return $this->getFeature(self::FEATURE_DEGREE_CONNECTING);
    }

    /**
     * @param bool $value
     */
    public function setAllowDegreeConnecting($value)
    {
        $this->setFeature(self::FEATURE_DEGREE_CONNECTING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowWorkConnecting()
    {
        return $this->getFeature(self::FEATURE_WORK_CONNECTING);
    }

    /**
     * @param bool $value
     */
    public function setAllowWorkConnecting($value)
    {
        $this->setFeature(self::FEATURE_WORK_CONNECTING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowExpositionPublishing()
    {
        return $this->getFeature(self::FEATURE_EXPOSITION_PUBLISHING);
    }

    /**
     * @param bool $value
     */
    public function setAllowExpositionPublishing($value)
    {
        $this->setFeature(self::FEATURE_EXPOSITION_PUBLISHING, $value);
    }

    /**
     * @return boolean
     */
    public function getAllowLimitedExpositionPublishing()
    {
        return $this->getFeature(self::FEATURE_LIMITED_EXPOSITION_PUBLISHING);
    }

    /**
     * @param boolean $value
     * @return void
     */
    public function setAllowLimitedExpositionPublishing($value)
    {
        $this->setFeature(self::FEATURE_LIMITED_EXPOSITION_PUBLISHING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowDegreePublishing()
    {
        return $this->getFeature(self::FEATURE_DEGREE_PUBLISHING);
    }

    /**
     * @param bool $value
     */
    public function setAllowDegreePublishing($value)
    {
        $this->setFeature(self::FEATURE_DEGREE_PUBLISHING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowApplicationPublishing()
    {
        return $this->getFeature(self::FEATURE_APPLICATION_PUBLISHING);
    }

    /**
     * @param bool $value
     */
    public function setAllowApplicationPublishing($value)
    {
        $this->setFeature(self::FEATURE_APPLICATION_PUBLISHING, $value);
    }

    /**
     * @return bool
     */
    public function getAllowProfile()
    {
        return $this->getFeature(self::FEATURE_PROFILE);
    }

    /**
     * @param bool $value
     */
    public function setAllowProfile($value)
    {
        $this->setFeature(self::FEATURE_PROFILE, $value);
    }

    /**
     * @return bool
     */
    public function getAllowLinkExternal()
    {
        return $this->getFeature(self::FEATURE_LINK_EXTERNAL);
    }

    /**
     * @param bool $value
     */
    public function setAllowLinkExternal($value)
    {
        $this->setFeature(self::FEATURE_LINK_EXTERNAL, $value);
    }

    /**
     * @return bool
     */
    public function getAllowMembershipSelection()
    {
        return $this->getFeature(self::FEATURE_MEMBERSHIP_SELECTION);
    }

    /**
     * @param bool $value
     */
    public function setAllowMembershipSelection($value)
    {
        $this->setFeature(self::FEATURE_MEMBERSHIP_SELECTION, $value);
    }

    /**
     * @return bool
     */
    public function getAllowApplicationProgram()
    {
        return $this->getFeature(self::FEATURE_APPLICATION_PROGRAM);
    }

    /**
     * @param bool $value
     */
    public function setAllowApplicationProgram($value)
    {
        $this->setFeature(self::FEATURE_APPLICATION_PROGRAM, $value);
    }

    /**
     * @return bool
     */
    public function getAllowExternalContent()
    {
        return $this->getFeature(self::FEATURE_EXTERNAL_CONTENT);
    }

    /**
     * @param bool $value
     */
    public function setAllowExternalContent($value)
    {
        $this->setFeature(self::FEATURE_EXTERNAL_CONTENT, $value);
    }

    /**
     * @param string $feature
     * @return boolean
     */
    public function getFeature($feature)
    {
        if (!isset($this->features[$feature])) {
            return false;
        }
        return $this->features[$feature];
    }

    /**
     * @param string $feature
     * @param bool $enabled
     */
    public function setFeature($feature, $enabled)
    {
        $this->features[$feature] = (boolean)$enabled;
    }
}