<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\StyleAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\StyleAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Model\Research\Exposition\Weave\Style\ShadowStyle;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;

/**
 * @ODM\Document(collection="Popover")
 * @ODM\Index(keys={"weave.$id"="asc"})
 */
class Popover extends GraphicalWeave implements StyleAwareInterface
{
    use StyleAwareTrait;

    const DEFAULT_SIZE_HEIGHT = 300;
    const DEFAULT_SIZE_WIDTH = 200;

    const VIEW_ON_MOUSE_OVER = 0;
    const VIEW_ON_CLICK = 1;

    const BACKGROUND_NONE = 0;
    const BACKGROUND_DARKEN = 1;

    const POSITION_LEFT = 0;
    const POSITION_RIGHT = 1;
    const POSITION_CENTER = 2;
    const POSITION_CENTER_SCREEN = 3;

    /**
     * @var Weave
     * @ODM\ReferenceOne(storeAs="dbRef")
     */
    protected $weave;

    /**
     * @var ToolInterface
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Exposition\Weave\Tool", storeAs="dbRef")
     */
    protected $tool;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $viewOn;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $position;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $backgroundOverlay;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $autoContent;

    public function __construct()
    {
        parent::__construct();
        $this->shadow = new ShadowStyle();
        $this->exposition = $this->getExposition();
        $this->width = self::DEFAULT_SIZE_WIDTH;
        $this->height = self::DEFAULT_SIZE_HEIGHT;
    }

    /**
     * @return array
     */
    public static function getViewOns()
    {
        return [
            self::VIEW_ON_CLICK      => 'view_on.click',
            self::VIEW_ON_MOUSE_OVER => 'view_on.mouse_over',
        ];
    }

    /**
     * @return array
     */
    public static function getPositions()
    {
        return [
            self::POSITION_RIGHT         => 'position.right',
            self::POSITION_LEFT          => 'position.left',
            self::POSITION_CENTER        => 'position.center',
            self::POSITION_CENTER_SCREEN => 'position.center_screen',
        ];
    }

    /**
     * @return array
     */
    public static function getBackgrounds()
    {
        return [
            self::BACKGROUND_DARKEN => 'background.darken',
            self::BACKGROUND_NONE   => 'background.none',
        ];
    }

    /**
     * @inheritDoc
     */
    public function getAclEntity()
    {
        return $this->getExposition();
    }

    /**
     * @inheritDoc
     */
    public function getExposition()
    {
        if (!isset($this->weave)) {
            return null;
        }
        return $this->weave->getExposition();
    }

    /**
     * @inheritDoc
     */
    public function setExposition(Exposition $exposition)
    {
        throw new \LogicException('The exposition of a popover cannot be changed directly! Replace the weave instead.');
    }

    /**
     * @return void
     */
    public function __clone()
    {
        parent::__clone();

        $toolList = [];
        foreach ($this->toolList as $tool) {
            $newTool = clone $tool;
            $newTool->setWeave($this);
            $toolList[] = $newTool;
        }
        $this->toolList = new ArrayCollection($toolList);
    }

    /**
     * @return Weave
     */
    public function getWeave()
    {
        return $this->weave;
    }

    /**
     * @param Weave $weave
     * @return $this
     */
    public function setWeave(Weave $weave)
    {
        $this->weave = $weave;
        $this->exposition = $this->getExposition();
        return $this;
    }

    /**
     * @return ToolInterface
     */
    public function getTool()
    {
        return $this->tool;
    }

    /**
     * @param ToolInterface $tool
     */
    public function setTool(ToolInterface $tool = null)
    {
        $this->tool = $tool;
    }

    /**
     * @return bool
     */
    public function hasTool()
    {
        if (!isset($this->tool)) {
            return false;
        }

        try {
            $this->tool->getName();
        } catch (DocumentNotFoundException $exception) {
            trigger_error($exception->getMessage(), E_USER_WARNING);

            $this->tool = null;
            PopoverRepository::persist($this);
        }

        return isset($this->tool);
    }

    /**
     * @return int
     */
    public function getViewOn()
    {
        return $this->viewOn;
    }

    /**
     * @param int $viewOn
     * @return $this
     */
    public function setViewOn($viewOn)
    {
        $this->viewOn = $viewOn;
        return $this;
    }

    /**
     * @return int
     */
    public function getPosition()
    {
        return $this->position;
    }

    /**
     * @param int $position
     * @return $this
     */
    public function setPosition($position)
    {
        $this->position = $position;
        return $this;
    }

    /**
     * @return int
     */
    public function getBackground()
    {
        return $this->backgroundOverlay;
    }

    /**
     * @param int $background
     * @return $this
     */
    public function setBackground($background)
    {
        $this->backgroundOverlay = $background;
        return $this;
    }

    /**
     * @return bool
     */
    public function isAutoContent()
    {
        return $this->autoContent;
    }

    /**
     * @param bool $isAutoContent
     */
    public function setAutoContent($isAutoContent)
    {
        $this->autoContent = $isAutoContent;
    }
}