<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\Core\OrmAwareTrait;
use Rc\Domain\Model\Media\MediaAwareTrait;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\MediaContainer;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;

/**
 * @ODM\Document
 * @ODM\InheritanceType("SINGLE_COLLECTION")
 * @ODM\DiscriminatorField("_class")
 * @ODM\Index(keys={"research.$id"="asc"})
 */
class SimpleMedia extends AbstractModel implements SimpleMediaInterface
{
    use OrmAwareTrait;
    use MediaAwareTrait;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     * @ODS\Type(name="Rc\Serializer\Type\Owner")
     */
    protected $owner;

    /**
     * @var Research
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research", inversedBy="simpleMedias", cascade={"persist"}, storeAs="dbRef")
     */
    protected $research;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $name;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $copyright;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $description;

    /**
     * @var bool
     * @ODM\Field(type="bool")
     */
    protected $hidden = false;

    /**
     * SimpleMedia constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->mediaList = new ArrayCollection();
    }

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        $mediaContainer = new MediaContainer($this->mediaList->toArray(), $this->thumb);
        $mediaContainer->setId($this->getId());
        $mediaContainer->setName($this->getName());
        $mediaContainer->setContentType($this->getMediaType());
        return $mediaContainer;
    }

    /**
     * @inheritDoc
     */
    public function getAclEntity()
    {
        return $this->research;
    }

    /**
     * @return User
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner)
    {
        $this->owner = $owner;
        return $this;
    }

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner)
    {
        return $owner === $this->owner;
    }

    /**
     * @return Research
     */
    public function getResearch()
    {
        return $this->research;
    }

    /**
     * @param Research $research
     */
    public function setResearch(Research $research)
    {
        $this->research = $research;
    }

    /**
     * @return bool
     */
    public function hasExposition()
    {
        return $this->research instanceof Exposition;
    }

    /**
     * @return bool
     */
    public function hasWork()
    {
        return $this->research instanceof Work;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @deprecated
     * @return string
     */
    public function getTitle()
    {
        return $this->getName();
    }

    /**
     * @deprecated
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->setName($title);
    }

    /**
     * @return string
     */
    public function getCopyright()
    {
        return $this->copyright;
    }

    /**
     * @param string $copyright
     * @return void
     */
    public function setCopyright($copyright)
    {
        $this->copyright = $copyright;
    }

    /**
     * @deprecated
     * @return string
     */
    public function getCopyrightHolder()
    {
        return $this->getCopyright();
    }

    /**
     * @deprecated
     * @param string $copyright
     */
    public function setCopyrightHolder($copyright)
    {
        $this->setCopyright($copyright);
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     * @return void
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return $this->hidden;
    }

    /**
     * @param bool $hidden
     */
    public function setHidden($hidden)
    {
        $this->hidden = $hidden;
    }

    /**
     * @return string|null
     */
    public function getMediaType()
    {
        if (!$this->hasMedia()) {
            return null;
        }
        return $this->getMedia()->getContentType();
    }

    /**
     * @param string $mediaType
     * @return bool
     */
    public function hasMediaType($mediaType)
    {
        return $this->getMediaType() == $mediaType;
    }

    /**
     * @return array
     */
    public function getMediaVersions()
    {
        if (!$this->hasMedia()) {
            return [];
        }
        return array_keys($this->getMedia()->getDerivativeList());
    }

    /**
     * @return array
     */
    public function getOptions()
    {
        return [];
    }
}