<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Comment;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Context;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\EmbeddedDocument
 */
class ExpositionContext extends Context
{
    const ROLE_ADMIN = 'admin';
    const ROLE_COLLABORATEUR = 'collaborateur';
    const ROLE_EDITOR = 'editor';
    const ROLE_OWNER = 'owner';
    const ROLE_REVIEWER = 'reviewer';
    const ROLE_SUPERVISOR = 'supervisor';

    /**
     * @param ToolInterface $tool
     * @return static
     */
    public static function from(ToolInterface $tool)
    {
        $user = $tool->getOwner();
        if (!isset($user)) {
            throw new \InvalidArgumentException('Cannot create context without a valid owner for ' . $tool . '!');
        }

        $exposition = $tool->getExposition();
        if (!isset($exposition)) {
            throw new \InvalidArgumentException('Cannot create context without a valid exposition for ' . $tool . '!');
        }

        $roles = [];
        if ($user->isAdmin()) {
            $roles[] = self::ROLE_ADMIN;
        }

        if ($exposition->isCollaborateur($user)) {
            $roles[] = self::ROLE_COLLABORATEUR;
        }

        if ($exposition->isEditor($user)) {
            $roles[] = self::ROLE_EDITOR;
        }

        if ($exposition->isOwner($user)) {
            $roles[] = self::ROLE_OWNER;
        }

        if ($exposition->isReviewer($user)) {
            $roles[] = self::ROLE_REVIEWER;
        }

        if ($exposition->isSupervisor($user)) {
            $roles[] = self::ROLE_SUPERVISOR;
        }

        return new static($roles);
    }

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    public function isVisibleTo(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        $exposition = $tool->getExposition();
        if (!isset($exposition)) {
            throw new \InvalidArgumentException('Cannot apply context without a valid exposition for ' . $tool . '!');
        }

        // notes are not visible when exposition is published
        if ($exposition->isPublished()) {
            return false;
        }

        // a privileged user sees all comments
        if ($user->isPrivilegedUser($exposition)) {
            return true;
        }

        // a reviewer sees only his comments (for kc portal the reviewer sees the comments of the supervisor too
        if ($exposition->isReviewer($user)) {
            return $tool->isOwner($user) || $this->isKcPortalReview($exposition) && $this->hasSupervisorRole();
        }

        // special users see all non reviewer comments
        if ($exposition->isOwner($user) ||
            $exposition->isCollaborateur($user) ||
            $exposition->isEditor($user) ||
            $exposition->isSupervisor($user)
        ) {
            return !$this->hasReviewerRole();
        }

        return false;
    }

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    public function isEditableBy(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        $exposition = $tool->getExposition();
        if (!isset($exposition)) {
            throw new \InvalidArgumentException('Cannot apply context without a valid exposition for ' . $tool . '!');
        }

        if ($user->isPrivilegedUser($exposition) || $tool->isOwner($user)) {
            return true;
        }

        $owner = $tool->getOwner();
        if (!isset($owner)) {
            return false;
        }

        if ($exposition->isOwner($owner) || $exposition->isCollaborateur($owner)) {
            return true;
        }

        if (!$owner->isPrivilegedUser($exposition) && !$exposition->isSupervisor($owner)) {
            return true;
        }

        return false;
    }

    /**
     * @param ToolInterface $tool
     * @param User|null $user
     * @return bool
     */
    public function isMetaEditableBy(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        return $tool->isOwner($user);
    }

    /**
     * @return bool
     */
    public function isActive()
    {
        $context = self::getCurrentContext();

        return empty(array_filter($context));
    }

    /**
     * @return bool
     */
    protected function hasReviewerRole()
    {
        return in_array(self::ROLE_REVIEWER, $this->roles);
    }

    /**
     * @return bool
     */
    protected function hasSupervisorRole()
    {
        return in_array(self::ROLE_SUPERVISOR, $this->roles);
    }

    /**
     * @param Exposition $exposition
     * @return bool
     */
    protected function isKcPortalReview(Exposition $exposition)
    {
        $reviewPortal = $exposition->getReviewPortal();
        if (!isset($reviewPortal)) {
            return null;
        }

        return $reviewPortal->getId() == 6;
    }
}