<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Tool;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use DOMDocument;
use Rc\Domain\Model\Research\Exposition\Weave\Shape\PlayerShape;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\PlayerAwareTrait;

/**
 * @ODM\Document
 */
class EmbedTool extends Tool
{

    use PlayerAwareTrait;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $url;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $urlApi;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $previewUrl;

    public function __construct()
    {
        parent::__construct();
        $this->player = new PlayerShape();
    }

    /**
     * @param string $url
     * @return array
     */
    public static function processUrl($url)
    {
        $retVal = [
            'url'              => $url,
            'urlApi'           => null,
            'previewUrl'       => null,
            'width'            => null,
            'height'           => null,
            'loop'             => true,
            'autoplay'         => true,
            'stopOtherPlayers' => true,
        ];

        if (strpos($url, 'youtube') !== false) {
            $retVal = self::processOembed('https://www.youtube.com/oembed?url=', $url) + $retVal;

        } elseif (strpos($url, 'vimeo') !== false) {
            $retVal = self::processOembed('https://vimeo.com/api/oembed.json?api=true&url=', $url) + $retVal;

        } elseif (strpos($url, 'sketchfab') !== false) {
            $retVal = self::processOembed('https://sketchfab.com/oembed?url=', $url) + $retVal;
            $retVal['loop'] = false;
            $retVal['stopOtherPlayers'] = false;

        } elseif (strpos($url, 'soundcloud') !== false) {
            $retVal = self::processOembed('https://soundcloud.com/oembed.json?url=', $url) + $retVal;

        } elseif (strpos($url, 'medienarchiv.zhdk.ch') !== false) {
            $retVal = self::processOembed('https://medienarchiv.zhdk.ch/oembed?url=', $url) + $retVal;
            $retVal['loop'] = false;
            $retVal['autoplay'] = false;
            $retVal['stopOtherPlayers'] = false;

        }

        return $retVal;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @param string $url
     */
    public function setUrl($url)
    {
        if ($url === $this->url) {
            return;
        }

        $data = self::processUrl($url);
        $this->url = $data['url'];
        $this->urlApi = $data['urlApi'];
        $this->previewUrl = $data['previewUrl'];
    }

    /**
     * @return string
     */
    public function getPreviewUrl()
    {
        return $this->previewUrl;
    }

    /**
     * @return string
     */
    public function getUrlApi()
    {
        return $this->urlApi;
    }

    /**
     * @param string $urlApi
     */
    public function setUrlApi($urlApi)
    {
        $this->urlApi = $urlApi;
    }

    /**
     * @return bool
     */
    public function isYoutube()
    {
        return strpos(parse_url($this->urlApi, PHP_URL_HOST), 'youtube.com') !== false;
    }

    /**
     * @return bool
     */
    public function isSoundcloud()
    {
        return strpos(parse_url($this->urlApi, PHP_URL_HOST), 'soundcloud') !== false;
    }

    /**
     * @return bool
     */
    public function isVimeo()
    {
        return strpos(parse_url($this->urlApi, PHP_URL_HOST), 'vimeo') !== false;
    }

    /**
     * @return bool
     */
    public function isSketchfab()
    {
        return strpos(parse_url($this->urlApi, PHP_URL_HOST), 'sketchfab') !== false;
    }

    /**
     * @return bool
     */
    public function isMadek()
    {
        return strpos(parse_url($this->urlApi, PHP_URL_HOST), 'medienarchiv.zhdk.ch') !== false;
    }

    /**
     * @return array
     */
    public function getSupportedMediaTypes()
    {
        return [];
    }

    protected static function processOembed($oembed, $url)
    {
        $defaults = [
            'html'          => null,
            'thumbnail_url' => null,
            'width'         => null,
            'height'        => null,
        ];
        $siteContent = json_decode(self::getSiteContent($oembed . urlencode($url)), true) + $defaults;

        preg_match('~src="([^"]+)"~i', $siteContent['html'], $matches);

        return [
            'urlApi'     => isset($matches[1]) ? $matches[1] : null,
            'previewUrl' => $siteContent['thumbnail_url'] ?: self::getMetaTag(self::getSiteContent($url), 'og:image'),
            'width'      => strpos($siteContent['width'], '%') !== false ? $siteContent['height'] : $siteContent['width'],
            'height'     => $siteContent['height'],
        ];
    }

    /**
     * @param string $url
     * @return string
     */
    protected static function getSiteContent($url)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.12) Gecko/20101026 Firefox/3.6.12');
        $siteContent = curl_exec($ch);
        curl_close($ch);

        return $siteContent;
    }

    /**
     * @param string $siteContent
     * @param string $name
     * @return null|string
     */
    protected static function getMetaTag($siteContent, $name)
    {
        $dom = new DOMDocument();
        @$dom->loadHTML($siteContent);
        foreach ($dom->getElementsByTagName('meta') as $meta) {
            if ($meta->getAttribute('property') == $name) {
                return $meta->getAttribute('content');
            }
        }

        return null;
    }
}