<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use EntityFeed;
use EntityFeedRepository;
use JMS\Serializer\Annotation as ODS;
use MyLocal;
use PortalFeed;
use PortalFeedRepository;
use Rc\Domain\Model\Core\Acl\AclGroup;
use Rc\Domain\Model\Generic\KeywordAwareInterface;
use Rc\Domain\Model\Generic\KeywordAwareTrait;
use Rc\Domain\Model\Media\MediaAwareInterface;
use Rc\Domain\Model\Media\MediaAwareTrait;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User\Settings;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use UserFeed;
use UserFeedRepository;

/**
 * @ODM\Document
 * @ODM\Index(keys={"portals.$id"="asc"})
 * @ODM\Index(keys={
 *     "name"="text",
 *     "realName"="text",
 *     "realName"="text",
 *     "affiliation"="text",
 *     "tagLine"="text",
 *     "memberInstitution"="text",
 *     "memberInstitution"="text",
 *     "description.en"="text"
 * })
 */
class User extends AbstractModel implements MediaAwareInterface, KeywordAwareInterface
{
    use MediaAwareTrait;
    use KeywordAwareTrait;

    const ROLE_SUPER_ADMIN = 'super-admin';

    const ROLE_PORTAL_ADMIN = 'portal-admin';

    const ROLE_REVIEWER = 'reviewer';

    const ROLE_LIMITED_USER = 'limited-user';

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $hide = false;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $hasPublicObjects = false;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $email;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $approved = false;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $password;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $name;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $realName;

    /**
     * @var string[]
     * @ODM\Field(type="collection")
     */
    protected $countryResidence = [];

    /**
     * @var string[]
     * @ODM\Field(type="collection")
     */
    protected $countryCitizenship = [];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $affiliation;

    /**
     * @var string[]
     * @ODM\Field(type="collection")
     */
    protected $favorites = [];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $tagLine;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $yearOfBirth;

    /**
     * @var ArrayCollection|Portal[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal", inversedBy="memberList", storeAs="dbRef")
     */
    protected $portals;

    /**
     * @var ArrayCollection|Portal[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal", inversedBy="memberList", storeAs="dbRef")
     */
    protected $pendingPortals;

    /**
     * @var string[]
     * @ODM\Field(type="collection")
     */
    protected $roles = [];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $passwordForgottenKey;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $sarMember = false;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $memberInstitution;

    /**
     * @var Settings
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\User\Settings")
     */
    protected $settings;

    /**
     * @var Translation
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\Translation")
     */
    protected $description;

    /**
     * @var array
     * @ODM\Field(type="collection")
     */
    protected $groups = [];

    /**
     * @var MediaInterface
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Media", cascade={"persist"}, storeAs="dbRef")
     */
    protected $curriculumVitae;

    /**
     * @var ArrayCollection|Exposition[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition", mappedBy="owner", storeAs="dbRef")
     */
    protected $researches;

    /**
     * @var ArrayCollection|Work[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Work", mappedBy="owner", criteria={"_class" : "Rc\Domain\Model\Research\Work"}, storeAs="dbRef")
     */
    protected $works;

    /**
     * @var int[]
     * @ODS\Exclude()
     */
    protected $followedEntityIds;

    public function __construct()
    {
        parent::__construct();
        $this->portals = new ArrayCollection();
        $this->researches = new ArrayCollection();
        $this->mediaList = new ArrayCollection();
    }

    /**
     * @return boolean
     */
    public function isHidden()
    {
        return $this->hide;
    }

    /**
     * @return bool
     */
    public function getHide()
    {
        return $this->isHidden();
    }

    /**
     * @param boolean $hide
     * @return void
     */
    public function setHide($hide)
    {
        $this->hide = $hide;
    }

    /**
     * @return bool
     */
    public function getHasPublicObjects()
    {
        return (bool)$this->hasPublicObjects;
    }

    /**
     * @param bool $hasPublicObjects
     */
    public function setHasPublicObjects($hasPublicObjects)
    {
        $this->hasPublicObjects = (bool)$hasPublicObjects;
    }

    /**
     * @return Translation
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param Translation $description
     * @return $this
     */
    public function setDescription(Translation $description)
    {
        $this->description = $description;
        return $this;
    }

    /**
     * @deprecated
     * @return Translation
     */
    public function getDescriptionList()
    {
        return $this->description ?: new Translation([]);
    }

    /**
     * @deprecated
     * @return string
     */
    public function getTitle()
    {
        return $this->getName();
    }

    /**
     * @return string
     */
    public function getName()
    {
        if ($this->deleted) {
            return 'Deleted user';
        }
        return $this->name;
    }

    /**
     * @param string $name
     * @return User
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return string
     */
    public function getRealName()
    {
        return $this->realName;
    }

    /**
     * @param string $realName
     * @return User
     */
    public function setRealName($realName)
    {
        $this->realName = $realName;
        return $this;
    }

    /**
     * @return string
     */
    public function getAffiliation()
    {
        return $this->affiliation;
    }

    /**
     * @param string $affiliation
     * @return User
     */
    public function setAffiliation($affiliation)
    {
        $this->affiliation = $affiliation;
        return $this;
    }

    /**
     * @return array
     */
    public function getFavorites()
    {
        return $this->favorites;
    }

    /**
     * @param array $favorites
     * @return User
     */
    public function setFavorites(array $favorites)
    {
        $this->favorites = $favorites;
        return $this;
    }

    /**
     * @return string
     */
    public function getTagLine()
    {
        return $this->tagLine;
    }

    /**
     * @param string $tagLine
     * @return User
     */
    public function setTagLine($tagLine)
    {
        $this->tagLine = $tagLine;
        return $this;
    }

    /**
     * @return int
     */
    public function getYearOfBirth()
    {
        return $this->yearOfBirth;
    }

    /**
     * @param int $year
     * @return $this
     */
    public function setYearOfBirth($year)
    {
        $this->yearOfBirth = $year;
        return $this;
    }

    /**
     * @return null|string
     */
    public function getTranslatedCountry()
    {
        $countryList = MyLocal::getTranslationList('territory', null, 2);

        $countryResidence = $this->getCountryResidence();
        $countryCitizenship = $this->getCountryCitizenship();

        $countries = [];
        if (!empty($countryResidence)) {
            $countries['residence'] = [];
            foreach ($countryResidence as $country) {
                $countries['residence'][] = $countryList[$country];
            }
        }
        if (!empty($countryCitizenship)) {
            $countries['citizenship'] = [];
            foreach ($countryCitizenship as $country) {
                $countries['citizenship'][] = $countryList[$country];
            }
        }

        if (empty($countries)) {
            return null;
        }

        if (count($countries) == 1) {
            return implode(', ', reset($countries)) . ' (' . key($countries) . ')';
        }

        if (array_diff($countries['residence'], $countries['citizenship']) == array_diff($countries['citizenship'],
                $countries['residence'])
        ) {
            return implode(', ', $countries['residence']);
        }

        return implode(', ', $countries['residence']) . ' (residence), ' . implode(', ',
                $countries['citizenship']) . ' (citizenship)';
    }

    /**
     * @return array
     */
    public function getCountryResidence()
    {
        return $this->countryResidence;
    }

    /**
     * @param array $countryResidence
     * @return User
     */
    public function setCountryResidence(array $countryResidence)
    {
        $this->countryResidence = $countryResidence;
        return $this;
    }

    /**
     * @return array
     */
    public function getCountryCitizenship()
    {
        return $this->countryCitizenship;
    }

    /**
     * @param array $countryCitizenship
     * @return User
     */
    public function setCountryCitizenship(array $countryCitizenship)
    {
        $this->countryCitizenship = $countryCitizenship;
        return $this;
    }

    /**
     * @return bool
     */
    public function hasResearches()
    {
        return !$this->researches->isEmpty();
    }

    /**
     * @return Exposition[]
     */
    public function getResearches()
    {
        return $this->researches->toArray();
    }

    /**
     * @return bool
     */
    public function hasCollaboratedResearches()
    {
        return !empty(ExpositionRepository::findCollaboratedByPerson($this));
    }

    /**
     * @return Exposition[]
     */
    public function getCollaboratedResearches()
    {
        return ExpositionRepository::findCollaboratedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasSupervisedResearches()
    {
        return !empty(ExpositionRepository::findSupervisedByPerson($this));
    }

    /**
     * @return Exposition[]
     */
    public function getSupervisedResearches()
    {
        return ExpositionRepository::findSupervisedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasEditedResearches()
    {
        return !empty(ExpositionRepository::findEditedByPerson($this));
    }

    /**
     * @return Exposition[]
     */
    public function getEditedResearches()
    {
        return ExpositionRepository::findEditedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasAttendedResearches()
    {
        return !empty(ExpositionRepository::findAttendedByPerson($this));
    }

    /**
     * @return Exposition[]
     */
    public function getAttendedResearches()
    {
        return ExpositionRepository::findAttendedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasWorks()
    {
        return !$this->works->isEmpty();
    }

    /**
     * @return Work[]
     */
    public function getWorks()
    {
        return $this->works->toArray();
    }

    /**
     * @return Work[]
     */
    public function getCoAuthoredWorks()
    {
        return WorkRepository::findCoAuthoredByPerson($this);
    }

    /**
     * @return Work[]
     */
    public function getCollaboratedWorks()
    {
        return WorkRepository::findCollaboratedByPerson($this);
    }

    /**
     * @return Work[]
     */
    public function getSupervisedWorks()
    {
        return WorkRepository::findSupervisedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasProjects()
    {
        return !empty(ProjectRepository::findByPerson($this));
    }

    /**
     * @return Project[]
     */
    public function getProjects()
    {
        return ProjectRepository::findByPerson($this);
    }

    /**
     * @return Project[]
     */
    public function getCollaboratedProjects()
    {
        return ProjectRepository::findCollaboratedByPerson($this);
    }

    /**
     * @return Project[]
     */
    public function getSupervisedProjects()
    {
        return ProjectRepository::findSupervisedByPerson($this);
    }

    /**
     * @return bool
     */
    public function hasDegrees()
    {
        return !empty(DegreeRepository::findByPerson($this));
    }

    /**
     * @return Degree[]
     */
    public function getDegrees()
    {
        return DegreeRepository::findByPerson($this);
    }

    /**
     * @return Degree[]
     */
    public function getCollaboratedDegrees()
    {
        return DegreeRepository::findCollaboratedByPerson($this);
    }

    /**
     * @return Degree[]
     */
    public function getSupervisedDegrees()
    {
        return DegreeRepository::findSupervisedByPerson($this);
    }

    /**
     * @return MediaInterface
     */
    public function getCurriculumVitae()
    {
        return $this->curriculumVitae;
    }

    /**
     * @param MediaInterface $curriculumVitae
     */
    public function setCurriculumVitae(MediaInterface $curriculumVitae = null)
    {
        $this->curriculumVitae = $curriculumVitae;
    }

    public function hasCurriculumVitae()
    {
        return isset($this->curriculumVitae);
    }

    /**
     * @deprecated
     */
    public function hasCv()
    {
        return isset($this->curriculumVitae);
    }

    /**
     * @deprecated
     */
    public function getCv()
    {
        return $this->getCurriculumVitae();
    }

    /**
     * @todo
     * @deprecated
     */
    public function setCv($file = null)
    {
        $this->setCurriculumVitae($file);
        return $this;
    }

    /**
     * @deprecated
     */
    public function removeCv()
    {
        $this->setCurriculumVitae(null);
        return $this;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     * @return $this
     */
    public function setEmail($email)
    {
        $this->email = strtolower($email) ?: $email;
        return $this;
    }

    /**
     * @return bool
     */
    public function getApproved()
    {
        return $this->approved;
    }

    /**
     * @return bool
     */
    public function isApproved()
    {
        return (bool)$this->approved;
    }

    /**
     * @param bool $approved
     */
    public function setApproved($approved)
    {
        $this->approved = $approved;
    }

    /**
     * @return string
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * @param string $password
     * @param bool $hashed
     * @return $this
     */
    public function setPassword($password, $hashed = false)
    {
        if (!$hashed) {
            $this->password = md5(config('app.salt') . $password);
        } else {
            $this->password = $password;
        }
        return $this;
    }

    /**
     * @return Settings
     */
    public function getSettings()
    {
        return $this->settings;
    }

    /**
     * @param Settings $settings
     * @return $this
     */
    public function setSettings(Settings $settings)
    {
        $this->settings = $settings;
        return $this;
    }

    /**
     * @param bool $reviewer
     */
    public function setReviewer($reviewer)
    {
        if ($reviewer) {
            $this->addRole(static::ROLE_REVIEWER);
            $this->hide = false;
        } else {
            $this->removeRole(static::ROLE_REVIEWER);
        }
    }

    /**
     * @return array
     */
    public function getGroups()
    {
        $defaultGroups = [
            AclGroup::RC_GUEST,
            AclGroup::RC_USER,
            AclGroup::USER($this->getId()),
        ];

        return array_merge($defaultGroups, $this->groups);
    }

    /**
     * @param array $groups
     * @return $this
     */
    public function setGroups(array $groups)
    {
        $this->groups = $groups;
        return $this;
    }

    /**
     * @param string $group
     * @return bool
     */
    public function hasGroup($group)
    {
        return in_array($group, $this->groups);
    }

    /**
     * @param string $groupPrefix
     * @return bool
     */
    public function hasGroupPrefix($groupPrefix)
    {
        foreach ($this->groups as $group) {
            if (substr($group, 0, strlen($groupPrefix)) == $groupPrefix) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param string $group
     * @return bool
     */
    public function addGroup($group)
    {
        if ($this->hasGroup($group)) {
            return false;
        }

        $groupPrefix = AclGroup::getGroupPrefix($group);
        if ($groupPrefix == AclGroup::getGroupPrefix(AclGroup::PORTAL_REVIEWER)) {
            $this->setReviewer(true);
        } elseif ($groupPrefix == AclGroup::getGroupPrefix(AclGroup::PORTAL_ADMIN)) {
            $this->setPortalAdmin(true);
        }

        $this->groups[] = $group;
        return true;
    }

    /**
     * @param string $group
     * @return bool
     */
    public function removeGroup($group)
    {
        $retVal = $this->removeGroups([$group]);

        $groupPrefix = AclGroup::getGroupPrefix($group);

        if ($this->hasGroupByPrefix($groupPrefix)) {
            return $retVal;
        }

        if ($groupPrefix == AclGroup::getGroupPrefix(AclGroup::PORTAL_REVIEWER)) {
            $this->setReviewer(false);
        } elseif ($groupPrefix == AclGroup::getGroupPrefix(AclGroup::PORTAL_ADMIN)) {
            $this->setPortalAdmin(false);
        }

        return $retVal;
    }

    /**
     * @param array $groups
     * @return bool
     */
    public function hasGroups(array $groups)
    {
        $retVal = true;
        foreach ($groups as $group) {
            $retVal &= $this->hasGroup($group);
        }
        return $retVal;
    }

    /**
     * @param array $groups
     * @return bool
     */
    public function addGroups(array $groups)
    {
        $changed = false;
        foreach ($groups as $group) {
            $changed |= $this->addGroup($group);
        }
        return $changed;
    }

    /**
     * @param array $groups
     * @return bool
     */
    public function removeGroups(array $groups)
    {
        $oldCount = count($this->groups);
        $this->groups = array_diff($this->groups, $groups);
        return $oldCount !== count($this->groups);
    }

    /**
     * @return array
     */
    public function getRoles()
    {
        return $this->roles;
    }

    /**
     * @param array $roles
     */
    public function setRoles(array $roles)
    {
        $this->roles = $roles;
    }

    /**
     * @param string $role
     * @return bool
     */
    public function hasRole($role)
    {
        return in_array($role, $this->roles);
    }

    /**
     * @param string $role
     */
    public function addRole($role)
    {
        if ($this->hasRole($role)) {
            return;
        }
        $this->roles[] = $role;
    }

    /**
     * @param string $role
     */
    public function removeRole($role)
    {
        $this->roles = array_diff($this->roles, [$role]);
    }

    /**
     * @return bool
     */
    public function isReviewer()
    {
        return $this->hasRole(self::ROLE_REVIEWER);
    }

    /**
     * @param Exposition $research
     * @return bool
     */
    public function isPrivilegedUser(Exposition $research)
    {
        return $this->isAdmin() || $research->isEditor($this);
    }

    /**
     * @return bool
     */
    public function isAdmin()
    {
        return $this->isPortalAdmin() || $this->isSuperAdmin();
    }

    /**
     * @return bool
     */
    public function isInvolved()
    {
        return $this->isReviewer() || $this->isAdmin();
    }

    /**
     * @return bool
     */
    public function isLimitedUser()
    {
        return $this->hasRole(self::ROLE_LIMITED_USER);
    }

    /**
     * @param bool $limitedUser
     */
    public function setLimitedUser($limitedUser)
    {
        if ($limitedUser) {
            $this->addRole(static::ROLE_LIMITED_USER);
        } else {
            $this->removeRole(static::ROLE_LIMITED_USER);
            $this->hide = false;
        }
    }

    /**
     * @param Portal|int|null $portal
     * @return bool
     */
    public function isPortalAdmin($portal = null)
    {
        if (!isset($portal)) {
            return $this->hasRole(self::ROLE_PORTAL_ADMIN);
        }
        if (is_numeric($portal)) {
            $portal = PortalRepository::getById($portal);
        }
        return $portal->isAdmin($this);
    }

    /**
     * @param bool $portalAdmin
     */
    public function setPortalAdmin($portalAdmin)
    {
        if ($portalAdmin) {
            $this->addRole(static::ROLE_PORTAL_ADMIN);
            $this->hide = false;
        } else {
            $this->removeRole(static::ROLE_PORTAL_ADMIN);
        }
    }

    /**
     * @return bool
     */
    public function isSuperAdmin()
    {
        return $this->hasRole(self::ROLE_SUPER_ADMIN);
    }

    /**
     * @param bool $superAdmin
     */
    public function setSuperAdmin($superAdmin)
    {
        if ($superAdmin) {
            $this->addRole(static::ROLE_SUPER_ADMIN);
            $this->hide = false;
        } else {
            $this->removeRole(static::ROLE_SUPER_ADMIN);
        }
    }

    /**
     * @param Portal $portal
     * @return boolean
     */
    public function isPortal(Portal $portal)
    {
        return $this->portals->contains($portal);
    }

    /**
     * @return Portal[]
     */
    public function getPortals()
    {
        return $this->portals->toArray();
    }

    /**
     * @return Portal[]
     */
    public function getNonDefaultPortals()
    {
        $portals = [];
        foreach ($this->portals as $portal) {
            if (!$portal->isDefault()) {
                $portals[] = $portal;
            }
        }
        return $portals;
    }

    /**
     * @param Portal $portal
     * @return bool
     */
    public function hasPortal(Portal $portal)
    {
        return $this->portals->contains($portal);
    }

    /**
     * @return array
     */
    public function getPortalIds()
    {
        return $this->createIdList($this->portals);
    }

    /**
     * @param array $portalIds
     */
    public function setPortalIds(array $portalIds)
    {
        $this->removeGroups(AclGroup::PORTAL($this->getPortalIds()));
        $this->addGroups(AclGroup::PORTAL($portalIds));

        $this->portals = new ArrayCollection($this->createReferenceList(Portal::class, $portalIds));
    }

    /**
     * @param int $portalId
     */
    public function addPortalId($portalId)
    {
        $this->addPortal($this->createReference(Portal::class, $portalId));
    }

    /**
     * @param Portal $portal
     */
    public function addPortal(Portal $portal)
    {
        if (!$this->portals->contains($portal)) {
            $this->portals->add($portal);
            $this->addGroups(AclGroup::PORTAL([$portal->getId()]));
        }
    }

    /**
     * @param Portal $portal
     * @return boolean
     */
    public function isPendingPortal(Portal $portal)
    {
        return $this->pendingPortals->contains($portal);
    }

    /**
     * @return Portal[]
     */
    public function getPendingPortals()
    {
        return $this->pendingPortals->toArray();
    }

    /**
     * @param array $pendingPortals
     */
    public function setPendingPortals(array $pendingPortals)
    {
        $this->pendingPortals = new ArrayCollection($pendingPortals);
    }

    /**
     * @param Portal $pendingPortal
     * @return bool
     */
    public function hasPendingPortal(Portal $pendingPortal)
    {
        return $this->pendingPortals->contains($pendingPortal);
    }

    /**
     * @return array
     */
    public function getPendingPortalIds()
    {
        return $this->createIdList($this->pendingPortals);
    }

    /**
     * @param array $pendingPortalIds
     */
    public function setPendingPortalIds(array $pendingPortalIds)
    {
        $this->setPendingPortals($this->createReferenceList(Portal::class, $pendingPortalIds));
    }

    /**
     * @param int $pendingPortalId
     */
    public function addPendingPortalId($pendingPortalId)
    {
        $pendingPortal = $this->createReference(Portal::class, $pendingPortalId);
        if (!$this->pendingPortals->contains($pendingPortal)) {
            $this->pendingPortals->add($pendingPortal);
        }
    }

    /**
     * @param Portal $portal
     */
    public function removePendingPortal(Portal $portal)
    {
        if ($this->pendingPortals->contains($portal)) {
            $this->pendingPortals->removeElement($portal);
        }
    }

    /**
     * @return bool
     */
    public function isMemberOfNonDefaultPortal()
    {
        return !empty($this->getNonDefaultPortals());
    }

    /**
     * @return string|null
     */
    public function getMemberInstitution()
    {
        return $this->memberInstitution;
    }

    /**
     * @param string|null $memberInstitution
     */
    public function setMemberInstitution($memberInstitution)
    {
        $this->memberInstitution = $memberInstitution;
    }

    /**
     * @return bool
     */
    public function getSarMember()
    {
        return $this->sarMember;
    }

    /**
     * @param bool $sarMember
     */
    public function setSarMember($sarMember)
    {
        $this->sarMember = $sarMember;
    }

    /**
     * @return string|null
     */
    public function getPasswordForgottenKey()
    {
        return $this->passwordForgottenKey;
    }

    /**
     * @param string $passwordForgottenKey
     */
    public function setPasswordForgottenKey($passwordForgottenKey)
    {
        $this->passwordForgottenKey = $passwordForgottenKey;
    }

    /**
     * @todo
     * @param Research $entity
     */
    public function followEntity(Research $entity)
    {
        if ($this->followsEntity($entity)) {
            return;
        }

        $entityFeed = new EntityFeed();
        $entityFeed->setUser($this);
        $entityFeed->setTarget($entity);
        EntityFeedRepository::add($entityFeed);
    }

    /**
     * @param Research $entity
     * @return bool
     */
    public function followsEntity(Research $entity)
    {
        return !empty($this->getFollowedEntityIds()[$entity->getId()]);
    }

    /**
     * @return array
     */
    public function getFollowedEntityIds()
    {
        if (!isset($this->followedEntityIds)) {
            $this->followedEntityIds = [];
            foreach (EntityFeedRepository::findByUser($this) as $entityFeed) {
                $this->followedEntityIds[$entityFeed->getTargetId()] = $entityFeed->getTargetId();
            }
        }

        return $this->followedEntityIds;
    }

    /**
     * @todo
     * @param Research $entity
     */
    public function unfollowEntity(Research $entity)
    {
        if (!$this->followsEntity($entity)) {
            return;
        }

        $entityFeed = EntityFeedRepository::findOneByUserAndTarget($this, $entity);
        if (isset($entityFeed)) {
            EntityFeedRepository::remove($entityFeed);
        }
    }

    /**
     * @todo
     * @param User $user
     */
    public function followUser(User $user)
    {
        if ($this->followsUser($user)) {
            return;
        }

        $userFeed = new UserFeed();
        $userFeed->setUser($this);
        $userFeed->setTarget($user);
        UserFeedRepository::add($userFeed);
    }

    /**
     * @todo
     * @param User $user
     * @return bool
     */
    public function followsUser(User $user)
    {
        return UserFeedRepository::findOneByUserAndTarget($this, $user) !== null;
    }

    /**
     * @todo
     * @param User $user
     */
    public function unfollowUser(User $user)
    {
        $userFeed = UserFeedRepository::findOneByUserAndTarget($this, $user);
        if (!isset($userFeed)) {
            return;
        }

        UserFeedRepository::remove($userFeed);
    }

    /**
     * @todo
     * @param Portal $portal
     */
    public function followPortal(Portal $portal)
    {
        if ($this->followsPortal($portal)) {
            return;
        }

        $portalFeed = new PortalFeed();
        $portalFeed->setUser($this);
        $portalFeed->setTarget($portal);
        PortalFeedRepository::add($portalFeed);
    }

    /**
     * @todo
     * @param Portal $portal
     * @return bool
     */
    public function followsPortal(Portal $portal = null)
    {
        if (!isset($portal)) {
            return false;
        }

        return PortalFeedRepository::findOneByUserAndTarget($this, $portal) !== null;
    }

    /**
     * @todo
     * @param Portal $portal
     */
    public function unfollowPortal(Portal $portal)
    {
        $portalFeed = PortalFeedRepository::findOneByUserAndTarget($this, $portal);
        if (!isset($portalFeed)) {
            return;
        }

        PortalFeedRepository::remove($portalFeed);
    }

    /**
     * @param string $prefix
     * @return bool
     */
    protected function hasGroupByPrefix($prefix)
    {
        foreach ($this->getGroups() as $group) {
            if ($prefix == AclGroup::getGroupPrefix($group)) {
                return true;
            }
        }

        return false;
    }
}