<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\IdMapping;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\Work;

/**
 * @method static IdMapping[] find(array $query, array $options = [])
 * @method static IdMapping|null findById($id)
 * @method static IdMapping getById($id)
 * @method static IdMapping|null findOne(array $query = [], array $options = [])
 * @method static IdMapping[] findByIds(array $id, array $options = [])
 * @method static IdMapping[] findAll(array $options = [])
 */
class IdMappingRepository extends MongoRepository
{

    /**
     * @var IdMapping[]
     */
    protected static $cache = [];

    /**
     * @var array
     */
    protected static $typeAliases = [
        Research::class    => [
            Degree::class,
            Exposition::class,
            Project::class,
            Work::class,
        ],
        Tool::class        => [
            Tool::class,
            Tool\AudioTool::class,
            Tool\CommentTool::class,
            Tool\EmbedTool::class,
            Tool\ObjectviewerTool::class,
            Tool\PdfTool::class,
            Tool\PictureTool::class,
            Tool\PlayTool::class,
            Tool\ShapeTool::class,
            Tool\SimpletextTool::class,
            Tool\SlideshowTool::class,
            Tool\TextTool::class,
            Tool\VideoTool::class,
        ],
        SimpleMedia::class => [
            SimpleMedia::class,
            SimpleMedia\ArchiveSimpleMedia::class,
            SimpleMedia\AudioSimpleMedia::class,
            SimpleMedia\ImageSimpleMedia::class,
            SimpleMedia\PdfSimpleMedia::class,
            SimpleMedia\TextSimpleMedia::class,
            SimpleMedia\VideoSimpleMedia::class,
        ],
        Weave::class       => [
            GraphicalWeave::class,
            TextWeave::class,
        ],
    ];

    /**
     * @param string $type
     * @return bool
     */
    public static function isAbstractClass(string $type): bool
    {
        return isset(self::$typeAliases[$type]);
    }

    /**
     * @param string $className
     * @param int $id
     * @return IdMapping
     */
    public static function getByTypeAndReferenceId($className, $id)
    {
        $className = (string)$className;

        if (!isset(self::$cache[$className . '_' . $id])) {
            self::$cache[$className . '_' . $id] = self::findOne(
                [
                    'referenceType' => ['$in' => self::$typeAliases[$className] ?? (array)$className],
                    'referenceId'   => (int)$id,
                ],
                ['require' => 1]
            );
        }

        return self::$cache[$className . '_' . $id];
    }

    /**
     * @param int $id
     * @return IdMapping
     */
    public static function getByReferenceId($id)
    {
        if (!isset(self::$cache[$id])) {
            self::$cache[$id] = self::findOne([
                'referenceId' => (int)$id,
                'type'        => ['$ne' => Portal::class],
            ], ['require' => 1]);
        }
        return self::$cache[$id];
    }

    /**
     * @return int
     */
    public static function getMaxReferenceId()
    {
        return self::findOne([], ['sort' => ['referenceId' => self::SORT_DESC], 'require' => 1])->getReferenceId();
    }

    /**
     * @param int $id
     * @return string|MongoRepository
     */
    public static function getRepositoryNameByModelId(int $id): string
    {
        $idMapping = self::getByReferenceId($id);

        return static::getRepositoryNameByModel($idMapping->getReferenceType());
    }

    /**
     * @param string $className
     * @return string|MongoRepository
     */
    public static function getRepositoryNameByModel(string $className)
    {
        foreach (static::$typeAliases as $alias => $models) {
            if (in_array($className, $models)) {
                $className = $alias;
                break;
            }
        }

        return str_replace('Model', 'Repository', $className) . 'Repository';
    }
}
