<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\Message;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\User;

/**
 * @method static Message|null findById($id)
 * @method static Message getById($id)
 * @method static Message[] findByIds(array $id, array $options = [])
 * @method static Message[] findAll(array $options = [])
 */
class MessageRepository extends MongoRepository
{
    /**
     * @var array
     */
    protected static $defaultOrder = ['created' => self::SORT_DESC];

    /**
     * @param User $sender
     * @param array $options
     * @return Message[]
     */
    public static function findBySender(User $sender, array $options = [])
    {
        return self::find(['sender.uuid' => (string)$sender->getUuid(), 'deletedForSender' => null], $options);
    }

    /**
     * @param User $sender
     * @return int
     */
    public static function countBySender(User $sender)
    {
        return self::count(['sender.uuid' => (string)$sender->getUuid(), 'deletedForSender' => null]);
    }

    /**
     * @param User $receiver
     * @param array $options
     * @return Message[]
     */
    public static function findByReceiver(User $receiver, array $options = [])
    {
        return static::find(['receiver.uuid' => (string)$receiver->getUuid(), 'deletedForReceiver' => null], $options);
    }

    /**
     * @param User $receiver
     * @return int
     */
    public static function countByReceiver(User $receiver)
    {
        return self::count(['receiver.uuid' => (string)$receiver->getUuid(), 'deletedForReceiver' => null]);
    }

    /**
     * @param User $receiver
     * @param array $options
     * @return Message[]
     */
    public static function findUnreadByReceiver(User $receiver, array $options = [])
    {
        return self::find(['receiver.uuid' => (string)$receiver->getUuid(), 'deletedForReceiver' => null, 'read' => false], $options);
    }

    /**
     * @param Message $message
     */
    public static function removeForSender(Message $message)
    {
        $message->setDeletedForSender(time());
        self::persist($message);
    }

    /**
     * @param Message $message
     */
    public static function removeForReceiver(Message $message)
    {
        $message->setDeletedForReceiver(time());
        self::persist($message);
    }

    /**
     * @param User $user
     */
    public static function removeByUser(User $user)
    {
        self::removeCollection(self::findBySender($user));
        self::removeCollection(self::findByReceiver($user));
    }

    /**
     * @param User $user
     * @param Exposition $exposition
     * @param array $options
     * @return Message
     */
    public static function findLastPendingInvite(User $user, Exposition $exposition, array $options = [])
    {
        return self::findOne([
            'receiver.uuid'      => (string)$user->getUuid(),
            'deletedForReceiver' => null,
            'type'               => Message::TYPE_COLLABORATION_ADD,
            'content'            => [
                '$regex' => '.*:' . $exposition->getId() . ',.*',
            ],
        ], $options);
    }
}