<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Doctrine\Common\Annotations\AnnotationRegistry;
use Doctrine\Common\Cache\ArrayCache;
use Doctrine\Common\Cache\FilesystemCache;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\MongoDB\Connection;
use Doctrine\MongoDB\Query\Builder;
use Doctrine\ODM\MongoDB\Configuration;
use Doctrine\ODM\MongoDB\Cursor;
use Doctrine\ODM\MongoDB\DocumentManager;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\LockException;
use Doctrine\ODM\MongoDB\Mapping\Driver\AnnotationDriver;
use Doctrine\ODM\MongoDB\Types\Type;
use Identity;
use InsufficientRightsException;
use MyLocal;
use Rc\Doctrine\ODM\MongoDB\Type\StringType;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\AutoIncrement;
use Rc\Domain\Model\Core\Acl\AclAwareInterface;
use Rc\Domain\Model\Core\Acl\AclDelegatorAwareInterface;
use Rc\Domain\Model\Core\Revision\RevisionAwareInterface;
use Rc\Domain\Model\Core\Revision\RevisionFactory;
use Rc\Domain\Model\IdMapping;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Repository\Generic\Suggest;
use UserGroups;
use Zend_Registry;

class MongoRepository
{
    const TYPE_DEGREE = 'degree';
    const TYPE_EXPOSITION = 'exposition';
    const TYPE_FILE = 'file';
    const TYPE_PORTAL = 'portal';
    const TYPE_PROJECT = 'project';
    const TYPE_TOOL = 'tool';
    const TYPE_USER = 'user';
    const TYPE_WEAVE = 'weave';
    const TYPE_WORK = 'work';
    const TYPE_APPLICATION = 'application';
    const TYPE_APPLICATION_FORM = 'application-form';
    const TYPE_APPLICATION_PROGRAM = 'application-program';

    const SORT_ASC = 1;
    const SORT_DESC = -1;

    /**
     * @var DocumentManager
     */
    protected static $client;

    /**
     * @type array
     */
    protected static $defaultOrder = ['id' => self::SORT_ASC];

    /**
     * @type array
     */
    protected static $defaultPrimeProperties = [];

    /**
     * @var array
     */
    protected static $suggestPropertyList = ['title'];

    /**
     * @var bool
     */
    protected static $isFlushing = false;

    /**
     * @var IdMapping[]
     */
    protected static $newIdMappings = [];

    /**
     * @return string
     */
    public static function getDb()
    {
        return config('databases.mongodb.database');
    }

    /**
     * @param array $options
     * @return object[]
     */
    public static function findAll(array $options = [])
    {
        return static::find([], $options);
    }

    /**
     * @param int[] $ids
     * @param array $options
     * @return object[]
     */
    public static function findByIds(array $ids, array $options = [])
    {
        if (empty($ids)) {
            return [];
        }
        return static::find(['id' => ['$in' => static::castListToInteger($ids)]], $options);
    }

    /**
     * @param string[] $uuids
     * @param array $options
     * @return object[]
     */
    public static function findByUuids(array $uuids, array $options = [])
    {
        if (empty($uuids)) {
            return [];
        }
        return static::find(['_id' => ['$in' => static::castListToString($uuids)]], $options);
    }

    /**
     * @param object $object
     * @return void
     */
    public static function remove($object)
    {
        static::enforcePermissions($object);

        static::getClient()->remove($object);

        static::flush();
    }

    /**
     * @param \Traversable $collection
     * @return void
     */
    public static function removeCollection($collection)
    {
        foreach ($collection as $object) {
            static::remove($object);
        }
    }

    /**
     * @param int $id
     * @throws InsufficientRightsException
     * @return object
     */
    public static function getById($id)
    {
        $result = static::findById($id);

        if (!isset($result)) {
            throw new InsufficientRightsException('Could not find ' . static::getModelName() . '#' . $id . '!');
        }

        return $result;
    }

    /**
     * @param int $id
     * @return null|ModelInterface
     * @throws \Exception
     */
    public static function findById($id)
    {
        if (empty($id)) {
            return null;
        }

        if (get_class() == get_called_class()) {
            return (IdMappingRepository::getRepositoryNameByModelId($id))::findById($id);
        }

        if (is_numeric($id)) {
            $object = static::findOne(['id' => (int)$id]);
        } else {
            $object = static::findOne(['uuid' => (string)$id]);
        }

        static::enforcePermissions($object, AclAwareInterface::TYPE_READ);

        return $object;
    }

    /**
     * @param ModelInterface $object
     * @param bool $flush
     */
    public static function persist(ModelInterface $object, $flush = true)
    {
        static::enforcePermissions($object);

        if ($object instanceof RevisionAwareInterface) {
            RevisionFactory::create($object);
        }

        static::getClient()->persist($object);

        if ($flush) {
            static::flush();
        }
    }

    /**
     * @param ModelInterface[] $objects
     * @param bool $flush
     */
    public static function persistCollection($objects, $flush = true)
    {
        foreach ($objects as $object) {
            static::persist($object, false);
        }

        if ($flush) {
            static::flush();
        }
    }

    /**
     * @return void
     */
    public static function flush()
    {
        static::$isFlushing = true;

        static::getClient()->flush();

        if (!empty(static::$newIdMappings)) {
            static::persistCollection(static::$newIdMappings, false);
            static::getClient()->flush();
            static::$newIdMappings = [];
        }

        static::$isFlushing = false;
    }

    /**
     * @return void
     */
    public static function clear()
    {
        static::getClient()->clear();
    }

    /**
     * @param ModelInterface $object
     */
    public static function detach(ModelInterface $object)
    {
        static::getClient()->detach($object);
    }

    /**
     * @param ModelInterface $object
     * @return object
     * @todo: if we don't clone object it will not be persisted, so changes are lost after reload
     * @throws LockException
     */
    public static function merge(ModelInterface $object)
    {
        /** @var AbstractModel $newObject */
        $newObject = clone static::getClient()->merge($object);
        $newObject->setNew(false);
        $newObject->setUuid($object->getUuid());
        $newObject->setId($object->getId());

        static::persist($newObject);

        return $newObject;
    }

    /**
     * @return DocumentManager
     */
    public static function getClient()
    {
        if (!isset(static::$client)) {
            AnnotationRegistry::registerLoader(function ($class) {
                return class_exists($class);
            });

            if (app_env('production')) {
                $cache = new FilesystemCache(cache_path('doctrine-metadata'));
            } else {
                $cache = new ArrayCache();
            }

            $config = new Configuration();
            $config->setProxyDir(cache_path('doctrine-proxies'));
            $config->setProxyNamespace('Proxies');
            $config->setHydratorDir(cache_path('doctrine-hydrators'));
            $config->setHydratorNamespace('Hydrators');
            $config->setDefaultDB(static::getDb());
            $config->setMetadataDriverImpl(AnnotationDriver::create(base_path('src/Domain/Model')));
            $config->setMetadataCacheImpl($cache);
            $config->setLoggerCallable(function (array $log) {
                // print_r($log);
            });

            Type::overrideType('string', StringType::class);

            static::$client = DocumentManager::create(new Connection(null, [], $config), $config);
        }
        return static::$client;
    }

    /**
     * @return void
     */
    public static function ensureIndexes()
    {
        static::getClient()->getSchemaManager()->updateIndexes();
        static::getClient()->getSchemaManager()->ensureIndexes();
    }

    /**
     * @param string $id
     * @return int
     */
    public static function enforceAutoIncrementId($id)
    {
        /** @var AutoIncrement $autoIncrement */
        $autoIncrement = static::getClient()->createQueryBuilder()
            ->findAndUpdate(AutoIncrement::class)
            // find
            ->field('id')->equals((string)$id)
            // update
            ->field('value')->inc(1)
            ->upsert(true)
            // select
            ->select('value')
            ->returnNew(true)
            ->getQuery()
            ->execute();

        return $autoIncrement->getValue();
    }

    /**
     * @param IdMapping $idMapping
     */
    public static function persistIdMapping(IdMapping $idMapping)
    {
        static::$newIdMappings[] = $idMapping;
    }

    /**
     * @return \Doctrine\ODM\MongoDB\Mapping\ClassMetadata
     */
    public static function getClassMetaData()
    {
        return static::getRepository()->getClassMetadata();
    }

    /**
     * @param array $query
     * @param array $options
     * @return mixed
     */
    public static function findOne(array $query = [], array $options = [])
    {
        $options['limit'] = 1;

        $result = static::find($query, $options);
        if (empty($result)) {
            return null;
        }

        return $result[0];
    }

    /**
     * @param array $query
     * @param array $options
     * @throws Exception
     * @return array
     */
    public static function find(array $query, array $options = [])
    {
        /** @var Cursor $result */
        $result = static::createQueryBuilder($query, $options)->getQuery()->execute();

        if (!empty($options['require']) && $options['require'] != $result->count(true)) {
            throw new Exception(
                'Result set with ' . $result->count(true) . ' does not match required amount of ' . $options['require'] .
                ' [type:' . get_called_class() . ', query: ' . var_export($query, true) . ']!'
            );
        }

        return $result->toArray(false);
    }

    /**
     * @param array $query
     * @param array $options
     * @return int
     */
    public static function count(array $query, array $options = [])
    {
        return static::createQueryBuilder($query, $options)
            ->count()
            ->getQuery()
            ->execute();
    }

    /**
     * @param string $field
     * @param string $searchTerm
     * @param array $options
     * @return Suggest[]
     */
    public static function suggest($field, $searchTerm, array $options = [])
    {
        if (!in_array($field, static::$suggestPropertyList)) {
            $field = reset(static::$suggestPropertyList);
        }

        if (static::isMultiLanguageField($field)) {
            $query = static::buildSearchRegexForTranslationList($field, $searchTerm);
        } else {
            $query = static::buildSearchRegex($field, $searchTerm);
        }

        if (!empty($options['isOwner'])) {
            $query['$and'][] = ['owner.uuid' => Identity::getUserUuid()];
        }

        if (!empty($options['approvedOnly'])) {
            $query['$and'][] = ['approved' => true];
        }

        /** @var AbstractModel[] $researches */
        $researches = static::createQueryBuilder($query, $options + ['excludeHidden' => true, 'excludeDeleted' => true, 'sort' => [$field => static::SORT_ASC]])
            ->getQuery()
            ->execute();

        $result = [];
        foreach ($researches as $research) {
            $value = $research->{'get' . $field}();
            if (!is_array($value) && !$value instanceof \Traversable) {
                $result[] = new Suggest($value, $research);
                continue;
            }
            foreach ($value as $singleValue) {
                if (!preg_match('~(^| )' . preg_quote(strtolower($searchTerm)) . '.*~i', $singleValue)) {
                    continue;
                }
                $result[] = new Suggest($singleValue, $research);
            }
        }
        return $result;
    }

    /**
     * @param Collection $collection
     * @param bool $ignoreNonExistingDocuments
     * @return ArrayCollection
     */
    public static function initializeCollection(Collection $collection, $ignoreNonExistingDocuments = true)
    {
        $existingDocuments = [];

        foreach ($collection as $item) {
            try {
                static::getClient()->getUnitOfWork()->initializeObject($item);
                $existingDocuments[] = $item;
            } catch (DocumentNotFoundException $exception) {
                if (!$ignoreNonExistingDocuments) {
                    throw $exception;
                }
            }
        }

        return new ArrayCollection($existingDocuments);
    }

    /**
     * @return string
     */
    public static function getModelName()
    {
        return str_replace('Repository', 'Model', substr(get_called_class(), 0, -strlen('Repository')));
    }

    /**
     * @param ModelInterface $model
     * @return string
     */
    public static function getClassName(ModelInterface $model)
    {
        return static::getClient()->getClassMetadata(get_class($model))->name;
    }

    /**
     * @return \Doctrine\ODM\MongoDB\DocumentRepository
     */
    protected static function getRepository()
    {
        return static::getClient()->getRepository(static::getModelName());
    }

    /**
     * @param array $list
     * @return int[]
     */
    protected static function castListToInteger(array $list)
    {
        return array_map(function ($value) {
            return (int)$value;
        }, array_values($list));
    }

    /**
     * @param array $list
     * @return string[]
     */
    protected static function castListToString(array $list)
    {
        return array_map(function ($value) {
            return (string)$value;
        }, $list);
    }

    /**
     * todo: replace hardcoded condition with dynamic solution using the doctrine class schema manager
     *
     * @param string $field
     * @return bool
     */
    protected static function isMultiLanguageField($field)
    {
        return $field == 'title' || $field == 'description';
    }

    /**
     * @param array $query
     * @param array $options
     * @return Builder
     */
    protected static function createQueryBuilder(array $query, array $options)
    {
        if (!isset($options['sort'])) {
            $options['sort'] = static::$defaultOrder;
        }

        if (!empty($options['require'])) {
            $options['limit'] = $options['require'];
        }

        if (!empty($options['excludeHidden'])) {
            $query = static::combineQueryConditions($query, [
                'hide' => false,
            ]);
        }

        if (!empty($options['excludeDeleted'])) {
            $query = static::combineQueryConditions($query, [
                'deleted' => false,
            ]);
        }

        if (isset($options['ids'])) {
            $query = static::combineQueryConditions($query, [
                'id' => ['$in' => static::castListToInteger($options['ids'])],
            ]);
        }

        if (is_a(get_called_class(), SingleCollectionInheritanceInterface::class, true)) {
            $query = static::combineQueryConditions($query, [
                '_class' => static::getModelName(),
            ]);
        }

        if (Identity::exists() &&
            Zend_Registry::get('RC_DILPS_AUTH')->getUID() != 'rc/admin' &&
            is_a(static::getModelName(), AclAwareInterface::class, true)
        ) {

            $query = static::combineQueryConditions($query, [
                '$or' => [
                    ['id' => ['$in' => Zend_Registry::get('RC_DILPS_AUTH')->getTemporaryGroups(UserGroups::TYPE_ENTITY_ID)]],
                    ['aclv2.user.read' => ['$in' => Identity::getUser()->getGroups()]],
                ],
            ]);
        }

        $queryBuilder = static::getRepository()
            ->createQueryBuilder()
            ->setQueryArray($query)
            ->sort($options['sort']);

        if (isset($options['limit'])) {
            $queryBuilder->limit($options['limit']);
        }

        if (isset($options['page'])) {
            $queryBuilder->skip((int)$options['page'] * $options['limit']);
        }

        if (isset($options['prime']) && $options['prime']) {
            if ($options['prime'] === true) {
                $options['prime'] = static::$defaultPrimeProperties;
            }
            foreach ($options['prime'] as $prime) {
                $queryBuilder->field($prime)->prime();
            }
        }

        return $queryBuilder;
    }

    /**
     * @param ModelInterface|null $object
     * @param string $type
     * @throws InsufficientRightsException
     */
    protected static function enforcePermissions(ModelInterface $object = null, $type = AclAwareInterface::TYPE_WRITE)
    {
        if ($object instanceof AclDelegatorAwareInterface) {
            $object = $object->getAclEntity();
        }

        if (!$object instanceof AclAwareInterface) {
            return;
        }

        if (Zend_Registry::get('RC_DILPS_AUTH')->getUID() == 'rc/admin') {
            return;
        }

        if (Identity::exists() && ($object->isOwner(Identity::getUser()) || $object->wasOwner(Identity::getUser()))) {
            return;
        }

        if (!$object->isAllowed(Identity::getUserOrNull(), $type)) {
            throw new InsufficientRightsException('Insufficient rights to access ' . $object . ($type ? ' (readonly)' : '') . '!');
        }
    }

    /**
     * @param string $field
     * @param string $rawSearchTerm
     * @return array
     */
    protected static function buildSearchRegex($field, $rawSearchTerm)
    {
        $rawSearchTerms = array_filter(explode(' ', $rawSearchTerm));
        if (empty($rawSearchTerms)) {
            return [];
        }

        $conditions = [];
        foreach ($rawSearchTerms as $searchTerm) {
            $conditions[] = [$field => ['$regex' => '(^| )' . preg_quote(strtolower($searchTerm)) . '.*', '$options' => 'i']];
        }
        return ['$and' => $conditions];
    }

    /**
     * @param string $field
     * @param string $rawSearchTerm
     * @return array
     */
    protected static function buildSearchRegexForTranslationList($field, $rawSearchTerm)
    {
        $conditions = [];
        foreach (array_filter(explode(' ', $rawSearchTerm)) as $searchTerm) {
            $languageCondition = [];
            foreach (array_keys(MyLocal::getTranslationList('language')) as $language) {
                $languageCondition[] = [
                    $field . '.' . $language => [
                        '$regex'   => '(^| )' . preg_quote(strtolower($searchTerm)) . '.*',
                        '$options' => 'i',
                    ],
                ];
            }
            $conditions[] = ['$or' => $languageCondition];
        }
        return ['$and' => $conditions];
    }

    /**
     * @param array $query1
     * @param array $query2
     * @param string $operator
     * @return array
     */
    protected static function combineQueryConditions(array $query1, array $query2, $operator = '$and')
    {
        if (empty($query1)) {
            return $query2;
        }
        if (empty($query2)) {
            return $query1;
        }
        return [
            $operator => [
                $query1,
                $query2,
            ],
        ];
    }

    /**
     * @param array $list
     * @return string[]
     */
    protected static function castListToUuidList(array $list)
    {
        return array_map(function ($value) {
            return (string)($value instanceof AbstractModel ? $value->getUuid() : $value);
        }, $list);
    }
}
