<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Repository\Research;

use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Generic\SearchResult;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\SingleCollectionInheritanceInterface;

/**
 * @method static Project|null findById($id)
 * @method static Project getById($id)
 * @method static Project[] findByIds(array $id, array $options = [])
 * @method static Project[] findAll(array $options = [])
 */
class ProjectRepository extends ResearchRepository implements SingleCollectionInheritanceInterface
{

    /**
     * @param array $arguments
     * @param array $options
     * @return Project[]|SearchResult
     */
    public static function search(array $arguments, array $options = [])
    {
        $queryBuilder = self::createQueryBuilder([], $options);

        // title
        if (isset($arguments['title'])) {
            $queryBuilder->addAnd(self::buildSearchRegexForTranslationList('title', $arguments['title']));
        }

        // genre
        if (isset($arguments['genre'])) {
            $queryBuilder->addAnd(['genre' => (string)$arguments['genre']]);
        }

        // area
        if (isset($arguments['area'])) {
            $queryBuilder->addAnd(['area' => (string)$arguments['area']]);
        }

        // main field of study
        if (isset($arguments['study'])) {
            $queryBuilder->addAnd(['mainFieldOfStudy' => (string)$arguments['study']]);
        }

        // keywords
        if (isset($arguments['keyword'])) {
            $queryBuilder->addAnd(['keywords' => ['$in' => [(string)$arguments['keyword']]]]);
        }

        // principal investigator
        if (isset($arguments['investigator'])) {
            $principalInvestigatorUuid = IdMappingRepository::getByTypeAndReferenceId(User::class, $arguments['investigator'])->getReferenceUuid();
            $queryBuilder->addAnd(['principalInvestigator.uuid' => $principalInvestigatorUuid]);
        }

        // contributor
        if (isset($arguments['contributor'])) {
            $contributorUuid = IdMappingRepository::getByTypeAndReferenceId(User::class, $arguments['contributor'])->getReferenceUuid();
            $queryBuilder->addAnd(['contributors.uuid' => $contributorUuid]);
        }

        // supervisor
        if (isset($arguments['supervisor'])) {
            $supervisorUuid = IdMappingRepository::getByTypeAndReferenceId(User::class, $arguments['supervisor'])->getReferenceUuid();
            $queryBuilder->addAnd(['contributors.uuid' => $supervisorUuid]);
        }

        // fulltext
        if (isset($arguments['fulltext'])) {
            $queryBuilder->text((string)$arguments['fulltext']);
            $queryBuilder->sortMeta('score', 'textScore');
        }

        // accessible condition
        if (!empty($arguments['accessible'])) {
            $queryBuilder->addAnd(self::getIsAccessibleCondition($arguments['accessible'][0], $arguments['accessible'][1]));
        } else {
            $queryBuilder->addAnd(self::getIsAccessibleCondition());
        }

        return new SearchResult($queryBuilder);
    }

}