<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Identity;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Model\User;

/**
 * @method static Research|null findById($id)
 * @method static Research getById($id)
 * @method static Research[] findByIds(array $id, array $options = [])
 * @method static Research[] findAll(array $options = [])
 */
class ResearchRepository extends MongoRepository
{

    /**
     * @param array $ids
     * @param array $options
     * @return Research[]
     */
    public static function findAccessibleByIds(array $ids, array $options = [])
    {
        return self::find([
            'id'   => ['$in' => self::castListToInteger($ids)],
            '$and' => [self::getIsAccessibleCondition(true, true)],
        ], $options);
    }

    /**
     * @param array $ids
     * @param array $options
     * @return Research[]
     */
    public static function findPublicAccessibleByIds(array $ids, array $options = [])
    {
        return self::find([
            'id'   => ['$in' => self::castListToInteger($ids)],
            '$and' => [self::getIsAccessibleCondition(false)],
        ], $options);
    }

    /**
     * @param array $options
     * @return Research[]
     */
    public static function findPublic(array $options = [])
    {
        return self::find([
            'blocked'     => false,
            'hide'        => false,
            '$and'        => [self::getIsAccessibleCondition(false)],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Research
     */
    public static function findOnePublicByPerson(User $user, array $options = [])
    {
        return self::findOne([
            'blocked' => false,
            'hide'    => false,
            '$and'    => [self::getIsAccessibleCondition(false, false)],
            '$or'     => [
                ['owner.uuid' => (string)$user->getUuid()],
                ['coAuthors.uuid' => (string)$user->getUuid()],
                ['contributors.uuid' => (string)$user->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Research[]
     */
    public static function findByPortalForReviewing(Portal $portal, array $options = [])
    {
        return self::findByPortalIdForReviewing($portal->getUuid(), $options);
    }

    /**
     * @param string $portalId
     * @param array $options
     * @return Research[]
     */
    public static function findByPortalIdForReviewing($portalId, array $options = [])
    {
        return self::findByPortalIdAndStatus($portalId, StatusAwareInterface::STATUS_REVIEW, $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Research[]
     */
    public static function findByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or' => [
                ['owner.uuid' => (string)$user->getUuid()],
                ['contributors.uuid' => (string)$user->getUuid()],
                ['coAuthors.uuid' => (string)$user->getUuid()],
                ['supervisors.uuid' => (string)$user->getUuid()],
                ['editors.uuid' => (string)$user->getUuid()],
                ['groups.uuid' => (string)$user->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Research[]
     */
    public static function findCoAuthoredByPerson(User $user, array $options = [])
    {
        return self::find([
            'owner.uuid' => ['$ne' => (string)$user->getUuid()],
            '$or'        => [
                ['coAuthors.uuid' => (string)$user->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Research[]
     */
    public static function findCollaboratedByPerson(User $user, array $options = [])
    {
        return self::find([
            'owner.uuid' => ['$ne' => (string)$user->getUuid()],
            '$or'        => [
                ['contributors.uuid' => (string)$user->getUuid()],
                ['coAuthors.uuid' => (string)$user->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Research[]
     */
    public static function findSupervisedByPerson(User $user, array $options = [])
    {
        return self::find([
            'owner.uuid'       => ['$ne' => (string)$user->getUuid()],
            'supervisors.uuid' => (string)$user->getUuid(),
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Research[]
     */
    public static function findPendingByPartOf(Portal $portal, array $options = [])
    {
        return self::find([
            'partOfPortalRequest.uuid' => (string)$portal->getUuid(),
            'partOfPortal.uuid'        => ['$ne' => (string)$portal->getUuid()],
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Research[]
     */
    public static function findConnected(Portal $portal, array $options = [])
    {
        return self::find([
            'partOfPortal.uuid' => (string)$portal->getUuid(),
        ], $options);
    }

    /**
     * @param Research $research
     * @param array $options
     * @return Research[]
     */
    public static function findRelations(Research $research, array $options = [])
    {
        return self::find([
            'relations' => ['$elemMatch' => ['$id' => new \MongoId($research->getUuid())]],
        ], $options);
    }

    /**
     * @param string $name
     * @param array $options
     * @return Research[]
     */
    public static function findByName($name, array $options = [])
    {
        return self::find(self::buildSearchRegexForTranslationList('title', $name), $options);
    }

    /**
     * @param string $value
     * @param array $options
     * @return Research[]
     */
    public static function findByTitleOrKeyword($value, array $options = [])
    {
        return self::find([
            '$or' => [
                ['keywords' => [
                    '$regex'   => '.*' . preg_quote($value) . '.*',
                    '$options' => 'i',
                ]],
            ],
        ], $options);
    }

    /**
     * @param string $portalId
     * @param string $status
     * @param array $options
     * @return Research[]
     */
    protected static function findByPortalIdAndStatus($portalId, $status, array $options = [])
    {
        return self::find([
            '$or' => [
                [
                    'portal.uuid' => (string)$portalId,
                    'status'      => (string)$status,
                ],
                [
                    'additionalPortals.uuid' => (string)$portalId,
                    'status'                 => StatusAwareInterface::STATUS_REVIEW_PUBLISHED,
                ],
            ],
        ], $options);
    }

    /**
     * @param bool|true $includePrivate
     * @param bool|true $evalIdentity
     * @return array
     */
    protected static function getIsAccessibleCondition($includePrivate = true, $evalIdentity = true)
    {
        $conditions = [
            ['accessLevel' => Exposition::ACCESS_LEVEL_PUBLIC],
            [
                'status'       => ['$in' => [Exposition::STATUS_PUBLISHED, Exposition::STATUS_REVIEW_PUBLISHED]],
                'publishRange' => ['$ne' => 'limited'],
                '_class'       => ['$ne' => Application::class],
            ],
        ];
        if ($evalIdentity && Identity::exists()) {
            $conditions[] = ['accessLevel' => Exposition::ACCESS_LEVEL_LIMITED];
            if ($includePrivate) {
                $conditions[] = [
                    'accessLevel' => Exposition::ACCESS_LEVEL_PRIVATE,
                    'owner.uuid'  => Identity::getUser()->getUuid(),
                ];
            }
            if (Identity::getUser()->isMemberOfNonDefaultPortal()) {
                $conditions[] = [
                    'accessLevel'             => Exposition::ACCESS_LEVEL_PORTAL,
                    'accessLevelPortals.uuid' => ['$in' => self::castListToUuidList(Identity::getUser()->getPortals())],
                ];
                $conditions[] = [
                    'status'                  => [
                        '$in' => [
                            Exposition::STATUS_PUBLISHED,
                            Exposition::STATUS_REVIEW_PUBLISHED
                        ]
                    ],
                    'accessLevelPortals.uuid' => ['$in' => self::castListToUuidList(Identity::getUser()->getPortals())],
                ];
            }
        }
        return ['$or' => $conditions];
    }
}