<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Handler;

use Doctrine\Common\Util\ClassUtils;
use JMS\Serializer\DeserializationContext;
use JMS\Serializer\GraphNavigator;
use JMS\Serializer\Handler\SubscribingHandlerInterface;
use JMS\Serializer\SerializationContext;
use JMS\Serializer\VisitorInterface;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Repository\MongoRepository;

class Id implements SubscribingHandlerInterface
{

    /**
     * @var array
     */
    protected $mapping = [];

    /**
     * Return format:
     *      array(
     *          array(
     *              'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
     *              'format' => 'json',
     *              'type' => 'DateTime',
     *              'method' => 'serializeDateTimeToJson',
     *          ),
     *      )
     * The direction and method keys can be omitted.
     *
     * @return array
     */
    public static function getSubscribingMethods()
    {
        $methods = [];
        $formats = ['json', 'xml', 'yml'];

        foreach ($formats as $format) {
            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_SERIALIZATION,
                'type'      => \Rc\Serializer\Type\Id::class,
                'format'    => $format,
                'method'    => 'serialize',
            ];

            $methods[] = [
                'direction' => GraphNavigator::DIRECTION_DESERIALIZATION,
                'type'      => \Rc\Serializer\Type\Id::class,
                'format'    => $format,
                'method'    => 'deserialize',
            ];
        }

        return $methods;
    }

    /**
     * @param VisitorInterface     $visitor
     * @param int                  $id
     * @param array                $type
     * @param SerializationContext $context
     * @return array
     */
    public function serialize(VisitorInterface $visitor, $id, array $type, SerializationContext $context)
    {
        return [
            '$id'    => $id,
            '$class' => ClassUtils::getClass($context->getObject()),
        ];
    }

    /**
     * @param VisitorInterface       $visitor
     * @param array                  $data
     * @param array                  $type
     * @param DeserializationContext $context
     * @return mixed|object
     */
    public function deserialize(VisitorInterface $visitor, array $data, array $type, DeserializationContext $context)
    {
        if (!$context->attributes->get('duplicate')->getOrElse(false)) {
            return $data['$id'];
        }

        return $this->map($data['$id']);
    }

    /**
     * @param int $id
     * @return int
     */
    public function map($id)
    {
        if (!isset($this->mapping[$id])) {
            $this->mapping[$id] = MongoRepository::enforceAutoIncrementId(ModelInterface::class);
        }

        return $this->mapping[$id];
    }

    /**
     * @return mixed
     */
    public function originalExpositionId()
    {
        // the exposition id should be the first entry in the mapping list because it was processed first
        reset($this->mapping);
        return key($this->mapping);
    }
}
