<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer\Metadata\Driver;

use Doctrine\Common\Annotations\Reader;
use Doctrine\ODM\MongoDB\Mapping\Annotations\EmbedMany;
use Doctrine\ODM\MongoDB\Mapping\Annotations\EmbedOne;
use Doctrine\ODM\MongoDB\Mapping\Annotations\Field;
use Doctrine\ODM\MongoDB\Mapping\Annotations\ReferenceMany;
use Doctrine\ODM\MongoDB\Mapping\Annotations\ReferenceOne;
use JMS\Serializer\Metadata\PropertyMetadata;
use Metadata\Driver\DriverInterface;
use Rc\Serializer\Type\Embed;
use Rc\Serializer\Type\Raw;
use Rc\Serializer\Type\Reference;

class AnnotationDriver implements DriverInterface
{
    /**
     * @var Reader
     */
    protected $reader;

    /**
     * @var \JMS\Serializer\Metadata\Driver\AnnotationDriver
     */
    protected $originalAnnotationDriver;

    /**
     * @var array
     */
    protected $fieldMapping = [
        'bool'       => 'boolean',
        'boolean'    => 'boolean',
        'collection' => 'array',
        'custom_id'  => 'string',
        'date'       => 'DateTime',
        'float'      => 'float',
        'double'     => 'float',
        'hash'       => 'array',
        'id'         => 'string',
        'int'        => 'integer',
        'integer'    => 'integer',
        'key'        => 'string',
        'object_id'  => 'string',
        'string'     => 'string',
        'timestamp'  => 'int',
        'raw'        => Raw::class,
    ];

    /**
     * @param Reader $reader
     */
    public function __construct(Reader $reader)
    {
        $this->reader = $reader;
        $this->originalAnnotationDriver = new \JMS\Serializer\Metadata\Driver\AnnotationDriver($reader);
    }

    /**
     * @param \ReflectionClass $class
     * @return \Metadata\ClassMetadata
     */
    public function loadMetadataForClass(\ReflectionClass $class)
    {
        $metadata = $this->originalAnnotationDriver->loadMetadataForClass($class);

        /** @var PropertyMetadata $propertyMetadata */
        foreach ($metadata->propertyMetadata as $property => $propertyMetadata) {
            if ($propertyMetadata->type) {
                continue;
            }

            foreach ($this->reader->getPropertyAnnotations($propertyMetadata->reflection) as $annotation) {
                if ($annotation instanceof Field) {
                    $type = $this->getFieldType($annotation);
                } elseif ($annotation instanceof ReferenceOne) {
                    $type = Reference::class;
                } elseif ($annotation instanceof ReferenceMany) {
                    $type = 'ArrayCollection<' . Reference::class . '>';
                } elseif ($annotation instanceof EmbedOne) {
                    $type = Embed::class;
                } elseif ($annotation instanceof EmbedMany) {
                    $type = 'ArrayCollection<' . Embed::class . '>';;
                }

                if (isset($type)) {
                    $propertyMetadata->setType($type);
                    break;
                }
            }
        }

        return $metadata;
    }

    /**
     * @param Field $field
     * @return string
     */
    protected function getFieldType(Field $field)
    {
        if (!isset($this->fieldMapping[$field->type])) {
            throw new \RuntimeException('Cannot map field type ' . $field->type . '!');
        }

        return $this->fieldMapping[$field->type];
    }

}
