<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Serializer;

use JMS\Serializer\Accessor\DefaultAccessorStrategy;
use JMS\Serializer\DeserializationContext;
use JMS\Serializer\EventDispatcher\EventDispatcherInterface;
use JMS\Serializer\Handler\HandlerRegistry;
use JMS\Serializer\JsonDeserializationVisitor;
use JMS\Serializer\JsonSerializationVisitor;
use JMS\Serializer\Naming\CamelCaseNamingStrategy;
use JMS\Serializer\Naming\SerializedNameAnnotationStrategy;
use JMS\Serializer\SerializationContext;
use JMS\Serializer\SerializerBuilder;
use Rc\Serializer\Builder\AnnotationDriverFactory;
use Rc\Serializer\Handler\Embed;
use Rc\Serializer\Handler\Id;
use Rc\Serializer\Handler\Owner;
use Rc\Serializer\Handler\Reference;
use Rc\Serializer\Handler\Tool;
use Rc\Serializer\Handler\Uuid;
use Rc\Serializer\Handler\Weave;
use Rc\Serializer\Listener\AllowCycles;
use Rc\Serializer\Visitor\JsonDeserializationVisitor as CustomJsonDeserializationVisitor;

class Serializer
{

    /**
     * @var string
     */
    protected $format = 'json';

    /**
     * @var array
     */
    protected $listeners = [
        AllowCycles::class,
    ];

    /**
     * @var array
     */
    protected $handlers = [
        Id::class,
        Owner::class,
        Reference::class,
        Uuid::class,
        Embed::class,
        Tool::class,
        Weave::class,
    ];

    /**
     * Serializes the given data to the specified output format.
     *
     * @param mixed                $data
     * @param SerializationContext $context
     * @return string
     */
    public function serialize($data, SerializationContext $context = null)
    {
        if (!isset($context)) {
            $context = new SerializationContext();
            $context->setSerializeNull(true);
        }

        return $this->createSerializer()->serialize($data, $this->format, $context);
    }

    /**
     * Deserializes the given data to the specified type.
     *
     * @param string                 $data
     * @param string                 $type
     * @param DeserializationContext $context
     * @return mixed
     */
    public function deserialize($data, $type, DeserializationContext $context = null)
    {
        if (isset($context)) {
            $duplicate = $context->attributes->get('duplicate')->getOrElse(false);
        } else {
            $duplicate = false;
        }

        return $this->createSerializer($duplicate)->deserialize($data, $type, $this->format, $context);
    }

    /**
     * @param boolean $duplicate
     * @return \JMS\Serializer\Serializer
     */
    protected function createSerializer($duplicate = false)
    {
        $accessorStrategy = $this->getAccessorStrategy();
        $propertyNamingStrategy = $this->getPropertyNamingStrategy();

        if ($duplicate) {
            $deserializationVisitor = new CustomJsonDeserializationVisitor($propertyNamingStrategy, $accessorStrategy);
        } else {
            $deserializationVisitor = new JsonDeserializationVisitor($propertyNamingStrategy, $accessorStrategy);
        }

        return SerializerBuilder::create()
            ->setMetadataDriverFactory(new AnnotationDriverFactory())
            ->setSerializationVisitor('json', new JsonSerializationVisitor($propertyNamingStrategy, $accessorStrategy))
            ->setDeserializationVisitor('json', $deserializationVisitor)
            ->addDefaultHandlers()
            ->addDefaultListeners()
            ->configureListeners(function (EventDispatcherInterface $eventDispatcher) {
                foreach ($this->listeners as $listener) {
                    $eventDispatcher->addSubscriber(new $listener);
                }
            })
            ->configureHandlers(function (HandlerRegistry $registry) {
                foreach ($this->handlers as $handler) {
                    $registry->registerSubscribingHandler(new $handler($registry));
                }
            })
            ->build();
    }

    /**
     * @return SerializedNameAnnotationStrategy
     */
    protected function getPropertyNamingStrategy()
    {
        return new SerializedNameAnnotationStrategy(new CamelCaseNamingStrategy());
    }

    /**
     * @return DefaultAccessorStrategy
     */
    protected function getAccessorStrategy()
    {
        return new DefaultAccessorStrategy();
    }
}
