<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service\Ingest;

use Predis\Client;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\Media\Resource\Type\RoleType;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\DimensionAwareInterface;

class ImageIngestService extends IngestService
{

    /**
     * @var Client
     */
    protected static $redisClient;

    /**
     * @return Client
     */
    public static function getRedisClient()
    {
        if (!isset(self::$redisClient)) {
            self::$redisClient = new Client();
            self::$redisClient->select(config('cache.ingest_service.database'));
        }
        return self::$redisClient;
    }

    /**
     * @param ImageResource $resource
     * @param DimensionAwareInterface $dimension
     * @return ImageResource
     */
    public static function createCachedResource($resource, $dimension)
    {
        $cacheResource = self::processDerivative($resource, $dimension->getWidth(), $dimension->getHeight(), RoleType::CACHE);

        if (isset($cacheResource)) {
            self::getRedisClient()->hset($resource->getCheckSum(), (string)$dimension, serialize($cacheResource));
        }

        return $cacheResource;
    }

    /**
     * @param ImageResource $resource
     * @param DimensionAwareInterface $dimension
     * @param bool $createIfMissing
     * @return ImageResource
     */
    public static function loadCachedResource($resource, $dimension, $createIfMissing = false)
    {
        $serializedResource = self::getRedisClient()->hget($resource->getCheckSum(), (string)$dimension);
        if (!empty($serializedResource)) {
            return unserialize($serializedResource);
        }

        if ($createIfMissing) {
            return self::createCachedResource($resource, $dimension);
        }

        return null;
    }

    /**
     * @param ImageResource $master
     * @return ImageResource
     */
    public static function processSplashScreen($master)
    {
        return self::processDerivative($master, $master->getWidth(), $master->getHeight(), RoleType::SPLASH_SCREEN);
    }

    /**
     * @param \Rc\Domain\Model\Media\Resource $master
     * @return \Rc\Domain\Model\Media\Resource[]
     */
    public static function processDerivativeList($master)
    {
        return [];
    }

    /**
     * @return ImageResource
     */
    public static function getResourcePrototype()
    {
        return new ImageResource();
    }

    /**
     * @param ImageResource $resource
     */
    protected static function enrichResource($resource)
    {
        $cmd = str_replace('SOURCE', $resource->getUri(), config('ingest.identify'));
        exec($cmd, $result);

        if (preg_match(config('ingest.identify_regex'), implode(' ', $result), $matches)) {
            $resource->setWidth(intval($matches[2]));
            $resource->setHeight(intval($matches[3]));
        }
    }

    /**
     * @param ImageResource $master
     * @param int $width
     * @param int $height
     * @param string $roleType
     * @return ImageResource|null
     */
    protected static function processDerivative($master, $width, $height, $roleType = RoleType::DERIVATIVE)
    {
        if (in_array($master->getFileExtension(), ['gif', 'svg'])) {
            /** @noinspection PhpIncompatibleReturnTypeInspection */
            return static::processResource($master->getUri(), new RoleType($roleType));
        }

        // if transform-target is bigger than current image, use current image size
        if ($width > $master->getWidth() && $height > $master->getHeight()) {
            $width = $master->getWidth();
            $height = $master->getHeight();
        }

        // get aspect ration of target image and set resize parameters
        $ratio = $master->getWidth() / $master->getHeight();
        if ($ratio > $width / $height) {
            $n_width = $width;
            $n_height = round($n_width / $ratio);
        } else {
            $n_height = $height;
            $n_width = round($n_height * $ratio);
        }

        $param = " -resize {$n_width}x{$n_height}";

        $output = shell_exec('identify -format "compression=%m&type=%[type]&colorspace=%[colorspace]" ' . escapeshellarg($master->getUri()) . ' 2> /dev/null');
        parse_str($output, $identityData);

        if (!empty($identityData) && $identityData['compression'] == 'JPEG') {
            if ($identityData['type'] == 'Palette' && $identityData['colorspace'] == 'sRGB') {
                $param .= ' -set colorspace RGB -colorspace sRGB';
            } elseif ($identityData['type'] == 'Grayscale' && $identityData['colorspace'] == 'Gray') {
                $param .= ' -set colorspace Gray -colorspace sRGB';
            }
        }

        $target = tempnam(temp_path(), 'ingest_') . '.png';
        $command = str_replace(
            ['SOURCE', 'PARAM', 'ADDITIONAL', 'TARGET'],
            [escapeshellarg($master->getUri() . '[0]'), $param, '', escapeshellarg($target)],
            config('ingest.convert')
        );

        exec($command);

        if (!file_exists($target)) {
            return null;
        }

        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return static::processResource($target, new RoleType($roleType));
    }
}