<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service\Ingest;

use Rc\Domain\Model\Media;
use Rc\Domain\Model\Media\Resource as MediaResource;
use Rc\Domain\Model\Media\Resource\Type\RoleType;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Repository\MediaRepository;

class IngestService
{

    const INGEST_TYPE_COPY = 'copy';
    const INGEST_TYPE_MOVE = 'move';
    const INGEST_TYPE_LINK = 'link';

    /**
     * @var array
     */
    protected static $serviceMap = [
        'pdf'   => PdfIngestService::class,
        'image' => ImageIngestService::class,
        'video' => VideoIngestService::class,
        'audio' => AudioIngestService::class,
    ];

    /**
     * @var string
     */
    protected static $defaultIngestType = self::INGEST_TYPE_COPY;

    /**
     * @var array
     */
    protected static $mediaCache = [];

    /**
     * @var array
     */
    protected static $checkSumCache = [];

    /**
     * @return string
     */
    public static function getDefaultIngestType()
    {
        return self::$defaultIngestType;
    }

    /**
     * @param string $defaultIngestType
     */
    public static function setDefaultIngestType($defaultIngestType)
    {
        self::$defaultIngestType = $defaultIngestType;
    }

    /**
     * @param string $file
     * @return bool
     */
    public static function isArchive($file)
    {
        return MediaTypes::getContentTypeFromFilename($file) == 'zip';
    }

    /**
     * @param string $file
     * @param bool $skipDerivativeProcessing
     * @return MediaInterface[]
     */
    public static function mastersFromArchive($file, $skipDerivativeProcessing = false)
    {
        $extractPath = temp_path(md5(uniqid($file)));
        shell_exec('unzip "' . $file . '" -d "' . $extractPath . '"');

        $mediaList = [];
        foreach (glob($extractPath . '/*') as $extractedFile) {
            $mediaList[] = self::masterFromFile($extractedFile, $skipDerivativeProcessing);
        }
        return $mediaList;
    }

    /**
     * @param string $file
     * @param bool $skipDerivativeProcessing
     * @param bool $replace
     * @return MediaInterface
     */
    public static function masterFromFile($file, $skipDerivativeProcessing = false, $replace = false)
    {
        if (isset(static::$mediaCache[$file])) {
            return static::$mediaCache[$file];
        }

        $media = MediaRepository::findByMasterCheckSum(static::getCheckSum($file)) ?: new Media();
        $ingestService = static::factory($file);

        if (!$media->hasMaster() || $replace) {
            $master = $ingestService::processResource($file, new RoleType(RoleType::MASTER));
            $media->setMaster($master);
        }

        if (!$media->hasSplashScreen() || $replace) {
            $splashScreen = $ingestService::processSplashScreen($media->getMaster());
            $media->setSplashScreen($splashScreen);
        }

        if (!$skipDerivativeProcessing && (!$media->hasDerivatives() || $replace)) {
            $derivativeList = $ingestService::processDerivativeList($media->getMaster());
            $media->setDerivativeList($derivativeList);
        }

        MediaRepository::persist($media);

        static::$mediaCache[$file] = $media;
        return $media;
    }

    /**
     * @param string $file
     * @param MediaInterface $media
     * @return MediaInterface
     */
    public static function derivativeFromFile($file, MediaInterface $media)
    {
        if (isset(static::$mediaCache[$file])) {
            return static::$mediaCache[$file];
        }

        if (empty($file) || !file_exists($file)) {
            return $media;
        }

        $ingestService = static::factory($file);
        $derivative = $ingestService::processResource($file, new RoleType(RoleType::DERIVATIVE));

        $media->addDerivative($derivative);

        static::$mediaCache[$file] = $media;
        return $media;
    }

    /**
     * @param string $file
     * @param RoleType $roleType
     * @return MediaResource
     */
    public static function processResource($file, RoleType $roleType)
    {
        $master = static::getResourcePrototype();
        $master->setRole($roleType);
        $master->setCheckSum(static::getCheckSum($file));
        $master->setContentType(MediaTypes::getContentTypeFromFilename($file));
        $master->setMimeType(MediaTypes::getMediaTypeFromFilename($file));
        $master->setPathSuffix(config('media.storage_path_suffix') . '/' . $roleType . '/');
        $master->setFileSize(filesize($file));
        $master->setFileName($master->getCheckSum() . '.' . $master->getFileExtension());

        static::addFileToStorageBase($master, $file);
        static::enrichResource($master);

        return $master;
    }

    /**
     * @param MediaResource $master
     * @return MediaResource
     */
    public static function processSplashScreen($master)
    {
        return null;
    }

    /**
     * @param MediaResource $master
     * @return MediaResource[]
     */
    public static function processDerivativeList($master)
    {
        return [];
    }

    /**
     * Returns for a given file the appropriate *IngestService
     *
     * @param string $file
     * @return $this
     */
    public static function factory($file)
    {
        $contentType = MediaTypes::getContentTypeFromFilename($file);
        if (isset(static::$serviceMap[$contentType])) {
            return static::$serviceMap[$contentType];
        } else {
            return new static();
        }
    }

    /**
     * @return MediaResource
     */
    public static function getResourcePrototype()
    {
        return new MediaResource();
    }

    /**
     * @param string $file
     * @return string
     */
    public static function getCheckSum($file)
    {
        $fileHash = md5($file);

        if (!isset(static::$checkSumCache[$fileHash])) {
            static::$checkSumCache[$fileHash] = md5_file($file);
        }

        return static::$checkSumCache[$fileHash];
    }

    /**
     * @param MediaResource $master
     * @param string $file
     */
    protected static function addFileToStorageBase(MediaResource $master, $file)
    {
        $newFile = $master->getUri();
        if (file_exists($newFile)) {
            return;
        }

        self::createDirectory(dirname($newFile), 0777);

        $type = null;
        if (strpos($file, config('media.storage_path') . '/') !== false || strpos($file, storage_path('zencoder/master')) !== false) {
            $type = 'symlink';
            $retVal = symlink($file, $newFile);
        } elseif (strpos($file, temp_path()) !== false) {
            $type = 'move';
            $retVal = rename($file, $newFile);
        } elseif (strpos($file, ini_get('upload_tmp_dir') . '/') !== false) {
            $type = 'move';
            $retVal = rename($file, $newFile);
        } else { // use default ingest type
            if (self::$defaultIngestType == self::INGEST_TYPE_MOVE) {
                $type = 'move';
//                $retVal = rename($file, $newFile);
            } elseif (self::$defaultIngestType == self::INGEST_TYPE_LINK) {
                $type = 'symlink';
                $retVal = symlink($file, $newFile);
            } else {
                $type = 'copy';
                $retVal = copy($file, $newFile);
            }
        }

        if (empty($retVal)) {
            throw new \RuntimeException('Could not ingest file! Permissions denied for ' . $type . ' ' . $file . ' ' . $newFile);
        }
    }

    /**
     * @param MediaResource $resource
     */
    protected static function enrichResource($resource)
    {
    }

    /**
     * @param string $directory
     * @param int $umask
     */
    protected static function createDirectory($directory, $umask = 0777)
    {
        if (file_exists($directory)) {
            return;
        }

        if (!mkdir($directory, $umask, true)) {
            throw new \RuntimeException('Could not create directory ' . $directory . '!');
        }

        if (!chmod($directory, $umask)) {
            throw new \RuntimeException('Could not set permissions for directory ' . $directory . '!');
        }
    }
}