<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service;

use DateTime;
use Exception;
use FilesystemIterator;
use Rc\Domain\Repository\LogRepository;
use Rc\Domain\Repository\MediaRepository;
use Rc\Service\Ingest\IngestService;
use Rc\Service\Ingest\MediaTypes;
use Services_Zencoder;
use Services_Zencoder_Exception;
use Services_Zencoder_Job;
use SplFileInfo;

/**
 * @category  SAR.RC
 */
class ZencoderService
{

    /**
     * @var array []
     */
    protected static $jobTemplates = [
        'audio' => [
            'region'               => 'europe',
            'download_connections' => 5,
            'private'              => 1,
            'input'                => null,
            'outputs'              => [
                'mp3' => [
                    'label'         => 'mp3',
                    'speed'         => 3,
                    'audio_codec'   => 'mp3',
                    'audio_bitrate' => '256',
                    'hint'          => true,
                ],
            ],
        ],
        'video' => [
            'region'               => 'europe',
            'download_connections' => 5,
            'private'              => 1,
            'input'                => null,
            'outputs'              => [
                'mp4' => [
                    'label'         => 'mp4',
                    'video_codec'   => 'h264',
                    'quality'       => 3,
                    'speed'         => 3,
                    'format'        => 'mp4',
                    'width'         => 1024,
                    'height'        => 768,
                    'aspect_mode'   => 'preserve',
                    'audio_codec'   => 'aac',
                    'audio_quality' => 3,
                    'deinterlace'   => 'detect',
                    'hint'          => true,
                ],
            ],
        ],
    ];

    /**
     * Transcode all media files which are in the configured incoming directory
     *
     * @return void
     */
    public static function transcodeAll()
    {
        /** @var SplFileInfo $file */
        foreach (new FilesystemIterator(self::directory('input')) as $file) {
            self::transcode($file);
        }
    }

    /**
     * Transcode supplied media file
     *
     * @param SplFileInfo $file
     * @return void
     */
    public static function transcode(SplFileInfo $file)
    {
        if (strlen(self::getIdentifier($file)) != 32) {
            self::addFileToErrorDirectory($file);

            echo 'unable to transcode dilps media file "' . $file->getFilename() . '"!', PHP_EOL;
            return;
        }

        $contentType = MediaTypes::getContentTypeFromFilename($file->getFilename());
        if (!isset(self::$jobTemplates[$contentType])) {
            self::addFileToErrorDirectory($file);

            echo 'unable to transcode file "' . $file->getFilename() . '" with content type "' . $contentType . '"!', PHP_EOL;
            return;
        }

        $job = self::createJob($file, self::$jobTemplates[$contentType]);
        if (!isset($job)) {
            return;
        }

        self::submitJob($file, $job);
    }

    /**
     * Ingest supplied media file
     *
     * @param SplFileInfo $file
     */
    public static function ingest(SplFileInfo $file)
    {
        $identifier = self::getIdentifier($file);
        if (strlen($identifier) != 32) {
            self::addFileToErrorDirectory($file);

            echo 'unable to ingest dilps media file "' . $file->getFilename() . '"!', PHP_EOL;
            return;
        }

        $media = MediaRepository::findByMasterCheckSum($identifier);
        if (!isset($media)) {
            self::addFileToErrorDirectory($file);

            echo 'unable to ingest derivative "' . $file->getFilename() . '", because of missing media parent!', PHP_EOL;
            return;
        }

        $exception = null;

        try {
            IngestService::derivativeFromFile($file->getPathname(), $media);
            MediaRepository::persist($media);
            MediaRepository::flush();

            self::addFileToOutputDirectory($file);
        } catch (Exception $exception) {

        } finally {
            self::moveFileFromProcessToMasterDirectory($file);

            self::logJobIngest($file, $exception);
        }
    }

    /**
     * @return void
     */
    public static function ingestAllRemote()
    {
        // @todo: fetch entries from LogRepository
    }

    /**
     * @param string $master
     * @param string $format
     * @param string $url
     * @return void
     */
    public static function ingestRemote($master, $format, $url)
    {
        $file = self::fetchRemoteFile($url, $master . '.' . $format);

        self::ingest($file);
    }

    /**
     * @param string $master
     * @return void
     */
    public static function failRemote($master)
    {
        $directory = self::directory('process');

        $file = self::findFileByMaster($directory, $master);

        self::moveFileFromProcessToErrorDirectory($file);
    }

    /**
     * @param SplFileInfo $file
     * @param array $jobRequest
     * @return void
     */
    protected static function submitJob(SplFileInfo $file, array $jobRequest)
    {
        $exception = null;
        $jobResponse = null;

        try {
            $service = new Services_Zencoder(config('services.zencoder.api_key'));
            $jobResponse = $service->jobs->create(json_encode($jobRequest, JSON_UNESCAPED_SLASHES));

            self::addFileToProcessDirectory($file);
        } catch (Exception $exception) {
            self::addFileToErrorDirectory($file);
        } finally {
            self::logJobSubmit($file, $jobRequest, $jobResponse, $exception);
        }
    }

    /**
     * @param SplFileInfo $file
     * @param array $jobTemplate
     * @return array
     */
    protected static function createJob(SplFileInfo $file, array $jobTemplate)
    {
        $identifier = self::getIdentifier($file);
        $media = MediaRepository::findByMasterCheckSum($identifier);
        if (!isset($media)) {
            echo 'unable to find media for "' . $file->getFilename() . '"!', PHP_EOL;
            return null;
        }

        $jobTemplate['input'] = $media->getMaster()->getExternalUri();

        foreach ($jobTemplate['outputs'] as $extension => &$output) {
            $notifications = [['url' => self::getNotificationUrl($file, $extension)]];
            if (count($jobTemplate['outputs']) == 1) {
                $jobTemplate['notifications'] = $notifications;
            } else {
                $output['notifications'] = $notifications;
            }
        }

        return $jobTemplate;
    }

    /**
     * @param SplFileInfo $file
     * @param string $format
     * @return string
     */
    protected static function getNotificationUrl(SplFileInfo $file, $format)
    {
        $notificationUrlTemplate = self::config('notification_url');

        return str_replace(['{master}', '{format}'], [self::getIdentifier($file), $format], $notificationUrlTemplate);
    }

    /**
     * @param SplFileInfo $file
     */
    protected static function addFileToProcessDirectory(SplFileInfo $file)
    {
        $processDir = self::directory('process');
        $targetFile = $processDir . $file->getFilename();

        rename($file->getPathname(), $targetFile);
    }

    /**
     * @param SplFileInfo $file
     */
    protected static function addFileToErrorDirectory(SplFileInfo $file)
    {
        $errorDir = self::directory('error');

        rename($file->getPathname(), $errorDir . $file->getFilename());
    }

    /**
     * @param SplFileInfo $file
     */
    protected static function addFileToOutputDirectory(SplFileInfo $file)
    {
        $outputDir = self::directory('output');

        rename($file->getPathname(), $outputDir . $file->getFilename());
    }

    /**
     * @param SplFileInfo $file
     */
    protected static function moveFileFromProcessToMasterDirectory(SplFileInfo $file)
    {
        $processDir = self::directory('process');
        $masterDir = self::directory('master');

        $masterFile = self::findFileByMaster($processDir, self::getIdentifier($file));
        if ($masterFile) {
            rename($masterFile, $masterDir . $file->getFilename());
        }
    }

    /**
     * @param SplFileInfo $file
     */
    protected static function moveFileFromProcessToErrorDirectory(SplFileInfo $file)
    {
        $processDir = self::directory('process');
        $errorDir = self::directory('error');

        $masterFile = self::findFileByMaster($processDir, self::getIdentifier($file));
        if ($masterFile) {
            rename($masterFile, $errorDir . $file->getFilename());
        }
    }

    /**
     * @param string $directory
     * @param string $master
     * @return SplFileInfo
     */
    protected static function findFileByMaster($directory, $master)
    {
        return new SplFileInfo(glob($directory . $master . '*')[0]);
    }

    /**
     * @param SplFileInfo $file
     * @param array $jobRequest
     * @param Services_Zencoder_Job $jobResponse
     * @param Services_Zencoder_Exception $exception
     */
    protected static function logJobSubmit(
        SplFileInfo $file,
        array $jobRequest,
        Services_Zencoder_Job $jobResponse = null,
        Services_Zencoder_Exception $exception = null
    )
    {
        $logEntry = [
            'id'            => null,
            'master'        => self::getIdentifier($file),
            'media_type'    => MediaTypes::getMediaTypeFromFilename($file),
            'formats'       => array_keys($jobRequest['outputs']),
            'status'        => 'transcoding',
            'started'       => new DateTime(),
            'ended'         => null,
            'error'         => null,
            'error_details' => [],
            'request'       => $jobRequest,
            'response'      => null,
        ];

        if (isset($exception)) {
            $logEntryExtra = [
                'status'        => 'failed',
                'ended'         => new DateTime(),
                'error'         => $exception->getMessage() . ' #' . $exception->getCode(),
                'error_details' => $exception instanceof Services_Zencoder_Exception ? $exception->getErrors() : [],
            ];
        } else {
            $logEntryExtra = [
                'id'       => $jobResponse->id,
                'response' => get_object_vars($jobResponse),
            ];
        }

        LogRepository::add('Zencoder', array_merge($logEntry, $logEntryExtra));
    }

    /**
     * @param SplFileInfo $file
     * @param Exception $exception
     */
    protected static function logJobIngest(SplFileInfo $file, Exception $exception = null)
    {
        if (isset($exception)) {
            $logEntry = [
                'master' => self::getIdentifier($file),
                'status' => 'failed',
                'ended'  => new DateTime(),
                'error'  => $exception->getMessage() . ' #' . $exception->getCode(),
            ];
        } else {
            $logEntry = [
                'master' => self::getIdentifier($file),
                'status' => 'success',
                'ended'  => new DateTime(),
            ];
        }

        LogRepository::update('Zencoder', [
            'master' => self::getIdentifier($file),
        ], $logEntry);
    }

    /**
     * @param SplFileInfo $file
     * @return string
     */
    protected static function getIdentifier(SplFileInfo $file)
    {
        return $file->getBasename('.' . $file->getExtension());
    }

    /**
     * @param string $url
     * @param string $filename
     * @return SplFileInfo
     */
    protected static function fetchRemoteFile($url, $filename)
    {
        $file = new SplFileInfo(self::directory('finished') . $filename);

        $fp = fopen($file->getPathname(), 'w+');

        set_time_limit(0);
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_FILE, $fp);
        curl_exec($ch);
        curl_close($ch);

        fclose($fp);

        return $file;
    }

    /**
     * @param string $key
     * @return mixed
     */
    protected static function config($key)
    {
        return config('services.zencoder.' . $key);
    }

    /**
     * @param string $path
     * @return mixed
     */
    protected static function directory($path)
    {
        $directory = static::config('directories.' . $path) . '/';

        if (!is_dir($directory)) {
            mkdir($directory, 0777, true);
        }

        return $directory;
    }
}
