<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

if (!function_exists('env')) {
    /**
     * Gets the value of an environment variable.
     *
     * @param string $key
     * @param mixed $default
     * @return mixed
     */
    function env(string $key, $default = null)
    {
        $value = getenv($key);

        if ($value === false) {
            return value($default);
        }

        switch (strtolower($value)) {
            case 'true':
            case '(true)':
                return true;
            case 'false':
            case '(false)':
                return false;
            case 'empty':
            case '(empty)':
                return '';
            case 'null':
            case '(null)':
                return null;
        }

        if (($valueLength = strlen($value)) > 1 && $value[0] === '"' && $value[$valueLength - 1] === '"') {
            return substr($value, 1, -1);
        }

        return $value;
    }
}

if (!function_exists('app_env')) {
    /**
     * Get or check the current application environment.
     *
     * @param string[] $env
     * @return mixed
     */
    function app_env(string ...$env)
    {
        $currentEnv = config('app.env');

        if (empty($env)) {
            return $currentEnv;
        }

        enforce_valid_env(...$env);

        return in_array($currentEnv, $env);
    }
}

if (!function_exists('enforce_valid_env')) {
    /**
     * Check the environment and if not valid throws exception
     *
     * @param string[] $env
     */
    function enforce_valid_env(string ...$env)
    {
        $supportedEnvs = config('app.supported_envs');

        $diff = array_diff($env, $supportedEnvs);
        if (!empty($diff)) {
            throw new LogicException('Environment "' . implode(', ', $diff) . '" is not supported. Please choose one of the list: ' . implode(', ', $supportedEnvs));
        }
    }
}

if (!function_exists('value')) {
    /**
     * Return the default value of the given value.
     *
     * @param mixed $value
     * @return mixed
     */
    function value($value)
    {
        return $value instanceof Closure ? $value() : $value;
    }
}

if (!function_exists('simplify_path')) {
    /**
     * Get the path to the base of the install.
     *
     * @param string $path
     * @return string
     */
    function simplify_path($path)
    {
        return realpath($path) ?: $path;
    }
}

if (!function_exists('base_path')) {
    /**
     * Get the path to the base of the install.
     *
     * @param string $path
     * @return string
     */
    function base_path($path = '')
    {
        return simplify_path(APPLICATION_PATH . '/..' . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('storage_path')) {
    /**
     * Get the path to the storage folder.
     *
     * @param  string $path
     * @return string
     */
    function storage_path($path = '')
    {
        return simplify_path(base_path('storage') . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('app_path')) {
    /**
     * Get the path to the application folder.
     *
     * @param  string $path
     * @return string
     */
    function app_path($path = '')
    {
        return simplify_path(base_path('application') . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('log_path')) {
    /**
     * Get the path to the log folder.
     *
     * @param  string $path
     * @return string
     */
    function log_path($path = '')
    {
        return simplify_path(storage_path('logs') . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('cache_path')) {
    /**
     * Get the path to the cache folder.
     *
     * @param  string $path
     * @return string
     */
    function cache_path($path = '')
    {
        return simplify_path(storage_path('framework/cache') . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('temp_path')) {
    /**
     * Get the path to the temp folder.
     *
     * @param  string $path
     * @return string
     */
    function temp_path($path = '')
    {
        return simplify_path(storage_path('framework/temporary') . ($path ? DIRECTORY_SEPARATOR . $path : $path));
    }
}

if (!function_exists('config')) {
    /**
     * Get / set the specified configuration value.
     * If an array is passed as the key, we will assume you want to set an array of values.
     *
     * @param  string $key
     * @param  mixed $default
     * @return mixed
     */
    function config($key = null, $default = null)
    {
        $config = Zend_Registry::get('Config');

        if (is_null($key)) {
            return $config;
        }

        if (strpos($key, '.') === false) {
            return $config[$key] ?? value($default);
        }

        foreach (explode('.', $key) as $segment) {
            if (is_array($config) && array_key_exists($segment, $config)) {
                $config = $config[$segment];
            } else {
                return value($default);
            }
        }

        return $config;
    }
}
if (!function_exists('sendfile')) {
    function sendfile(string $file): string
    {
        if (!defined('APPLICATION_PATH')) {
            $path = $file;
        } else if (strpos($file, storage_path()) !== false) {
            $path = '/storage/' . str_replace(storage_path() . '/', '', $file);
        } else if (strpos($file, 'media.researchcatalogue.net') !== false) {
            $path = '/media/' . str_replace('/home/www/vhosts/media.researchcatalogue.net/', '', $file);
        } else {
            $path = $file;
        }

        return '/accel-redirect' . $path;
    }
}