<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Utility;

use Doctrine\Common\Util\ClassUtils;
use Identity;
use JMS\Serializer\DeserializationContext;
use Rc\Domain\Model\Application\ApplicationSnapshot;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Serializer\Serializer;

class DuplicateFacade
{

    /**
     * @var Serializer
     */
    protected $serializer;

    public function __construct()
    {
        $this->serializer = new Serializer();
    }

    /**
     * @param Research    $research
     * @param string|null $data
     * @param array       $options
     * @return Research
     */
    public function duplicate(Research $research, $data = null, array $options = [])
    {
        $context = new DeserializationContext();
        $context->setAttribute('duplicate', true);
        $context->setAttribute('owner', array_key_exists('owner', $options) ? $options['owner'] : Identity::getUserOrNull());

        /** @var Research $newResearch */
        $newResearch = $this->serializer->deserialize(
            $data ?: $this->serializer->serialize($research),
            ClassUtils::getClass($research),
            $context
        );

        $newResearch->setSource($research);
        $newResearch->setRevision(new \DateTime());

        $newResearch->setRelations([]);
        $newResearch->setPartOfPortalId(null);
        $newResearch->setPartOfPortalRequestId(null);
        $newResearch->setAccessLevelPortalsIds([]);

        $this->applyOptions($newResearch, $research, $options );

        return $newResearch;
    }

    /**
     * @param Research $research
     * @param string   $data
     * @param array    $options
     * @return Research
     */
    public function restore(Research $research, $data, array $options = [])
    {
        $newResearch = $this->serializer->deserialize(
            $data,
            ClassUtils::getClass($research)
        );

        $this->applyOptions($newResearch, $research, $options);

        return $newResearch;
    }

    /**
     * @param Exposition  $exposition
     * @param string|null $data
     * @param array       $options
     * @return Exposition
     */
    public function duplicateExposition(Exposition $exposition, $data = null, array $options = [])
    {
        /** @var Exposition $newExposition */
        $newExposition = $this->duplicate($exposition, $data, $options);

        return $newExposition;
    }

    /**
     * @param Exposition  $exposition
     * @param string|null $data
     * @param array       $options
     * @return Exposition
     */
    public function restoreExposition(Exposition $exposition, $data = null, array $options = [])
    {
        /** @var Exposition $newExposition */
        $newExposition = $this->restore($exposition, $data, $options);

        return $newExposition;
    }

    /**
     * @param Application $application
     * @return ApplicationSnapshot
     */
    public function applicationSnapshot(Application $application)
    {
        $snapshot = new ApplicationSnapshot();

        foreach ($application->getResearches() as $research) {
            $snapshot->addResearch(
                $this->duplicate($research, null, [
                    'owner' => null,
                ])
            );
        }

        return $snapshot;
    }

    /**
     * @param Research $new
     * @param Research $source
     * @param array    $options
     */
    protected function applyOptions(Research $new, Research $source, array $options)
    {
        $new->setAccessLevel(isset($options['access-level']) ? $options['access-level'] : Exposition::ACCESS_LEVEL_PRIVATE);

        if ($new instanceof StatusAwareInterface) {
            $new->setStatus(isset($options['status']) ? $options['status'] : StatusAwareInterface::STATUS_PROGRESS);
        }

        $this->applyCollaborationOptions($new, $source, $options);
    }

    /**
     * @param Research $new
     * @param Research $current
     * @param array    $options
     */
    protected function applyCollaborationOptions(Research $new, Research $current, array $options)
    {
        if (empty($options['collaboration'])) {
            return;
        }

        switch ($options['collaboration']) {
            case 'private':
                $new->setContributors([]);
                $new->setCoAuthors([]);
                $new->setSupervisors([]);
                $new->setEditors([]);
                return;

            case 'keep_all':
                $new->setContributors($current->getContributors());
                $new->setCoAuthors($current->getCoAuthors());
                $new->setSupervisors($current->getSupervisors());
                $new->setEditors($current->getEditors());

                $new->setPendingInvites($current->getPendingInvites());
                $new->setDisapprovedCollaborators($current->getDisapprovedCollaborators());
                return;

            case 'keep_active':
                $new->setContributors($current->getContributors());
                $new->setCoAuthors($current->getCoAuthors());
                $new->setSupervisors($current->getSupervisors());
                $new->setEditors($current->getEditors());

                $new->removePendingContributors();
                $new->setDisapprovedCollaborators([]);
                return;

            case 'restore':
                $new->removePendingContributors();
                $new->setDisapprovedCollaborators([]);
                return;
        }
    }

}
