<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Utility;

use CommentRepository;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Project;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\DegreeRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\ProjectRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\ResearchRepository;
use ReportRepository;

class ResearchUtility
{
    /**
     * @param Research $research
     */
    public static function removeResearch(Research $research)
    {
        if ($research instanceof Exposition) {
            self::removeExposition($research);
        } else if ($research instanceof Work) {
            self::removeWork($research);
        } else if ($research instanceof Project) {
            self::removeProject($research);
        } else if ($research instanceof Degree) {
            self::removeDegree($research);
        } elseif ($research instanceof Application) {
            self::removeApplication($research);
        } else {
            throw new \InvalidArgumentException('Cannot remove research! Unsupported type ' . get_class($research) . '.');
        }
    }

    /**
     * @param Exposition $exposition
     */
    public static function removeExposition(Exposition $exposition)
    {
        ReportRepository::removeByResearch($exposition);
        CommentRepository::removeByResearch($exposition);
        ExpositionRepository::remove($exposition);
    }

    /**
     * @param Work $work
     */
    public static function removeWork(Work $work)
    {
        WorkRepository::remove($work);
    }

    /**
     * @param Project $project
     */
    public static function removeProject(Project $project)
    {
        ProjectRepository::remove($project);
    }

    /**
     * @param Degree $degree
     */
    public static function removeDegree(Degree $degree)
    {
        DegreeRepository::remove($degree);
    }

    /**
     * @param Application $application
     */
    public static function removeApplication(Application $application)
    {
        foreach ($application->getSnapshots() as $snapshot) {
            foreach ($snapshot->getExpositions() as $exposition) {
                static::removeExposition($exposition);
            }
            foreach ($snapshot->getWorks() as $work) {
                static::removeWork($work);
            }
        }

        ApplicationRepository::remove($application);
    }

    /**
     * @param User $user
     * @return int
     */
    public static function removeResearchesByUser(User $user)
    {
        /** @var Work[] $works */
        $works = [];
        $preservedResearchCount = 0;

        foreach (ResearchRepository::findByPerson($user) as $research) {
            $isPublished = $research instanceof StatusAwareInterface && $research->isPublished();

            // remove contribution
            if (!$research->isOwner($user)) {
                $research->removeSupervisor($user);
                $research->removeContributor($user);

                if (!$isPublished) {
                    $research->removeCoAuthor($user);
                }

                ResearchRepository::persist($research);
                continue;
            }

            // don't touch published researches
            if ($isPublished) {
                ++$preservedResearchCount;
                continue;
            }

            // delay removal of works to prevent issues with shared works
            if ($research instanceof Work) {
                $works[] = $research;
                continue;
            }

            self::removeResearch($research);
        }

        foreach ($works as $work) {
            if (empty(ExpositionRepository::findBySharedWork($work))) {
                self::removeWork($work);
            } else {
                ++$preservedResearchCount;
                $work->setAccessLevel(Work::ACCESS_LEVEL_PRIVATE);
                WorkRepository::persist($work);
            }
        }

        return $preservedResearchCount;
    }

    /**
     * @param Exposition $exposition
     * @param User|null $user
     * @return array
     */
    public static function expositionMetricTags(Exposition $exposition, User $user = null)
    {
        return [
                'uniquevisitor' => $_COOKIE['uniquevisitor'],
                'user'          => isset($user) ? $user->getId() : null,
                'country'       => GeoIp::country($_SERVER['REMOTE_ADDR']),
                'uuid'          => $exposition->getUuid(),
                'id'            => $exposition->getId(),
                'title'         => (string)$exposition->getTitle(),
                'issue'         => $exposition->getPortalEdition(),
                'portal'        => $exposition->getPortal() ? $exposition->getPortal()->getShortName() : null,
                'status'        => $exposition->getStatus(),
            ] + $exposition->getRoleNames($user);
    }
}